<?php

/**
 * Custom fields and filters of post
 *
 * @package CustomFieldFilterPost
 * @version 2.0.2
 * @author davidhs
 * @copyright 2014-2020 by davidhs
 * @license Creative Commons Attribution 3.0 Unported License https://creativecommons.org/licenses/by/3.0/
 */

if (!defined('SMF'))
	die('Hacking attempt...');

// Gets ID column free on group and creates column if it is not exists.
function cffpGetIdColumn($id_group)
{
	global $db_prefix;

	if (empty($id_group))
		return false;

	// Get id_column.
	$request = db_query("
		SELECT id_column
		FROM {$db_prefix}cffp_settings_fields
		WHERE ID_GROUP = $id_group
		ORDER BY id_column ASC", __FILE__, __LINE__);
	$id_column = 1;
	while ($row = mysql_fetch_assoc($request))
	{
		if ($id_column < $row['id_column'])
			break; // Exit while.
		else
			$id_column++;
	}
	mysql_free_result($request);

	// Create column if it is not exists.
	cffpAddFieldsColumn($id_column);

	return $id_column;
}

// Gets fields of group with column in moderation list.
function cffpGetModerationListFields($id_group)
{
	global $db_prefix;

	if (empty($id_group))
		return false;

	$request = db_query("
		SELECT name, id_column, show_moderation_list_type, show_moderation_list_width, show_moderation_list_name
		FROM {$db_prefix}cffp_settings_fields
		WHERE ID_GROUP = $id_group
			AND show_moderation_list_type <> 'no'
			AND active = 1
		ORDER BY field_order ASC", __FILE__, __LINE__);
	$list = array();
	$multi = array();
	while ($row = mysql_fetch_assoc($request))
	{
		if ('one' == $row['show_moderation_list_type'])
			$list[] = $row;
		else // multi.
		{
			$column = (!empty($row['show_moderation_list_width']) ? $row['show_moderation_list_width'] : 'auto') . ',' . $row['show_moderation_list_name'];
			if (!in_array($column, $multi))
			{
				$multi[] = $column;
				$list[] = $row;
			}
		}
	}
	mysql_free_result($request);

	return $list;
}

// Gets fields of record with column in search result.
function cffpGetRecordListFields($id_group)
{
	global $db_prefix;

	if (empty($id_group))
		return false;

	$request = db_query("
		SELECT name, id_column, show_record_list_type, show_record_list_width, show_record_list_name
		FROM {$db_prefix}cffp_settings_fields
		WHERE ID_GROUP = $id_group
			AND show_record_list_type <> 'no'
			AND active = 1
		ORDER BY field_order ASC", __FILE__, __LINE__);
	$list = array();
	$multi = array();
	while ($row = mysql_fetch_assoc($request))
	{
		if ('one' == $row['show_record_list_type'])
			$list[] = $row;
		else // multi.
		{
			$column = (!empty($row['show_record_list_width']) ? $row['show_record_list_width'] : 'auto') . ',' . $row['show_record_list_name'];
			if (!in_array($column, $multi))
			{
				$multi[] = $column;
				$list[] = $row;
			}
		}
	}
	mysql_free_result($request);

	return $list;
}

// Gets list of settings fields of a group.
function cffpGetListSettingsFields($id_group, $only_active)
{
	global $db_prefix;

	if (empty($id_group))
		return false;

	$request = db_query("
		SELECT
			ID_FIELD, name, field_desc, description, field_order,
			type, length, options, options_text, mask, bbc,
			show_message_index,
			not_empty_value, unique_value, subset, active
		FROM {$db_prefix}cffp_settings_fields
		WHERE ID_GROUP = $id_group" . (!$only_active ? '' : "
			AND active = 1") . "
		ORDER BY field_order ASC", __FILE__, __LINE__);
	$list = array();
	while ($row = mysql_fetch_assoc($request))
		$list[] = $row;
	mysql_free_result($request);

	return $list;
}

// Gets more information of field.
function cffpGetMoreInfoField($field)
{
	global $txt, $func, $sourcedir;

	if (empty($field))
		return false;

	require_once ($sourcedir . '/Subs-CustomFieldFilterPost.php');

	$text = array();
	switch ($field['type'])
	{
		case 'text':
		case 'textarea':
		{
			if (!empty($field['length']))
				$text[] = $txt['cffp_length'] . ': ' . $field['length'];
			if ('text' == $field['type'])
				$text[] = $txt['cffp_mask'] . ': ' . $txt['cffp_mask_' . ('regex' == $func['substr']($field['mask'], 0, 5) ? 'regex' : $field['mask'])];
			if (!empty($field['bbc']))
				$text[] = $txt['cffp_allow_bbc'];
			break;
		}
		case 'select':
		case 'combobox':
		{
			$options = explode(',', $field['options']);
			$select_multiple = array_shift($options);
			array_shift($options);

			if (0 < strlen($field['options_text']))
				$options = explode(',', $field['options_text']);

			if ('combobox' == $field['type'] && !empty($field['length']))
				$text[] = $txt['cffp_length'] . ': ' . $field['length'];
			if ($select_multiple)
				$text[] = $txt['cffp_select_multiple'];
			if (!empty($options))
				$text[] = $txt['cffp_options'] . ': ' . implode(', ', $options);
			break;
		}
		case 'radio':
		case 'check':
		{
			$options = 0 < strlen($field['options_text']) ? $field['options_text'] : $field['options'];
			if (0 < strlen($options))
				$text[] = $txt['cffp_options'] . ': ' . str_replace(',', ', ', $options);
			break;
		}
	}
	if (!empty($field['not_empty_value']))
		$text[] = $txt['cffp_not_empty_value'];
	if (!empty($field['unique_value']))
		$text[] = $txt['cffp_unique_value'];
	if (cffpIsNotEmptyString($field['subset']))
		$text[] = $txt['cffp_subset'] . ': ' . $field['subset'];

	return empty($text) ? '' : implode('<br />', $text);
}

// Gets columns multi-field of moderation list
function cffpGetModerationColumnsMulti($id_group)
{
	global $db_prefix;

	if (empty($id_group))
		return false;

	$request = db_query("
		SELECT DISTINCT show_moderation_list_width, show_moderation_list_name
		FROM {$db_prefix}cffp_settings_fields
		WHERE ID_GROUP = $id_group
			AND show_moderation_list_type = 'multi'
		ORDER BY show_moderation_list_name ASC", __FILE__, __LINE__);
	$moderation_column_multi_array = array();
	while ($row = mysql_fetch_assoc($request))
		$moderation_column_multi_array[] = $row;
	mysql_free_result($request);

	return $moderation_column_multi_array;
}

// Gets columns multi-field of records list
function cffpGetRecordColumnsMulti($id_group)
{
	global $db_prefix;

	if (empty($id_group))
		return false;

	$request = db_query("
		SELECT DISTINCT show_record_list_width, show_record_list_name
		FROM {$db_prefix}cffp_settings_fields
		WHERE ID_GROUP = $id_group
			AND show_record_list_type = 'multi'
		ORDER BY show_record_list_name ASC", __FILE__, __LINE__);
	$record_column_multi_array = array();
	while ($row = mysql_fetch_assoc($request))
		$record_column_multi_array[] = $row;
	mysql_free_result($request);

	return $record_column_multi_array;
}

// Gets tags for hidden filters of a group of fields.
function cffpGetHideFilter($id_group)
{
	global $db_prefix;

	if (empty($id_group))
		return false;

	$request = db_query("
		SELECT DISTINCT hide_filter
		FROM {$db_prefix}cffp_settings_fields
		WHERE ID_GROUP = $id_group
			AND hide_filter <> ''
		ORDER BY hide_filter ASC", __FILE__, __LINE__);
	$hide_filter_array = array();
	while ($row = mysql_fetch_assoc($request))
		$hide_filter_array[] = $row['hide_filter'];
	mysql_free_result($request);

	return $hide_filter_array;
}

// Gets search pages of posts.
function cffpGetSearchPagePost()
{
	global $db_prefix;

	$search_page_array = array();

	// Get search pages with one or more than one values.
	$request = db_query("
		SELECT DISTINCT search_page_name, search_page_order_by
		FROM {$db_prefix}cffp_settings_groups
		WHERE type IN ('fields', 'member', 'filters')
			AND search_page_name <> ''", __FILE__, __LINE__);
	while ($row = mysql_fetch_assoc($request))
	{
		if (strpos($row['search_page_name'], ','))
		{
			$search_page_name_array = explode(',', $row['search_page_name']);
			$search_page_order_by_array = explode(',', $row['search_page_order_by']);
			foreach ($search_page_name_array as $k => $search_page_name)
				$search_page_array[] = $search_page_name . ',' . $search_page_order_by_array[$k];
		}
		else
			$search_page_array[] = $row['search_page_name'] . ',' . $row['search_page_order_by'];
	}
	mysql_free_result($request);

	// Remove duplicate search pages.
	$search_page_array = array_unique($search_page_array);
	sort($search_page_array);

	return $search_page_array;
}

// Gets subset of a group of fields.
function cffpGetSubset($id_group)
{
	global $db_prefix;

	if (empty($id_group))
		return false;

	$request = db_query("
		SELECT DISTINCT subset
		FROM {$db_prefix}cffp_settings_fields
		WHERE ID_GROUP = $id_group
			AND subset <> ''
		ORDER BY subset ASC", __FILE__, __LINE__);
	$subset_array = array();
	while ($row = mysql_fetch_assoc($request))
		$subset_array[] = $row['subset'];
	mysql_free_result($request);

	return $subset_array;
}

// Adds a fields column of group.
function cffpAddFieldsColumn($id_column)
{
	global $context, $sourcedir;

	if (empty($id_column))
		return false;

	require_once ($sourcedir . '/Subs-Db-CustomFieldFilterPost-mysql.php');
	require_once ($sourcedir . '/Themes-CustomFieldFilterPost.php');

	$column = array(
		'name' => 'column_' . $id_column,
		'type' => 'text',
		'default' => '',
	);
	$list_columns = $context['cffp']['list_columns']['fields'];

	// Create column if it is not exists.
	if (!in_array($column['name'], $list_columns))
	{
		cffp_db_add_column('cffp_fields', $column);

		// Update columns in $context.
		cffp_load_theme('list_columns_fields');
	}
}

// Removes a fields column of group.
function cffpRemoveFieldsColumn($id_group, $id_column)
{
	global $db_prefix, $sourcedir;

	if (empty($id_group) || empty($id_column))
		return false;

	require_once ($sourcedir . '/Subs-Db-CustomFieldFilterPost-mysql.php');
	require_once ($sourcedir . '/Themes-CustomFieldFilterPost.php');

	$request = db_query("
		SELECT id_column
		FROM {$db_prefix}cffp_settings_fields
		WHERE ID_GROUP <> $id_group
			AND id_column = $id_column", __FILE__, __LINE__);
	$num_rows = mysql_num_rows($request);
	mysql_free_result($request);

	if (0 == $num_rows)
	{
		cffp_db_remove_column('cffp_fields', 'column_' . $id_column);

		// Update columns in $context.
		cffp_load_theme('list_columns_fields');
	}

	return 0 == $num_rows;
}

// Adds counter record columns.
function cffpAddCounterRecordColumns($id_group)
{
	global $context, $sourcedir;

	if (empty($id_group))
		return false;

	require_once ($sourcedir . '/Subs-Db-CustomFieldFilterPost-mysql.php');
	require_once ($sourcedir . '/Themes-CustomFieldFilterPost.php');

	$columns = array(
		'groups' => array(
			'name' => 'cffp' . $id_group . '_groups',
			'type' => 'mediumint',
			'size' => 8,
			'default' => '0',
			'unsigned' => true,
		),
		'ID_MEMBERGROUP' => array(
			'name' => 'cffp' . $id_group . '_ID_MEMBERGROUP',
			'type' => 'int',
			'size' => 10,
			'default' => '0',
		),
	);
	$list_columns = $context['cffp']['list_columns']['members'];

	// Create columns if they are not exist.
	$update_columns = false;
	foreach ($columns as $column)
	{
		if (in_array($column['name'], $list_columns))
			break;

		$update_columns = true;
		cffp_db_add_column('members', $column);
	}

	// Update columns in $context.
	if ($update_columns)
		cffp_load_theme('list_columns_members');
}

// Removes counter record columns.
function cffpRemoveCounterRecordColumns($id_group)
{
	global $db_prefix, $sourcedir;

	if (empty($id_group))
		return false;

	require_once ($sourcedir . '/Subs-Db-CustomFieldFilterPost-mysql.php');
	require_once ($sourcedir . '/Themes-CustomFieldFilterPost.php');

	$request = db_query("
		SELECT ID_GROUP
		FROM {$db_prefix}cffp_settings_groups
		WHERE ID_GROUP = $id_group
			AND type = 'record'", __FILE__, __LINE__);
	$num_rows = mysql_num_rows($request);
	mysql_free_result($request);

	if (0 == $num_rows)
	{
		cffp_db_remove_column('members', 'cffp' . $id_group . '_groups');
		cffp_db_remove_column('members', 'cffp' . $id_group . '_ID_MEMBERGROUP');

		// Update columns in $context.
		cffp_load_theme('list_columns_members');

		// Delete membergroups.
		db_query("
			DELETE FROM {$db_prefix}cffp_membergroups
			WHERE ID_GROUP = $id_group", __FILE__, __LINE__);
	}

	return 0 == $num_rows;
}

// Admin area settings (edit field).
function cffpSettingsEditField()
{
	global $txt, $context, $func, $db_prefix, $sourcedir;

	require_once ($sourcedir . '/Subs-CustomFieldFilterPost.php');
	require_once ($sourcedir . '/Subs-Db-CustomFieldFilterPost-mysql.php');

	loadTemplate('CustomFieldFilterPost');

	// Sort out the context!
	$context['gid'] = isset($_GET['gid']) ? (int) $_GET['gid'] : 0;
	$context['fid'] = isset($_GET['fid']) ? (int) $_GET['fid'] : 0;
	$context['settings_title'] = $context['fid'] ? $txt['cffp_edit_field_title'] : $txt['cffp_add_field_title'];
	$start = isset($_GET['start']) ? (int) $_GET['start'] : 0;
	$sort = isset($_GET['sort']) ? $_GET['sort'] : 'group_order';
	$context['params_url'] = ';start=' . $start . ';sort=' . $sort . (isset($_GET['desc']) ? ';desc' : '');
	$context['admin_tabs']['tabs']['cffp']['is_selected'] = true;
	$context['sub_template'] = 'cffp_settings_edit_field';

	if ($context['gid'])
	{
		$type_filter_array = array(
			'text' => array(
				array('text', 'textarea'),
				array('text', 'empty_non_empty'),
			),
			'combobox' => array(
				array('select', 'combobox', 'radio', 'check'),
				array('text', 'select', 'combobox', 'empty_non_empty'),
			),
			'boolean' => array(
				array('boolean'),
				array('select'),
			),
		);

		if ($context['fid'])
		{
			$request = db_query("
				SELECT
					name, id_column,
					field_desc, field_long_desc, field_search_desc, field_search_long_desc,
					description, field_order,
					type, type_filter, length, options, options_text, mask, bbc, default_value,
					text_previous, text_next,
					enclose_type, enclose_text,
					show_message_index, show_filter_operator, hide_filter, show_filter_used_options,
					show_record_list_type, show_record_list_width, show_record_list_name,
					show_moderation_list_type, show_moderation_list_width, show_moderation_list_name,
					not_empty_value, unique_value, subset, permissions, active
				FROM {$db_prefix}cffp_settings_fields
				WHERE ID_FIELD = " . $context['fid'] . "
					AND ID_GROUP = " . $context['gid'], __FILE__, __LINE__);
			$row = mysql_fetch_assoc($request);
			mysql_free_result($request);

			if ($row)
			{
				if ('%' == substr($row['show_record_list_width'], -1))
				{
					$show_record_list_width_unit = '%';
					$show_record_list_width_value = substr($row['show_record_list_width'], 0, -1);
				}
				elseif ('px' == substr($row['show_record_list_width'], -2))
				{
					$show_record_list_width_unit = 'px';
					$show_record_list_width_value = substr($row['show_record_list_width'], 0, -2);
				}
				else // Empty.
				{
					$show_record_list_width_unit = 'auto';
					$show_record_list_width_value = 0;
				}
				if ('multi' == $row['show_record_list_type'])
					$show_record_list_column = (!empty($row['show_record_list_width']) ? $row['show_record_list_width'] : 'auto') . ',' . $row['show_record_list_name'];
				else
					$show_record_list_column = '';

				if ('%' == substr($row['show_moderation_list_width'], -1))
				{
					$show_moderation_list_width_unit = '%';
					$show_moderation_list_width_value = substr($row['show_moderation_list_width'], 0, -1);
				}
				elseif ('px' == substr($row['show_moderation_list_width'], -2))
				{
					$show_moderation_list_width_unit = 'px';
					$show_moderation_list_width_value = substr($row['show_moderation_list_width'], 0, -2);
				}
				else // Empty.
				{
					$show_moderation_list_width_unit = 'auto';
					$show_moderation_list_width_value = 0;
				}
				if ('multi' == $row['show_moderation_list_type'])
					$show_moderation_list_column = (!empty($row['show_moderation_list_width']) ? $row['show_moderation_list_width'] : 'auto') . ',' . $row['show_moderation_list_name'];
				else
					$show_moderation_list_column = '';

				if ('textarea' == $row['type'])
					list ($rows, $cols) = explode(',', $row['options']);
				else
				{
					$rows = 3;
					$cols = 30;
				}
				if ('select' == $row['type'] || 'combobox' == $row['type'])
				{
					$options = explode(',', $row['options']);
					$select_multiple = array_shift($options);
					$select_size = array_shift($options);
					$row['options'] = implode(',', $options);
				}
				else
				{
					$select_multiple = 0;
					$select_size = 1;
				}

				$multiple = (('select' == $row['type'] || 'combobox' == $row['type']) && $select_multiple) || 'check' == $row['type'];

				$context['params'] = array(
					'id_column' => $row['id_column'],
					'name' => $row['name'],
					'field_desc' => $row['field_desc'],
					'field_long_desc' => $row['field_long_desc'],
					'field_search_desc' => $row['field_search_desc'],
					'field_search_long_desc' => $row['field_search_long_desc'],
					'description' => $row['description'],
					'field_order' => $row['field_order'],
					'type' => $row['type'],
					'type_filter_text' => in_array($row['type'], $type_filter_array['text'][0]) ? $row['type_filter'] : $type_filter_array['text'][1][0],
					'type_filter_combobox' => in_array($row['type'], $type_filter_array['combobox'][0]) ? $row['type_filter'] : $type_filter_array['combobox'][1][1],
					'type_filter_boolean' => in_array($row['type'], $type_filter_array['boolean'][0]) ? $row['type_filter'] : $type_filter_array['boolean'][1][0],
					'length' => $row['length'],
					'rows' => $rows,
					'cols' => $cols,
					'select_multiple' => $select_multiple,
					'select_size' => $select_size,
					'options_old' => 0 < strlen($row['options']) ? explode(',', $row['options']) : array(),
					'options' => 0 < strlen($row['options']) ? explode(',', $row['options']) : array('', '', ''),
					'options_text' => 0 < strlen($row['options_text']) ? explode(',', $row['options_text']) : array('', '', ''),
					'mask' => $row['mask'],
					'regex' => 'regex' == $func['substr']($row['mask'], 0, 5) ? $func['substr']($row['mask'], 5) : '',
					'bbc' => $row['bbc'] ? true : false,
					'default_value_text' => 'text' == $row['type'] ? $row['default_value'] : '',
					'default_value_textarea' => 'textarea' == $row['type'] ? $row['default_value'] : '',
					'default_value_options' => !$multiple ? $row['default_value'] : '',
					'default_value_options_multiple' => $multiple && 0 < strlen($row['default_value']) ? explode(',', $row['default_value']) : array(),
					'default_value_boolean' => 'boolean' == $row['type'] ? $row['default_value'] : '',
					'text_previous' => $func['htmlspecialchars']($row['text_previous'], ENT_QUOTES),
					'text_next' => $func['htmlspecialchars']($row['text_next'], ENT_QUOTES),
					'enclose_type' => $row['enclose_type'],
					'enclose_text' => $func['htmlspecialchars']($row['enclose_text'], ENT_QUOTES),
					'show_message_index' => $row['show_message_index'],
					'show_filter_operator' => $row['show_filter_operator'],
					'hide_filter' => $row['hide_filter'],
					'show_filter_used_options' => $row['show_filter_used_options'],
					'show_record_list_type' => $row['show_record_list_type'],
					'show_record_list_width_unit' => $show_record_list_width_unit,
					'show_record_list_width_value' => $show_record_list_width_value,
					'show_record_list_column' => $show_record_list_column,
					'show_record_list_name' => '',
					'show_moderation_list_type' => $row['show_moderation_list_type'],
					'show_moderation_list_width_unit' => $show_moderation_list_width_unit,
					'show_moderation_list_width_value' => $show_moderation_list_width_value,
					'show_moderation_list_column' => $show_moderation_list_column,
					'show_moderation_list_name' => '',
					'not_empty_value' => $row['not_empty_value'],
					'unique_value' => $row['unique_value'],
					'subset' => $row['subset'],
					'permissions' => !empty($row['permissions']) ? explode(',', $row['permissions']) : array(),
					'active' => $row['active'],
				);
			}
		}

		// Setup the default values as needed.
		if (empty($context['params']))
		{
			$context['params'] = array(
				'id_column' => 0,
				'name' => '',
				'field_desc' => '',
				'field_long_desc' => '',
				'field_search_desc' => '',
				'field_search_long_desc' => '',
				'description' => '',
				'field_order' => 0,
				'type' => 'text',
				'type_filter_text' => $type_filter_array['text'][1][0],
				'type_filter_combobox' => $type_filter_array['combobox'][1][1],
				'type_filter_boolean' => $type_filter_array['boolean'][1][0],
				'length' => 255,
				'rows' => 4,
				'cols' => 30,
				'select_multiple' => 0,
				'select_size' => 1,
				'options_old' => array(),
				'options' => array('', '', ''),
				'options_text' => array('', '', ''),
				'mask' => 'nohtml',
				'regex' => '',
				'bbc' => 0,
				'default_value_text' => '',
				'default_value_textarea' => '',
				'default_value_options' => '',
				'default_value_options_multiple' => array(),
				'default_value_boolean' => '',
				'text_previous' => '',
				'text_next' => '',
				'enclose_type' => 'html',
				'enclose_text' => '',
				'show_message_index' => 0,
				'show_filter_operator' => 0,
				'hide_filter' => '',
				'show_filter_used_options' => 0,
				'show_record_list_type' => 'no',
				'show_record_list_width_unit' => 'auto',
				'show_record_list_width_value' => 0,
				'show_record_list_column' => '',
				'show_record_list_name' => '',
				'show_moderation_list_type' => 'no',
				'show_moderation_list_width_unit' => 'auto',
				'show_moderation_list_width_value' => 0,
				'show_moderation_list_column' => '',
				'show_moderation_list_name' => '',
				'not_empty_value' => 0,
				'unique_value' => 0,
				'subset' => '',
				'permissions' => array(),
				'active' => 1,
			);
		}

		// Others data.
		$show_record_list_type_array = array('no', 'one', 'multi');
		$context['params']['show_record_list_type_array'] = array();
		foreach ($show_record_list_type_array as $show_record_list_type)
			$context['params']['show_record_list_type_array'][$show_record_list_type] = $txt['cffp_show_list_type_' . $show_record_list_type];

		$show_record_list_width_unit_array = array('auto', '%', 'px');
		$context['params']['show_record_list_width_unit_array'] = array();
		foreach ($show_record_list_width_unit_array as $show_record_list_width_unit)
			$context['params']['show_record_list_width_unit_array'][$show_record_list_width_unit] = $txt['cffp_show_list_width_unit_' . $show_record_list_width_unit];

		$context['params']['show_record_list_column_array'] = cffpGetRecordColumnsMulti($context['gid']);

		$show_moderation_list_type_array = array('no', 'one', 'multi');
		$context['params']['show_moderation_list_type_array'] = array();
		foreach ($show_moderation_list_type_array as $show_moderation_list_type)
			$context['params']['show_moderation_list_type_array'][$show_moderation_list_type] = $txt['cffp_show_list_type_' . $show_moderation_list_type];

		$show_moderation_list_width_unit_array = array('auto', '%', 'px');
		$context['params']['show_moderation_list_width_unit_array'] = array();
		foreach ($show_moderation_list_width_unit_array as $show_moderation_list_width_unit)
			$context['params']['show_moderation_list_width_unit_array'][$show_moderation_list_width_unit] = $txt['cffp_show_list_width_unit_' . $show_moderation_list_width_unit];

		$context['params']['show_moderation_list_column_array'] = cffpGetModerationColumnsMulti($context['gid']);

		$enclose_type_array = array('html', 'bbc', 'php');
		$context['params']['enclose_type_array'] = array();
		foreach ($enclose_type_array as $enclose_type)
			$context['params']['enclose_type_array'][$enclose_type] = $txt['cffp_template_type_' . $enclose_type];

		$type_array = array('text', 'textarea', 'select', 'combobox', 'radio', 'check', 'boolean');
		$context['params']['type_array'] = array();
		foreach ($type_array as $type)
			$context['params']['type_array'][$type] = $txt['cffp_fieldtype_' . $type];

		$context['params']['type_filter_array'] = $type_filter_array;

		$context['params']['filtertype_array'] = array(
			'empty_non_empty' => $txt['cffp_filtertype_empty_non_empty'],
		);

		$mask_array = array('nohtml', 'email', 'number', 'regex');
		$context['params']['mask_array'] = array();
		foreach ($mask_array as $mask)
			$context['params']['mask_array'][$mask] = $txt['cffp_mask_' . $mask];

		$context['params']['hide_filter_array'] = cffpGetHideFilter($context['gid']);

		$context['params']['subset_array'] = cffpGetSubset($context['gid']);

		$context['params']['permissions_users'] = array(
			'guest' => $txt['guest'],
			'user' => $txt['user'],
			'localmod' => $txt['cffp_local_moderator'],
			'globalmod' => $txt['cffp_global_moderator'],
			'admin' => $txt['cffp_administrator'],
		);
		$context['params']['permissions_actions'] = array(
			'view' => $txt[305],
			'filter' => $txt['cffp_filter'],
			'sort' => $txt['cffp_sort'],
			'search' => $txt[182],
		);

		$request = db_query("
			SELECT name, type, show_stats
			FROM {$db_prefix}cffp_settings_groups
			WHERE ID_GROUP = " . $context['gid'], __FILE__, __LINE__);
		$context['params']['group'] = mysql_fetch_assoc($request);
		mysql_free_result($request);

		$context['params']['group']['type_txt'] = $txt['cffp_grouptype_' . $context['params']['group']['type']];
	}

	// Are we saving?
	if (isset($_POST['save']) && $context['gid'])
	{
		checkSession('get');

		// Everyone needs a name - even the (bracket) unknown...
		if ('' == trim($_POST['name']))
			fatal_lang_error('error_cffp_field_need_name');

		// Validate inputs.
		$_POST['name'] = $func['htmlspecialchars']($_POST['name'], ENT_QUOTES);
		$_POST['field_desc'] = $func['htmlspecialchars']($_POST['field_desc'], ENT_QUOTES);
		$_POST['field_long_desc'] = $func['htmlspecialchars']($_POST['field_long_desc'], ENT_QUOTES);
		$_POST['field_search_desc'] = $func['htmlspecialchars']($_POST['field_search_desc'], ENT_QUOTES);
		$_POST['field_search_long_desc'] = $func['htmlspecialchars']($_POST['field_search_long_desc'], ENT_QUOTES);
		$_POST['description'] = $func['htmlspecialchars']($_POST['description'], ENT_QUOTES);
		$_POST['subset'] = $func['htmlspecialchars']($_POST['subset'], ENT_QUOTES);
		$_POST['hide_filter'] = $func['htmlspecialchars']($_POST['hide_filter'], ENT_QUOTES);

		if (in_array($_POST['type'], $context['params']['type_filter_array']['text'][0]))
			$type_filter = $_POST['type_filter_text'];
		elseif (in_array($_POST['type'], $context['params']['type_filter_array']['combobox'][0]))
			$type_filter = $_POST['type_filter_combobox'];
		else // in_array($_POST['type'], $context['params']['type_filter_array']['boolean'][0]).
			$type_filter = $_POST['type_filter_boolean'];

		$exists_select_option = in_array($_POST['type'], array('select', 'combobox', 'radio', 'check'));
		$select_multiple = isset($_POST['select_multiple']) ? 1 : 0;
		$bbc = !$exists_select_option && isset($_POST['bbc']) ? 1 : 0;
		$select_multiple = isset($_POST['select_multiple']) ? 1 : 0;
		$show_message_index = isset($_POST['show_message_index']) ? 1 : 0;

		if ('record' == $context['params']['group']['type'] && 'no' != $_POST['show_record_list_type'])
		{
			$show_record_list_type = $_POST['show_record_list_type'];
			if ('multi' == $show_record_list_type)
			{
				if (empty($_POST['show_record_list_column']))
				{
					if ('auto' == $_POST['show_record_list_width_unit'])
						$show_record_list_width = '';
					else // %, px.
					{
						$show_record_list_width_value = (int) $_POST['show_record_list_width_value'];
						if (empty($show_record_list_width_value))
							$show_record_list_width = '';
						else
							$show_record_list_width = $show_record_list_width_value . $_POST['show_record_list_width_unit'];
					}

					$show_record_list_name = $func['htmlspecialchars']($_POST['show_record_list_name'], ENT_QUOTES);
					if (!cffpIsNotEmptyString($show_record_list_name))
						$show_record_list_type = 'one';
				}
				else
				{
					$show_record_list_width = strtok($_POST['show_record_list_column'], ',');
					if ('auto' == $show_record_list_width)
						$show_record_list_width = '';
					$show_record_list_name = strtok("\n");
				}
			}
			else // one.
			{
				if ('auto' == $_POST['show_record_list_width_unit'])
					$show_record_list_width = '';
				else // %, px.
				{
					$show_record_list_width_value = (int) $_POST['show_record_list_width_value'];
					if (empty($show_record_list_width_value))
						$show_record_list_width = '';
					else
						$show_record_list_width = $show_record_list_width_value . $_POST['show_record_list_width_unit'];
				}
				$show_record_list_name = '';
			}
		}
		else
		{
			$show_record_list_type = 'no';
			$show_record_list_width = '';
			$show_record_list_name = '';
		}

		if ('member' == $context['params']['group']['type'] && 'no' != $_POST['show_moderation_list_type'])
		{
			$show_moderation_list_type = $_POST['show_moderation_list_type'];
			if ('multi' == $show_moderation_list_type)
			{
				if (empty($_POST['show_moderation_list_column']))
				{
					if ('auto' == $_POST['show_moderation_list_width_unit'])
						$show_moderation_list_width = '';
					else // %, px.
					{
						$show_moderation_list_width_value = (int) $_POST['show_moderation_list_width_value'];
						if (empty($show_moderation_list_width_value))
							$show_moderation_list_width = '';
						else
							$show_moderation_list_width = $show_moderation_list_width_value . $_POST['show_moderation_list_width_unit'];
					}

					$show_moderation_list_name = $func['htmlspecialchars']($_POST['show_moderation_list_name'], ENT_QUOTES);
					if (!cffpIsNotEmptyString($show_moderation_list_name))
						$show_moderation_list_type = 'one';
				}
				else
				{
					$show_moderation_list_width = strtok($_POST['show_moderation_list_column'], ',');
					if ('auto' == $show_moderation_list_width)
						$show_moderation_list_width = '';
					$show_moderation_list_name = strtok("\n");
				}
			}
			else // one.
			{
				if ('auto' == $_POST['show_moderation_list_width_unit'])
					$show_moderation_list_width = '';
				else // %, px.
				{
					$show_moderation_list_width_value = (int) $_POST['show_moderation_list_width_value'];
					if (empty($show_moderation_list_width_value))
						$show_moderation_list_width = '';
					else
						$show_moderation_list_width = $show_moderation_list_width_value . $_POST['show_moderation_list_width_unit'];
				}
				$show_moderation_list_name = '';
			}
		}
		else
		{
			$show_moderation_list_type = 'no';
			$show_moderation_list_width = '';
			$show_moderation_list_name = '';
		}

		$not_empty_value = 'boolean' != $_POST['type'] && isset($_POST['not_empty_value']) ? 1 : 0;
		$unique_value = 'boolean' != $_POST['type'] && isset($_POST['unique_value']) ? 1 : 0;
		$active = isset($_POST['active']) ? 1 : 0;

		if ('empty_non_empty' == $type_filter && $not_empty_value)
		{
			if (in_array($_POST['type'], $context['params']['type_filter_array']['text'][0]))
				$type_filter = $context['params']['type_filter_array']['text'][1][0];
			else // in_array($_POST['type'], $context['params']['type_filter_array']['combobox'][0])
				$type_filter = $context['params']['type_filter_array']['combobox'][1][1];
		}

		$mask = 'text' == $_POST['type'] && isset($_POST['mask']) ? $_POST['mask'] : '';
		if ('regex' == $mask && isset($_POST['regex']))
			$mask .= $_POST['regex'];

		if (in_array($_POST['type'], array('text', 'textarea', 'combobox')))
			$length = isset($_POST['length']) ? (int) $_POST['length'] : 255;
		else
			$length = 0;
		$field_order = isset($_POST['field_order']) ? (int) $_POST['field_order'] : 0;
		$select_size = isset($_POST['select_size']) ? (int) $_POST['select_size'] : 1;

		$permissions = !empty($_POST['permissions']) ? implode(',', $_POST['permissions']) : '';

		$multiple = (('select' == $_POST['type'] || 'combobox' == $_POST['type']) && $select_multiple) || 'check' == $_POST['type'];
		$options = '';
		$options_text = '';
		$option_new_array = array();
		$default = '';
		if ($exists_select_option && !empty($_POST['select_option']))
		{
			$option_array = array();
			$option_text_array = array();
			$default_array = array();
			foreach ($_POST['select_option'] as $k => $v)
			{
				// Clean, clean, clean...
				$v = $func['htmlspecialchars']($v, ENT_QUOTES);
				$v = strtr($v, array(',' => ''));
				$v = trim($v);

				if (!cffpIsNotEmptyString($v))
					continue;

				if ('combobox' == $_POST['type'] && $_POST['length'])
					$v = $func['substr']($v, 0, $_POST['length']);

				if (in_array($v, $option_array))
					continue;

				$option_array[] = $v;
				$option_new_array[$k] = $v;
				if ('combobox' != $_POST['type'] && isset($_POST['select_option_text'][$k]))
				{
					$t = $func['htmlspecialchars']($_POST['select_option_text'][$k], ENT_QUOTES);
					$t = strtr($t, array(',' => ''));
					$t = trim($t);
					$option_text_array[] = cffpIsNotEmptyString($t) ? $t : $v;
				}

				// Is it default?
				if (!$multiple && isset($_POST['default_value_options']) && $_POST['default_value_options'] == $k)
					$default = $v;
				elseif ($multiple && isset($_POST['default_value_options_multiple']) && in_array($k, $_POST['default_value_options_multiple']))
					$default_array[] = $v;
			}
			$options = implode(',', $option_array);
			if ($multiple)
				$default = implode(',', $default_array);
			$options_text = implode(',', $option_text_array);
			if ($options_text == $options)
				$options_text = '';

			if ('combobox' == $type_filter && 0 < strlen($options_text))
				$type_filter = 'select';
		}
		elseif ('text' == $_POST['type'])
		{
			$default = $func['htmlspecialchars']($_POST['default_value_text'], ENT_QUOTES);
			if ($_POST['length'])
				$default = $func['substr']($default, 0, $_POST['length']);
			if (cffpIsNotEmptyString($default))
			{
				$error = cffpValidateMask($mask, $default);
				if (!empty($error))
					$default = '';
			}
		}
		elseif ('textarea' == $_POST['type'])
		{
			$options = (int) $_POST['rows'] . ',' . (int) $_POST['cols'];
			$default = $func['htmlspecialchars']($_POST['default_value_textarea'], ENT_QUOTES);
			if ($_POST['length'])
				$default = $func['substr']($default, 0, $_POST['length']);
		}
		elseif ('boolean' == $_POST['type'])
			$default = isset($_POST['default_value_boolean']) ? '1' : '0';

		$show_filter_operator = isset($_POST['show_filter_operator']) && 'boolean' != $_POST['type'] && 'empty_non_empty' != $type_filter ? 1 : 0;
		$show_filter_used_options = isset($_POST['show_filter_used_options']) && in_array($type_filter, array('select', 'combobox')) ? 1 : 0;

		if (in_array($_POST['type'], array('select', 'radio', 'check')) && 0 == strlen($options))
			fatal_lang_error('error_cffp_field_need_options');
		elseif (in_array($_POST['type'], array('select', 'combobox')))
		{
			$options2 = $select_multiple . ',' . $select_size;
			if (0 < strlen($options))
				$options2 .= ',' . $options;
		}
		else
			$options2 = $options;

		// Do the insertion/updates.
		if ($context['fid'])
		{
			db_query("
				UPDATE {$db_prefix}cffp_settings_fields
				SET
					name = '" . $_POST['name'] . "',
					field_desc = '" . $_POST['field_desc'] . "', field_long_desc = '" . $_POST['field_long_desc'] . "',
					field_search_desc = '" . $_POST['field_search_desc'] . "', field_search_long_desc = '" . $_POST['field_search_long_desc'] . "',
					description = '" . $_POST['description'] . "', field_order = $field_order,
					type = '" . $_POST['type'] . "', type_filter = '$type_filter',
					length = $length,
					options = '$options2', options_text = '$options_text',
					mask = '$mask', bbc = $bbc,
					default_value = '$default',
					text_previous = '" . $_POST['text_previous'] . "', text_next = '" . $_POST['text_next'] . "',
					enclose_type = '" . $_POST['enclose_type'] . "', enclose_text = '" . $_POST['enclose_text'] . "',
					show_message_index = $show_message_index,
					show_filter_operator = $show_filter_operator, hide_filter = '" . $_POST['hide_filter'] . "',
					show_filter_used_options = $show_filter_used_options,
					show_record_list_type = '$show_record_list_type', show_record_list_width = '$show_record_list_width',
					show_record_list_name = '$show_record_list_name',
					show_moderation_list_type = '$show_moderation_list_type', show_moderation_list_width = '$show_moderation_list_width',
					show_moderation_list_name = '$show_moderation_list_name',
					not_empty_value = $not_empty_value, unique_value = $unique_value,
					subset = '" . $_POST['subset'] . "',
					permissions = '" . $permissions . "', active = $active
				WHERE ID_FIELD = " . $context['fid'] . "
					AND ID_GROUP = " . $context['gid'], __FILE__, __LINE__);

			// Fix old values with new settings.
			$name_field = 'column_' . $context['params']['id_column'];
			$exists_select_option_old = in_array($context['params']['type'], array('select', 'combobox', 'radio', 'check'));
			if ('boolean' == $_POST['type'] && 'boolean' != $context['params']['type'])
			{
				db_query("
					UPDATE {$db_prefix}cffp_fields
					SET $name_field = '0'
					WHERE ID_GROUP = " . $context['gid'], __FILE__, __LINE__);
			}
			elseif ($exists_select_option && !$exists_select_option_old)
			{
				db_query("
					UPDATE {$db_prefix}cffp_fields
					SET $name_field = ''
					WHERE ID_GROUP = " . $context['gid'], __FILE__, __LINE__);
			}
			elseif ($exists_select_option && $exists_select_option_old && 'combobox' != $_POST['type'] && !empty($context['params']['options_old']))
			{
				if (empty($option_new_array))
				{
					db_query("
						UPDATE {$db_prefix}cffp_fields
						SET $name_field = ''
						WHERE ID_GROUP = " . $context['gid'], __FILE__, __LINE__);
				}
				else
				{
					$option_delete_array = array_diff($context['params']['options_old'], $option_new_array);
					if (!empty($option_delete_array))
					{
						db_query("
							UPDATE {$db_prefix}cffp_fields
							SET $name_field = ''
							WHERE ID_GROUP = " . $context['gid'] . "
								AND $name_field IN ('" . implode("', '", $option_delete_array) . "')", __FILE__, __LINE__);

						foreach ($option_delete_array as $option)
						{
							db_query("
								UPDATE {$db_prefix}cffp_fields
								SET $name_field = REPLACE($name_field, ',$option,', ',')
								WHERE ID_GROUP = " . $context['gid'] . "
									AND $name_field LIKE '%,$option,%'", __FILE__, __LINE__);
							db_query("
								UPDATE {$db_prefix}cffp_fields
								SET $name_field = REPLACE($name_field, '$option,', '')
								WHERE ID_GROUP = " . $context['gid'] . "
									AND $name_field LIKE '$option,%'", __FILE__, __LINE__);
							db_query("
								UPDATE {$db_prefix}cffp_fields
								SET $name_field = REPLACE($name_field, ',$option', '')
								WHERE ID_GROUP = " . $context['gid'] . "
									AND $name_field LIKE '%,$option'", __FILE__, __LINE__);
						}
					}
				}
			}

			if ($not_empty_value && !$context['params']['not_empty_value'])
			{
				db_query("
					UPDATE {$db_prefix}cffp_fields
					SET $name_field = '0'
					WHERE ID_GROUP = " . $context['gid'] . "
						AND $name_field = ''", __FILE__, __LINE__);
			}

			if ($unique_value && !$context['params']['unique_value'])
			{
				$request = db_query("
					SELECT COUNT(*) AS num, $name_field
					FROM {$db_prefix}cffp_fields
					WHERE ID_GROUP = " . $context['gid'] . "
						AND $name_field <> ''
					GROUP BY $name_field
					HAVING num > 1", __FILE__, __LINE__);
				while ($row = mysql_fetch_assoc($request))
				{
					db_query("
						UPDATE {$db_prefix}cffp_fields
						SET $name_field = ''
						WHERE ID_GROUP = " . $context['gid'] . "
							AND $name_field = '" . $row[$name_field] . "'", __FILE__, __LINE__);
				}
				mysql_free_result($request);
			}

			//TO DO: Adjust based on new text length limits.

		}
		else
		{
			$id_column = cffpGetIdColumn($context['gid']);
			cffp_db_insert('',
				'cffp_settings_fields',
				array(
					'ID_GROUP' => 'int', 'id_column' => 'int',
					'name' => 'string',
					'field_desc' => 'string', 'field_long_desc' => 'string',
					'field_search_desc' => 'string', 'field_search_long_desc' => 'string',
					'description' => 'string', 'field_order' => 'int',
					'type' => 'string', 'type_filter' => 'string',
					'length' => 'int',
					'options' => 'string', 'options_text' => 'string',
					'mask' => 'string', 'bbc' => 'int',
					'default_value' => 'string',
					'text_previous' => 'string', 'text_next' => 'string',
					'enclose_type' => 'string', 'enclose_text' => 'string',
					'show_message_index' => 'int',
					'show_filter_operator' => 'int', 'hide_filter' => 'string',
					'show_filter_used_options' => 'int',
					'show_record_list_type' => 'string', 'show_record_list_width' => 'string',
					'show_record_list_name' => 'string',
					'show_moderation_list_type' => 'string', 'show_moderation_list_width' => 'string',
					'show_moderation_list_name' => 'string',
					'not_empty_value' => 'int', 'unique_value' => 'int',
					'subset' => 'string',
					'permissions' => 'string', 'active' => 'int',
				),
				array(
					$context['gid'], $id_column,
					$_POST['name'],
					$_POST['field_desc'], $_POST['field_long_desc'],
					$_POST['field_search_desc'], $_POST['field_search_long_desc'],
					$_POST['description'], $field_order,
					$_POST['type'], $type_filter,
					$length,
					$options2, $options_text,
					$mask, $bbc,
					$default,
					$_POST['text_previous'], $_POST['text_next'],
					$_POST['enclose_type'], $_POST['enclose_text'],
					$show_message_index,
					$show_filter_operator, $_POST['hide_filter'],
					$show_filter_used_options,
					$show_record_list_type, $show_record_list_width,
					$show_record_list_name,
					$show_moderation_list_type, $show_moderation_list_width,
					$show_moderation_list_name,
					$not_empty_value, $unique_value,
					$_POST['subset'],
					$permissions, $active,
				)
			);
		}

		if ($context['params']['group']['show_stats'])
			cffpUpdateStats($context['gid'], 0, true);
	}
	// Deleting?
	elseif (isset($_GET['delete']) && $context['gid'] && $context['fid'])
	{
		checkSession('get');

		// Get column of field.
		$request = db_query("
			SELECT id_column
			FROM {$db_prefix}cffp_settings_fields
			WHERE ID_FIELD = " . $context['fid'], __FILE__, __LINE__);
		$row = mysql_fetch_assoc($request);
		mysql_free_result($request);
		$id_column = $row ? $row['id_column'] : 0;

		// Delete setting field.
		db_query("
			DELETE FROM {$db_prefix}cffp_settings_fields
			WHERE ID_FIELD = " . $context['fid'], __FILE__, __LINE__);

		// Delete column and field on posts.
		if (!empty($id_column))
		{
			if (!cffpRemoveFieldsColumn($context['gid'], $id_column))
			{
				db_query("
					UPDATE {$db_prefix}cffp_fields
					SET column_$id_column = ''
					WHERE ID_GROUP = " . $context['gid'], __FILE__, __LINE__);
			}
		}

		// Update statistics.
		if ($context['params']['group']['show_stats'])
			cffpUpdateStats($context['gid'], 0, true);
	}

	if (isset($_GET['delete']) || isset($_POST['save']))
	{
		checkSession('get');
		redirectexit('action=featuresettings;sa=cffp' . $context['params_url']);
	}
}

// Admin area settings (edit group).
function cffpSettingsEditGroup()
{
	global $txt, $context, $func, $db_prefix, $sourcedir;

	require_once ($sourcedir . '/Subs-CustomFieldFilterPost.php');
	require_once ($sourcedir . '/Subs-Db-CustomFieldFilterPost-mysql.php');

	loadTemplate('CustomFieldFilterPost');

	// Sort out the context!
	$context['gid'] = isset($_GET['gid']) ? (int) $_GET['gid'] : 0;
	$context['settings_title'] = $context['gid'] ? $txt['cffp_edit_group_title'] : $txt['cffp_add_group_title'];
	$start = isset($_GET['start']) ? (int) $_GET['start'] : 0;
	$sort = isset($_GET['sort']) ? $_GET['sort'] : 'group_order';
	$context['params_url'] = ';start=' . $start . ';sort=' . $sort . (isset($_GET['desc']) ? ';desc' : '');
	$context['admin_tabs']['tabs']['cffp']['is_selected'] = true;
	$context['sub_template'] = 'cffp_settings_edit_group';

	if ($context['gid'])
	{
		$request = db_query("
			SELECT
				ID_GROUP, name, description, group_order, text_input_before, text_input, text_output,
				boards, child_boards, type,
				show_group_name_message_index, show_group_name_filter, show_group_name_search,
				show_fields_name, show_fields_set,
				show_empty_fields_post, show_empty_fields_message_index,
				show_poster_data, show_modified_data,
				show_actions_as,
				show_member_expanded, show_member_expanded_data,
				show_member_expanded_data_recent, show_member_expanded_new,
				show_member_recent_at_top, member_max_height, member_one_value,
				show_stats,
				show_stats_post_text,
				show_stats_post_chart_average, show_stats_post_chart_hbars,
				show_stats_post_chart_vbars, show_stats_post_summary,
				show_stats_message_index_text,
				show_stats_message_index_chart_average, show_stats_message_index_chart_hbars,
				show_stats_message_index_chart_vbars, show_stats_message_index_summary,
				show_stats_order_by,
				show_sticky, show_locked,
				show_first_post_topic, show_replies_topic, no_show_with_subject_prefix,
				show_in_topic, show_filters, show_filters_operator, javascript,
				search_page_name, search_page_order_by,
				counter_show, counter_name, counter_number_name, counter_total_name, counter_average_name,
				top_poster_show, top_poster_name,
				moderation_list_show, moderation_list_name, moderation_list_order_by,
				permissions, active
			FROM {$db_prefix}cffp_settings_groups
			WHERE ID_GROUP = " . $context['gid'], __FILE__, __LINE__);
		$row = mysql_fetch_assoc($request);
		mysql_free_result($request);

		if ($row)
		{
			$context['params'] = array(
				'name' => $row['name'],
				'description' => $row['description'],
				'group_order' => $row['group_order'],
				'text_input_before' => $row['text_input_before'],
				'text_input' => $row['text_input'],
				'text_output' => $row['text_output'],
				'boards' => explode(',', $row['boards']),
				'child_boards' => $row['child_boards'],
				'type' => $row['type'],
				'show_group_name_message_index' => $row['show_group_name_message_index'],
				'show_group_name_filter' => $row['show_group_name_filter'],
				'show_group_name_search' => $row['show_group_name_search'],
				'show_fields_name' => $row['show_fields_name'],
				'show_fields_set' => $row['show_fields_set'],
				'show_empty_fields_post' => $row['show_empty_fields_post'],
				'show_empty_fields_message_index' => $row['show_empty_fields_message_index'],
				'show_poster_data' => $row['show_poster_data'],
				'show_modified_data' => $row['show_modified_data'],
				'show_actions_as' => $row['show_actions_as'],
				'show_member_expanded' => $row['show_member_expanded'],
				'show_member_expanded_data' => $row['show_member_expanded_data'],
				'show_member_expanded_data_recent' => $row['show_member_expanded_data_recent'],
				'show_member_expanded_new' => $row['show_member_expanded_new'],
				'show_member_recent_at_top' => $row['show_member_recent_at_top'],
				'member_max_height' => $row['member_max_height'],
				'member_one_value' => $row['member_one_value'],
				'show_stats' => $row['show_stats'],
				'show_stats_post_text' => $row['show_stats_post_text'],
				'show_stats_post_chart_average' => !empty($row['show_stats_post_chart_average']) ? explode(',', $row['show_stats_post_chart_average']) : array(),
				'show_stats_post_chart_hbars' => !empty($row['show_stats_post_chart_hbars']) ? explode(',', $row['show_stats_post_chart_hbars']) : array(),
				'show_stats_post_chart_vbars' => !empty($row['show_stats_post_chart_vbars']) ? explode(',', $row['show_stats_post_chart_vbars']) : array(),
				'show_stats_post_summary' => !empty($row['show_stats_post_summary']) ? explode(',', $row['show_stats_post_summary']) : array(),
				'show_stats_message_index_text' => $row['show_stats_message_index_text'],
				'show_stats_message_index_chart_average' => !empty($row['show_stats_message_index_chart_average']) ? explode(',', $row['show_stats_message_index_chart_average']) : array(),
				'show_stats_message_index_chart_hbars' => !empty($row['show_stats_message_index_chart_hbars']) ? explode(',', $row['show_stats_message_index_chart_hbars']) : array(),
				'show_stats_message_index_chart_vbars' => !empty($row['show_stats_message_index_chart_vbars']) ? explode(',', $row['show_stats_message_index_chart_vbars']) : array(),
				'show_stats_message_index_summary' => !empty($row['show_stats_message_index_summary']) ? explode(',', $row['show_stats_message_index_summary']) : array(),
				'show_stats_order_by' => !empty($row['show_stats_order_by']) ? explode(',', $row['show_stats_order_by']) : array(),
				'show_sticky' => $row['show_sticky'],
				'show_locked' => $row['show_locked'],
				'show_first_post_topic' => $row['show_first_post_topic'],
				'show_replies_topic' => $row['show_replies_topic'],
				'no_show_with_subject_prefix' => $row['no_show_with_subject_prefix'],
				'show_in_topic' => $row['show_in_topic'],
				'show_filters' => !empty($row['show_filters']) ? explode(',', $row['show_filters']) : array(),
				'show_filters_operator' => !empty($row['show_filters_operator']) ? explode(',', $row['show_filters_operator']) : array(),
				'javascript' => $row['javascript'],
				'search_page_post_default' => 'record' != $row['type'] ? !cffpIsNotEmptyString($row['search_page_name']) : 1,
				'search_page_name_post' => 'record' != $row['type'] && cffpIsNotEmptyString($row['search_page_name']) ? explode(',', $row['search_page_name']) : array('', '', ''),
				'search_page_name_record' => 'record' == $row['type'] ? $row['search_page_name'] : '',
				'search_page_order_by_post' => 'record' != $row['type'] && cffpIsNotEmptyString($row['search_page_name']) ? explode(',', $row['search_page_order_by']) : array('m.posterTime desc', 'm.posterTime desc', 'm.posterTime desc'),
				'search_page_order_by_record' => 'record' == $row['type'] ? $row['search_page_order_by'] : 'cffpf0.posterTime desc',
				'counter_show' => $row['counter_show'],
				'counter_name' => $row['counter_name'],
				'counter_number_name' => $row['counter_number_name'],
				'counter_total_name' => $row['counter_total_name'],
				'counter_average_name' => $row['counter_average_name'],
				'top_poster_show' => $row['top_poster_show'],
				'top_poster_name' => $row['top_poster_name'],
				'moderation_list_show' => $row['moderation_list_show'],
				'moderation_list_name' => $row['moderation_list_name'],
				'moderation_list_order_by' => $row['moderation_list_order_by'],
				'permissions' => !empty($row['permissions']) ? explode(',', $row['permissions']) : array(),
				'active' => $row['active'],
			);
		}
	}

	// Setup the default values as needed.
	if (empty($context['params']))
	{
		$context['params'] = array(
			'name' => '',
			'description' => '',
			'group_order' => 0,
			'text_input_before' => '',
			'text_input' => '',
			'text_output' => '',
			'boards' => array(),
			'child_boards' => 0,
			'type' => 'fields',
			'show_group_name_message_index' => 0,
			'show_group_name_filter' => 0,
			'show_group_name_search' => 0,
			'show_fields_name' => 1,
			'show_fields_set' => 0,
			'show_empty_fields_post' => 0,
			'show_empty_fields_message_index' => 0,
			'show_poster_data' => 1,
			'show_modified_data' => 1,
			'show_actions_as' => 'icon_text',
			'show_member_expanded' => 0,
			'show_member_expanded_data' => 0,
			'show_member_expanded_data_recent' => 0,
			'show_member_expanded_new' => 0,
			'show_member_recent_at_top' => 0,
			'member_max_height' => 0,
			'member_one_value' => 0,
			'show_stats' => 0,
			'show_stats_post_text' => '',
			'show_stats_post_chart_average' => array(),
			'show_stats_post_chart_hbars' => array(),
			'show_stats_post_chart_vbars' => array(),
			'show_stats_post_summary' => array(),
			'show_stats_message_index_text' => '',
			'show_stats_message_index_chart_average' => array(),
			'show_stats_message_index_chart_hbars' => array(),
			'show_stats_message_index_chart_vbars' => array(),
			'show_stats_message_index_summary' => array(),
			'show_stats_order_by' => array(),
			'show_sticky' => null,
			'show_locked' => null,
			'show_first_post_topic' => 1,
			'show_replies_topic' => 1,
			'no_show_with_subject_prefix' => '',
			'show_in_topic' => 'no',
			'show_filters' => array(),
			'show_filters_operator' => array(),
			'javascript' => '',
			'search_page_post_default' => 1,
			'search_page_name_post' => array('', '', ''),
			'search_page_name_record' => '',
			'search_page_order_by_post' => array('m.posterTime desc', 'm.posterTime desc', 'm.posterTime desc'),
			'search_page_order_by_record' => 'cffpf0.posterTime desc',
			'counter_show' => 0,
			'counter_name' => '',
			'counter_number_name' => '',
			'counter_total_name' => '',
			'counter_average_name' => '',
			'top_poster_show' => 0,
			'top_poster_name' => '',
			'moderation_list_show' => 0,
			'moderation_list_name' => '',
			'moderation_list_order_by' => '',
			'permissions' => array(),
			'active' => 1,
		);
	}

	// Others data.
	$type_array = array('fields', 'record', 'member', 'filters');
	$context['params']['type_array'] = array();
	foreach ($type_array as $type)
		$context['params']['type_array'][$type] = $txt['cffp_grouptype_' . $type];

	$show_actions_as_array = array('button', 'icon_text', 'icon', 'text');
	$context['params']['show_actions_as_array'] = array();
	foreach ($show_actions_as_array as $show_actions_as)
		$context['params']['show_actions_as_array'][$show_actions_as] = $txt['cffp_show_actions_as_' . $show_actions_as];

	$show_in_topic_array = array('no', 'begin_all', 'end_all', 'begin_first', 'end_first');
	$context['params']['show_in_topic_array'] = array();
	foreach ($show_in_topic_array as $show_in_topic)
		$context['params']['show_in_topic_array'][$show_in_topic] = $txt['cffp_show_in_topic_' . $show_in_topic];

	$context['params']['stats_position'] = array('post', 'message_index');

	$show_stats_summary_array = array('average' => true, 'sum' => true, 'max' => true, 'min' => true, 'highest' => true, 'lowest' => true, 'most_frequent' => false, 'least_frequent' => false, 'total' => false);
	$context['params']['show_stats_summary_array'] = array();
	foreach ($show_stats_summary_array as $show_stats_summary => $is_numeric)
		$context['params']['show_stats_summary_array'][$show_stats_summary] = array($txt['cffp_show_stats_summary_' . $show_stats_summary], $is_numeric);

	$context['params']['show_stats_summary2_array'] = $context['params']['show_stats_summary_array'];
	unset($context['params']['show_stats_summary2_array']['max']);
	unset($context['params']['show_stats_summary2_array']['min']);

	$context['params']['show_filters_array'] = cffpGetSettingsPredefinedFilters();

	$context['params']['search_page_order_by_post_array'] = array(
		'm.subject asc' => $txt[70] . ' (' . $txt['cffp_order_by_asc'] . ')',
		'm.subject desc' => $txt[70] . ' (' . $txt['cffp_order_by_desc'] . ')',
		'm.posterTime asc' => $txt['cffp_poster_time'] . ' (' . $txt['cffp_order_by_asc'] . ')',
		'm.posterTime desc' => $txt['cffp_poster_time'] . ' (' . $txt['cffp_order_by_desc'] . ')',
		'display_name asc' => $txt[279] . ' (' . $txt['cffp_order_by_asc'] . ')',
		'display_name desc' => $txt[279] . ' (' . $txt['cffp_order_by_desc'] . ')',
	);

	$fields = $context['gid'] ? cffpGetRecordListFields($context['gid']) : array();
	$context['params']['search_page_order_by_record_array'] = array();
	foreach ($fields as $field)
	{
		$context['params']['search_page_order_by_record_array']['cffpf0.column_' . $field['id_column'] . ' asc'] = $field['name'] . ' (' . $txt['cffp_order_by_asc'] . ')';
		$context['params']['search_page_order_by_record_array']['cffpf0.column_' . $field['id_column'] . ' desc'] = $field['name'] . ' (' . $txt['cffp_order_by_desc'] . ')';
	}
	$context['params']['search_page_order_by_record_array']['cffpf0.posterTime asc'] = $txt['cffp_poster_time'] . ' (' . $txt['cffp_order_by_asc'] . ')';
	$context['params']['search_page_order_by_record_array']['cffpf0.posterTime desc'] = $txt['cffp_poster_time'] . ' (' . $txt['cffp_order_by_desc'] . ')';
	$context['params']['search_page_order_by_record_array']['display_name asc'] = $txt[279] . ' (' . $txt['cffp_order_by_asc'] . ')';
	$context['params']['search_page_order_by_record_array']['display_name desc'] = $txt[279] . ' (' . $txt['cffp_order_by_desc'] . ')';

	foreach ($context['params']['search_page_order_by_post'] as $k => $v)
	{
		if (!isset($context['params']['search_page_order_by_post_array'][$v]))
			$context['params']['search_page_order_by_post'][$k] = 'm.posterTime desc';
	}

	if (!isset($context['params']['search_page_order_by_record_array'][$context['params']['search_page_order_by_record']]))
		$context['params']['search_page_order_by_record'] = 'cffpf0.posterTime desc';

	$context['params']['search_page_post_more_array'] = cffpGetSearchPagePost();

	$context['params']['moderation_list_order_by_array'] = array(
		'm.subject asc' => $txt[70] . ' (' . $txt['cffp_order_by_asc'] . ')',
		'm.subject desc' => $txt[70] . ' (' . $txt['cffp_order_by_desc'] . ')',
	);
	$fields = $context['gid'] ? cffpGetModerationListFields($context['gid']) : array();
	foreach ($fields as $field)
	{
		$context['params']['moderation_list_order_by_array']['cffpf.column_' . $field['id_column'] . ' asc'] = $field['name'] . ' (' . $txt['cffp_order_by_asc'] . ')';
		$context['params']['moderation_list_order_by_array']['cffpf.column_' . $field['id_column'] . ' desc'] = $field['name'] . ' (' . $txt['cffp_order_by_desc'] . ')';
	}
	$context['params']['moderation_list_order_by_array']['cffpf.posterTime asc'] = $txt['cffp_poster_time'] . ' (' . $txt['cffp_order_by_asc'] . ')';
	$context['params']['moderation_list_order_by_array']['cffpf.posterTime desc'] = $txt['cffp_poster_time'] . ' (' . $txt['cffp_order_by_desc'] . ')';
	$context['params']['moderation_list_order_by_array']['display_name asc'] = $txt[279] . ' (' . $txt['cffp_order_by_asc'] . ')';
	$context['params']['moderation_list_order_by_array']['display_name desc'] = $txt[279] . ' (' . $txt['cffp_order_by_desc'] . ')';

	if (!isset($context['params']['moderation_list_order_by_array'][$context['params']['moderation_list_order_by']]))
		$context['params']['moderation_list_order_by'] = 'cffpf.posterTime desc';

	$context['params']['permissions_users'] = array(
		'guest' => $txt['guest'],
		'user' => $txt['user'],
		'owner' => array($txt['cffp_owner_post'], $txt['cffp_owner_group']),
		'localmod' => $txt['cffp_local_moderator'],
		'globalmod' => $txt['cffp_global_moderator'],
		'admin' => $txt['cffp_administrator'],
	);
	$context['params']['permissions_actions'] = array(
		'view' => array($txt[305], $txt['cffp_view_message_index']),
		'stats' => $txt['cffp_view_stats'],
		'add' => $txt['cffp_add'],
		'modify' => $txt[17],
		'remove' => $txt[31],
		'move' => $txt['cffp_move'],
		'search' => $txt[182],
	);

	loadJumpTo();
	$context['params']['all_boards'] = $context['jump_to'];

	// Are we saving?
	if (isset($_POST['save']))
	{
		checkSession('get');

		// Everyone needs a name - even the (bracket) unknown...
		if ('' == trim($_POST['name']))
			fatal_lang_error('error_cffp_group_need_name');

		// Validate inputs.
		$_POST['name'] = $func['htmlspecialchars']($_POST['name'], ENT_QUOTES);
		$_POST['description'] = $func['htmlspecialchars']($_POST['description'], ENT_QUOTES);
		$_POST['text_input'] = 'filters' != $_POST['type'] ? $func['htmlspecialchars']($_POST['text_input'], ENT_QUOTES) : '';
		$_POST['text_output'] = 'filters' != $_POST['type'] ? $func['htmlspecialchars']($_POST['text_output'], ENT_QUOTES) : '';

		$show_group_name_message_index = 'filters' != $_POST['type'] && isset($_POST['show_group_name_message_index']) ? 1 : 0;
		$show_fields_name = 'filters' != $_POST['type'] && isset($_POST['show_fields_name']) ? 1 : 0;
		$show_empty_fields_post = 'filters' != $_POST['type'] && isset($_POST['show_empty_fields_post']) ? 1 : 0;
		$show_empty_fields_message_index = 'filters' != $_POST['type'] && isset($_POST['show_empty_fields_message_index']) ? 1 : 0;
		$show_actions_as = 'filters' != $_POST['type'] ? $_POST['show_actions_as'] : 'icon_text';

		$show_fields_set = 'fields' == $_POST['type'] && isset($_POST['show_fields_set']) ? 1 : 0;

		$_POST['text_input_before'] = 'record' == $_POST['type'] ? $func['htmlspecialchars']($_POST['text_input_before'], ENT_QUOTES) : '';
		$show_poster_data = 'record' == $_POST['type'] && isset($_POST['show_poster_data']) ? 1 : 0;
		$show_modified_data = $show_poster_data && isset($_POST['show_modified_data']) ? 1 : 0;
		$counter_show = 'record' == $_POST['type'] ? $_POST['counter_show'] : 0;
		$counter_name = $counter_show ? $_POST['counter_name'] : '';
		$counter_number_name = $counter_show ? $_POST['counter_number_name'] : '';
		$counter_total_name = $counter_show ? $_POST['counter_total_name'] : '';
		$counter_average_name = $counter_show ? $_POST['counter_average_name'] : '';
		$top_poster_show = 'record' == $_POST['type'] && isset($_POST['top_poster_show']) ? 1 : 0;
		$top_poster_name = $top_poster_show ? $_POST['top_poster_name'] : '';
		$profile_list_show = 0;
		$profile_list_name = '';

		$show_member_expanded = 'member' == $_POST['type'] ? $_POST['show_member_expanded'] : 0;
		$show_member_expanded_data = 'member' == $_POST['type'] ? $_POST['show_member_expanded_data'] : 0;
		$show_member_expanded_data_recent = 'member' == $_POST['type'] && !empty($_POST['show_member_expanded_data_recent']) ? (int) $_POST['show_member_expanded_data_recent'] : 0;
		$show_member_expanded_new = 'member' == $_POST['type'] ? $_POST['show_member_expanded_new'] : 0;
		$show_member_recent_at_top = 'member' == $_POST['type'] && isset($_POST['show_member_recent_at_top']) ? 1 : 0;
		$member_max_height = 'member' == $_POST['type'] && !empty($_POST['member_max_height']) ? (int) $_POST['member_max_height'] : 0;
		$member_one_value = 'member' == $_POST['type'] && isset($_POST['member_one_value']) ? 1 : 0;
		$show_stats = 'member' == $_POST['type'] && isset($_POST['show_stats']) ? 1 : 0;
		$show_stats_post_text = $show_stats ? $func['htmlspecialchars']($_POST['show_stats_post_text'], ENT_QUOTES) : '';
		$show_stats_post_chart_average = $show_stats && !empty($_POST['show_stats_post_chart_average']) && in_array('show', $_POST['show_stats_post_chart_average']) ? implode(',', $_POST['show_stats_post_chart_average']) : '';
		$show_stats_post_chart_hbars = $show_stats && !empty($_POST['show_stats_post_chart_hbars']) && in_array('show', $_POST['show_stats_post_chart_hbars']) ? implode(',', $_POST['show_stats_post_chart_hbars']) : '';
		$show_stats_post_chart_vbars = $show_stats && !empty($_POST['show_stats_post_chart_vbars']) && in_array('show', $_POST['show_stats_post_chart_vbars']) ? implode(',', $_POST['show_stats_post_chart_vbars']) : '';
		$show_stats_post_summary = $show_stats && !empty($_POST['show_stats_post_summary']) ? implode(',', $_POST['show_stats_post_summary']) : '';
		$show_stats_message_index_text = $show_stats ? $func['htmlspecialchars']($_POST['show_stats_message_index_text'], ENT_QUOTES) : '';
		$show_stats_message_index_chart_average = $show_stats && !empty($_POST['show_stats_message_index_chart_average']) && in_array('show', $_POST['show_stats_message_index_chart_average']) ? implode(',', $_POST['show_stats_message_index_chart_average']) : '';
		$show_stats_message_index_chart_hbars = $show_stats && !empty($_POST['show_stats_message_index_chart_hbars']) && in_array('show', $_POST['show_stats_message_index_chart_hbars']) ? implode(',', $_POST['show_stats_message_index_chart_hbars']) : '';
		$show_stats_message_index_chart_vbars = $show_stats && !empty($_POST['show_stats_message_index_chart_vbars']) && in_array('show', $_POST['show_stats_message_index_chart_vbars']) ? implode(',', $_POST['show_stats_message_index_chart_vbars']) : '';
		$show_stats_message_index_summary = $show_stats && !empty($_POST['show_stats_message_index_summary']) ? implode(',', $_POST['show_stats_message_index_summary']) : '';
		$show_stats_order_by = $show_stats && !empty($_POST['show_stats_order_by']) ? implode(',', $_POST['show_stats_order_by']) : '';
		$moderation_list_show = 'member' == $_POST['type'] && isset($_POST['moderation_list_show']) ? 1 : 0;
		$moderation_list_menu = 0;
		if ($moderation_list_show)
		{
			$moderation_list_name = $_POST['moderation_list_name'];
			$moderation_list_order_by = !empty($_POST['moderation_list_order_by']) ? $_POST['moderation_list_order_by'] : 'cffpf.posterTime desc';
		}
		else
		{
			$moderation_list_name = '';
			$moderation_list_order_by = '';
		}

		$show_group_name_filter = isset($_POST['show_group_name_filter']) ? 1 : 0;
		$show_group_name_search = isset($_POST['show_group_name_search']) ? 1 : 0;
		if ('filters' == $_POST['type'] && !empty($_POST['show_filters']))
		{
			$show_filters = implode(',', $_POST['show_filters']);
			$show_filters_operator_array = !empty($_POST['show_filters_operator']) ? $_POST['show_filters_operator'] : array();
			$show_filters_operator_array = array_intersect($show_filters_operator_array, $_POST['show_filters']);
			$show_filters_operator = !empty($show_filters_operator_array) ? implode(',', $show_filters_operator_array) : '';
		}
		else
		{
			$show_filters = '';
			$show_filters_operator = '';
		}

		if ('record' == $_POST['type'])
		{
			if (!cffpIsNotEmptyString($_POST['search_page_name_record']))
				$search_page_name = '';
			else
			{
				$search_page_name = $func['htmlspecialchars']($_POST['search_page_name_record'], ENT_QUOTES);
				$search_page_name = strtr($search_page_name, array(',' => ''));
				$search_page_name = trim($search_page_name);

				if (in_array($search_page_name, $context['params']['search_page_post_more_array']))
					fatal_lang_error('error_cffp_search_page_in_use');

				$request = db_query("
					SELECT ID_GROUP
					FROM {$db_prefix}cffp_settings_groups
					WHERE type = 'record'" . (!$context['gid'] ? '' : '
						AND ID_GROUP <> ' . $context['gid']). "
						AND search_page_name = '" . $search_page_name . "'", __FILE__, __LINE__);
				$num_rows = mysql_num_rows($request);
				mysql_free_result($request);
				if (0 < $num_rows)
					fatal_lang_error('error_cffp_search_page_in_use');
			}
			$search_page_order_by = $_POST['search_page_order_by_record'];
		}
		elseif (empty($_POST['search_page_post_default']))
		{
			$search_page_name_array = array();
			$search_page_order_by_array = array();

			foreach ($_POST['search_page_name_post'] as $k => $v)
			{
				// Clean, clean, clean...
				$v = $func['htmlspecialchars']($v, ENT_QUOTES);
				$v = strtr($v, array(',' => ''));
				$v = trim($v);

				if (!cffpIsNotEmptyString($v))
					continue;

				if (in_array($v, $search_page_name_array))
					continue;

				$request = db_query("
					SELECT ID_GROUP
					FROM {$db_prefix}cffp_settings_groups
					WHERE type = 'record'
						AND search_page_name = '$v'", __FILE__, __LINE__);
				$num_rows = mysql_num_rows($request);
				mysql_free_result($request);
				if (0 < $num_rows)
					fatal_lang_error('error_cffp_search_page_in_use_record');

				$search_page_name_array[] = $v;
				$search_page_order_by_array[] = $_POST['search_page_order_by_post'][$k];
			}

			if (!empty($search_page_name_array))
			{
				$search_page_name = implode(',', $search_page_name_array);
				$search_page_order_by = implode(',', $search_page_order_by_array);
			}
			else
			{
				$search_page_name = '';
				$search_page_order_by = 'm.posterTime desc';
			}
		}
		else
		{
			$search_page_name = '';
			$search_page_order_by = 'm.posterTime desc';
		}

		$show_sticky = 'filters' != $_POST['type'] ? $_POST['show_sticky'] : null;
		if (is_numeric($show_sticky))
			$db_type_show_sticky = 'int';
		else
		{
			$db_type_show_sticky = 'raw';
			$show_sticky = 'NULL';
		}
		$show_locked = 'filters' != $_POST['type'] ? $_POST['show_locked'] : null;
		if (is_numeric($show_locked))
			$db_type_show_locked = 'int';
		else
		{
			$db_type_show_locked = 'raw';
			$show_locked = 'NULL';
		}
		$show_first_post_topic = 'filters' != $_POST['type'] && isset($_POST['show_first_post_topic']) ? 1 : 0;
		$show_replies_topic = 'filters' != $_POST['type'] && isset($_POST['show_replies_topic']) ? 1 : 0;
		$show_in_topic = 'member' == $_POST['type'] && $show_first_post_topic && !$show_replies_topic ? $_POST['show_in_topic'] : 'no';
		$active = isset($_POST['active']) ? 1 : 0;

		$group_order = isset($_POST['group_order']) ? (int) $_POST['group_order'] : 0;

		if ('filters' != $_POST['type'])
		{
			$no_show_with_subject_prefix = array_map('trim', explode(',', $_POST['no_show_with_subject_prefix']));
			$no_show_with_subject_prefix = array_filter($no_show_with_subject_prefix, 'cffpIsNotEmptyString');
			$no_show_with_subject_prefix = array_unique($no_show_with_subject_prefix);
			$no_show_with_subject_prefix = $func['htmlspecialchars'](implode(',', $no_show_with_subject_prefix), ENT_QUOTES);
		}
		else
			$no_show_with_subject_prefix = '';

		$javascript = 'filters' != $_POST['type'] ? $_POST['javascript'] : '';

		if (empty($_POST['boards']))
			$boards = '';
		else
			$boards = in_array('all', $_POST['boards']) ? 'all' : implode(',', $_POST['boards']);
		if (is_numeric($_POST['child_boards']))
		{
			$db_type_child_boards = 'int';
			$child_boards = $_POST['child_boards'];
		}
		else
		{
			$db_type_child_boards = 'raw';
			$child_boards = 'NULL';
		}

		$permissions = !empty($_POST['permissions']) ? $_POST['permissions'] : array();
		$permissions_remove = array();
		switch ($_POST['type'])
		{
			case 'fields':
			{
				foreach ($context['params']['permissions_users'] as $pu => $txt_pu)
				{
					$permissions_remove[] = $pu . ':stats';
					$permissions_remove[] = $pu . ':add';
					$permissions_remove[] = $pu . ':remove';
				}
				break;
			}
			case 'record':
			{
				foreach ($context['params']['permissions_users'] as $pu => $txt_pu)
					$permissions_remove[] = $pu . ':stats';
				break;
			}
			case 'member':
			{
				$permissions_remove[] = 'owner:add';
				if (!$show_stats)
				{
					foreach ($context['params']['permissions_users'] as $pu => $txt_pu)
						$permissions_remove[] = $pu . ':stats';
				}
				break;
			}
			case 'filters':
			{
				foreach ($context['params']['permissions_users'] as $pu => $txt_pu)
				{
					$permissions_remove[] = $pu . ':stats';
					$permissions_remove[] = $pu . ':add';
					$permissions_remove[] = $pu . ':modify';
					$permissions_remove[] = $pu . ':remove';
					$permissions_remove[] = $pu . ':move';
				}
				$permissions_remove[] = 'owner:view';
				break;
			}
		}
		if (!empty($permissions_remove))
			$permissions = array_diff($permissions, $permissions_remove);
		$permissions = implode(',', $permissions);

		// Do the insertion/updates.
		if ($context['gid'])
		{
			db_query("
				UPDATE {$db_prefix}cffp_settings_groups
				SET
					name = '" . $_POST['name'] . "', description = '" . $_POST['description'] . "',
					group_order = $group_order,
					text_input_before = '" . $_POST['text_input_before'] . "', text_input = '" . $_POST['text_input'] . "',
					text_output = '" . $_POST['text_output'] . "',
					boards = '$boards', child_boards = $child_boards,
					type = '" . $_POST['type'] . "',
					show_group_name_message_index = $show_group_name_message_index, show_group_name_filter = $show_group_name_filter,
					show_group_name_search = $show_group_name_search,
					show_fields_name = $show_fields_name, show_fields_set = $show_fields_set,
					show_empty_fields_post = $show_empty_fields_post, show_empty_fields_message_index = $show_empty_fields_message_index,
					show_poster_data = $show_poster_data, show_modified_data = $show_modified_data,
					show_actions_as = '$show_actions_as',
					show_member_expanded = $show_member_expanded, show_member_expanded_data = $show_member_expanded_data,
					show_member_expanded_data_recent = $show_member_expanded_data_recent, show_member_expanded_new = $show_member_expanded_new,
					show_member_recent_at_top = $show_member_recent_at_top, member_max_height = $member_max_height,
					member_one_value = $member_one_value,
					show_stats = $show_stats,
					show_stats_post_text = '$show_stats_post_text',
					show_stats_post_chart_average = '$show_stats_post_chart_average', show_stats_post_chart_hbars = '$show_stats_post_chart_hbars',
					show_stats_post_chart_vbars = '$show_stats_post_chart_vbars', show_stats_post_summary = '$show_stats_post_summary',
					show_stats_message_index_text = '$show_stats_message_index_text',
					show_stats_message_index_chart_average = '$show_stats_message_index_chart_average', show_stats_message_index_chart_hbars = '$show_stats_message_index_chart_hbars',
					show_stats_message_index_chart_vbars = '$show_stats_message_index_chart_vbars', show_stats_message_index_summary = '$show_stats_message_index_summary',
					show_stats_order_by = '$show_stats_order_by',
					show_sticky = $show_sticky, show_locked = $show_locked,
					show_first_post_topic = $show_first_post_topic, show_replies_topic = $show_replies_topic,
					no_show_with_subject_prefix = '$no_show_with_subject_prefix', show_in_topic = '$show_in_topic',
					show_filters = '$show_filters', show_filters_operator = '$show_filters_operator',
					javascript = '$javascript',
					search_page_name = '$search_page_name', search_page_order_by = '$search_page_order_by',
					counter_show = $counter_show, counter_name = '$counter_name',
					counter_number_name = '$counter_number_name',
					counter_total_name = '$counter_total_name', counter_average_name = '$counter_average_name',
					top_poster_show = $top_poster_show, top_poster_name = '$top_poster_name',
					profile_list_show = $profile_list_show, profile_list_name = '$profile_list_name',
					moderation_list_show = $moderation_list_show, moderation_list_menu = $moderation_list_menu,
					moderation_list_name = '$moderation_list_name', moderation_list_order_by = '$moderation_list_order_by',
					permissions = '$permissions', active = $active
				WHERE ID_GROUP = " . $context['gid'], __FILE__, __LINE__);

			// Fix old values with new settings.
			if (('member' == $context['params']['type'] && 'member' != $_POST['type']) || ('member' != $context['params']['type'] && 'member' == $_POST['type']))
			{
				db_query("
					DELETE FROM {$db_prefix}cffp_fields
					WHERE ID_GROUP = " . $context['gid'], __FILE__, __LINE__);

				if ('member' == $context['params']['type'] && 'member' != $_POST['type'])
				{
					db_query("
						UPDATE {$db_prefix}cffp_settings_fields
						SET
							show_moderation_list_type = 'no',
							show_moderation_list_width = '',
							show_moderation_list_name = ''
						WHERE ID_GROUP = " . $context['gid'], __FILE__, __LINE__);
				}
			}

			if ('member' == $context['params']['type'] && 'member' == $_POST['type'] && !$context['params']['member_one_value'] && $member_one_value)
			{
				$request = db_query("
					SELECT ID_MSG, ID_MEMBER, COUNT(*) AS num
					FROM {$db_prefix}cffp_fields
					WHERE ID_GROUP = " . $context['gid'] . "
						AND ID_MEMBER <> 0
					GROUP BY ID_MSG, ID_MEMBER
					HAVING num > 1", __FILE__, __LINE__);
				while ($row = mysql_fetch_assoc($request))
				{
					db_query("
						DELETE FROM {$db_prefix}cffp_fields
						WHERE ID_GROUP = " . $context['gid'] . "
							AND ID_MSG = " . $row['ID_MSG'] . "
							AND ID_MEMBER = " . $row['ID_MEMBER'], __FILE__, __LINE__);
				}
				mysql_free_result($request);
			}

			if ('record' == $context['params']['type'] && 'record' != $_POST['type'])
			{
				db_query("
					UPDATE {$db_prefix}cffp_settings_fields
					SET
						show_record_list_type = 'no',
						show_record_list_width = '',
						show_record_list_name = ''
					WHERE ID_GROUP = " . $context['gid'], __FILE__, __LINE__);
				db_query("
					UPDATE {$db_prefix}cffp_settings_groups
					SET
						template_quick_input_regex = '',
						template_quick_input_params = '',
						template_quick_input_help = '',
						template_quick_input_show_modify = 0
					WHERE ID_GROUP = " . $context['gid'], __FILE__, __LINE__);
				cffpRemoveCounterRecordColumns($context['gid']);
			}
			elseif ('record' != $context['params']['type'] && 'record' == $_POST['type'])
			{
				cffpAddCounterRecordColumns($context['gid']);
				if ($active)
					cffpUpdateCounterRecordAllMembers($context['gid'], false);
			}
			elseif ('record' == $context['params']['type'] && 'record' == $_POST['type'])
			{
				if (!$context['params']['active'] && $active)
					cffpUpdateCounterRecordAllMembers($context['gid'], false);
			}

			if ('filters' != $context['params']['type'] && 'filters' == $_POST['type'])
			{
				db_query("
					DELETE FROM {$db_prefix}cffp_fields
					WHERE ID_GROUP = " . $context['gid'], __FILE__, __LINE__);
				db_query("
					DELETE FROM {$db_prefix}cffp_settings_fields
					WHERE ID_GROUP = " . $context['gid'], __FILE__, __LINE__);
				db_query("
					UPDATE {$db_prefix}cffp_settings_groups
					SET
						template_quick_input_regex = '',
						template_quick_input_params = '',
						template_quick_input_help = '',
						template_quick_input_show_modify = 0,
						template_text = '',
						template_message_index_text = ''
					WHERE ID_GROUP = " . $context['gid'], __FILE__, __LINE__);
			}

			if ($context['params']['show_stats'] && !$show_stats)
			{
				db_query("
					DELETE FROM {$db_prefix}cffp_stats
					WHERE ID_GROUP = " . $context['gid'], __FILE__, __LINE__);
			}
			elseif (!$context['params']['show_stats'] && $show_stats)
				cffpUpdateStats($context['gid'], 0, true);

			$id_group = $context['gid'];
		}
		else
		{
			cffp_db_insert('',
				'cffp_settings_groups',
				array(
					'name' => 'string', 'description' => 'string',
					'group_order' => 'int',
					'text_input_before' => 'string', 'text_input' => 'string', 'text_output' => 'string',
					'boards' => 'string', 'child_boards' => $db_type_child_boards,
					'type' => 'string',
					'show_group_name_message_index' => 'int', 'show_group_name_filter' => 'int',
					'show_group_name_search' => 'int',
					'show_fields_name' => 'int', 'show_fields_set' => 'int',
					'show_empty_fields_post' => 'int', 'show_empty_fields_message_index' => 'int',
					'show_poster_data' => 'int', 'show_modified_data' => 'int',
					'show_actions_as' => 'string',
					'show_member_expanded' => 'int', 'show_member_expanded_data' => 'int',
					'show_member_expanded_data_recent' => 'int', 'show_member_expanded_new' => 'int',
					'show_member_recent_at_top' => 'int', 'member_max_height' => 'int',
					'member_one_value' => 'int',
					'show_stats' => 'int',
					'show_stats_post_text' => 'string',
					'show_stats_post_chart_average' => 'string', 'show_stats_post_chart_hbars' => 'string',
					'show_stats_post_chart_vbars' => 'string', 'show_stats_post_summary' => 'string',
					'show_stats_message_index_text' => 'string',
					'show_stats_message_index_chart_average' => 'string', 'show_stats_message_index_chart_hbars' => 'string',
					'show_stats_message_index_chart_vbars' => 'string', 'show_stats_message_index_summary' => 'string',
					'show_stats_order_by' => 'string',
					'show_sticky' => $db_type_show_sticky, 'show_locked' => $db_type_show_locked,
					'show_first_post_topic' => 'int', 'show_replies_topic' => 'int',
					'no_show_with_subject_prefix' => 'string', 'show_in_topic' => 'string',
					'show_filters' => 'string', 'show_filters_operator' => 'string',
					'javascript' => 'string',
					'search_page_name' => 'string', 'search_page_order_by' => 'string',
					'counter_show' => 'int', 'counter_name' => 'string',
					'counter_number_name' => 'string',
					'counter_total_name' => 'string', 'counter_average_name' => 'string',
					'top_poster_show' => 'int', 'top_poster_name' => 'string',
					'profile_list_show' => 'int', 'profile_list_name' => 'string',
					'moderation_list_show' => 'int', 'moderation_list_menu' => 'int',
					'moderation_list_name' => 'string', 'moderation_list_order_by' => 'string',
					'permissions' => 'string', 'active' => 'int',
				),
				array(
					$_POST['name'], $_POST['description'],
					$group_order,
					$_POST['text_input_before'], $_POST['text_input'], $_POST['text_output'],
					$boards, $child_boards,
					$_POST['type'],
					$show_group_name_message_index, $show_group_name_filter,
					$show_group_name_search,
					$show_fields_name, $show_fields_set,
					$show_empty_fields_post, $show_empty_fields_message_index,
					$show_poster_data, $show_modified_data,
					$show_actions_as,
					$show_member_expanded, $show_member_expanded_data,
					$show_member_expanded_data_recent, $show_member_expanded_new,
					$show_member_recent_at_top, $member_max_height,
					$member_one_value,
					$show_stats,
					$show_stats_post_text,
					$show_stats_post_chart_average, $show_stats_post_chart_hbars,
					$show_stats_post_chart_vbars, $show_stats_post_summary,
					$show_stats_message_index_text,
					$show_stats_message_index_chart_average, $show_stats_message_index_chart_hbars,
					$show_stats_message_index_chart_vbars, $show_stats_message_index_summary,
					$show_stats_order_by,
					$show_sticky, $show_locked,
					$show_first_post_topic, $show_replies_topic,
					$no_show_with_subject_prefix, $show_in_topic,
					$show_filters, $show_filters_operator,
					$javascript,
					$search_page_name, $search_page_order_by,
					$counter_show, $counter_name,
					$counter_number_name,
					$counter_total_name, $counter_average_name,
					$top_poster_show, $top_poster_name,
					$profile_list_show, $profile_list_name,
					$moderation_list_show, $moderation_list_menu,
					$moderation_list_name, $moderation_list_order_by,
					$permissions, $active,
				)
			);
			$id_group = db_insert_id();
			if (!empty($id_group) && 'record' == $_POST['type'])
			{
				cffpAddCounterRecordColumns($id_group);
				if ($active)
					cffpUpdateCounterRecordAllMembers($id_group, false);
			}
		}

		// Fix old values with new settings.
		if (!empty($id_group) && cffpIsNotEmptyString($search_page_name))
		{
			$search_page_name_array = explode(',', $search_page_name);
			$search_page_order_by_array = explode(',', $search_page_order_by);
			foreach ($search_page_name_array as $k => $v)
			{
				$request = db_query("
					SELECT ID_GROUP, search_page_name, search_page_order_by
					FROM {$db_prefix}cffp_settings_groups
					WHERE ID_GROUP <> $id_group
						AND type IN ('fields', 'member', 'filters')
						AND (search_page_name = '$v' OR search_page_name LIKE '$v,%' OR search_page_name LIKE '%,$v' OR search_page_name LIKE '%,$v,%')", __FILE__, __LINE__);
				while ($row = mysql_fetch_assoc($request))
				{
					if (strpos($row['search_page_name'], ','))
					{
						$search_page_name_array2 = explode(',', $row['search_page_name']);
						$search_page_order_by_array2 = explode(',', $row['search_page_order_by']);
						foreach ($search_page_name_array2 as $k2 => $v2)
						{
							if ($v2 == $v)
							{
								if ($search_page_order_by_array2[$k2] != $search_page_order_by_array[$k])
								{
									$search_page_order_by_array2[$k2] = $search_page_order_by_array[$k];
									db_query("
										UPDATE {$db_prefix}cffp_settings_groups
										SET search_page_order_by = '" . implode(',', $search_page_order_by_array2) . "'
										WHERE ID_GROUP = " . $row['ID_GROUP'], __FILE__, __LINE__);
								}
								break; // Exit foreach.
							}
						}
					}
					elseif ($row['search_page_order_by'] != $search_page_order_by_array[$k])
					{
						db_query("
							UPDATE {$db_prefix}cffp_settings_groups
							SET search_page_name = '$v', search_page_order_by = '" . $search_page_order_by_array[$k] . "'
							WHERE ID_GROUP = " . $row['ID_GROUP'], __FILE__, __LINE__);
					}
				}
				mysql_free_result($request);
			}
		}
	}
	// Deleting?
	elseif (isset($_GET['delete']) && $context['gid'])
	{
		checkSession('get');

		// Delete statistics of group.
		db_query("
			DELETE FROM {$db_prefix}cffp_stats
			WHERE ID_GROUP = " . $context['gid'], __FILE__, __LINE__);

		// Delete group on posts.
		db_query("
			DELETE FROM {$db_prefix}cffp_fields
			WHERE ID_GROUP = " . $context['gid'], __FILE__, __LINE__);

		// Get columns of group.
		$id_column_array = array();
		$request = db_query("
			SELECT id_column
			FROM {$db_prefix}cffp_settings_fields
			WHERE ID_GROUP = " . $context['gid'], __FILE__, __LINE__);
		while ($row = mysql_fetch_assoc($request))
			$id_column_array[] = $row['id_column'];
		mysql_free_result($request);

		// Delete setting fields of group.
		db_query("
			DELETE FROM {$db_prefix}cffp_settings_fields
			WHERE ID_GROUP = " . $context['gid'], __FILE__, __LINE__);

		// Delete columns of fields on posts.
		foreach ($id_column_array as $id_column)
			cffpRemoveFieldsColumn($context['gid'], $id_column);

		// Delete setting group.
		db_query("
			DELETE FROM {$db_prefix}cffp_settings_groups
			WHERE ID_GROUP = " . $context['gid'], __FILE__, __LINE__);

		// Delete counter record.
		cffpRemoveCounterRecordColumns($context['gid']);
	}

	if (isset($_GET['delete']) || isset($_POST['save']))
	{
		checkSession('get');
		redirectexit('action=featuresettings;sa=cffp' . $context['params_url']);
	}
}

// Admin area settings (edit template of group).
function cffpSettingsEditTemplate()
{
	global $txt, $context, $func, $db_prefix, $sourcedir;

	require_once ($sourcedir . '/Subs-Db-CustomFieldFilterPost-mysql.php');

	loadTemplate('CustomFieldFilterPost');

	// Sort out the context!
	$context['gid'] = isset($_GET['gid']) ? (int) $_GET['gid'] : 0;
	$context['settings_title'] = '';
	$start = isset($_GET['start']) ? (int) $_GET['start'] : 0;
	$sort = isset($_GET['sort']) ? $_GET['sort'] : 'group_order';
	$context['params_url'] = ';start=' . $start . ';sort=' . $sort . (isset($_GET['desc']) ? ';desc' : '');
	$context['admin_tabs']['tabs']['cffp']['is_selected'] = true;
	$context['sub_template'] = 'cffp_settings_edit_template';

	if ($context['gid'])
	{
		$request = db_query("
			SELECT
				name, type,
				template_quick_input_regex, template_quick_input_params,
				template_quick_input_help, template_quick_input_show_modify,
				template_text, template_type,
				template_message_index_text, template_message_index_type
			FROM {$db_prefix}cffp_settings_groups
			WHERE ID_GROUP = " . $context['gid'], __FILE__, __LINE__);
		$group = mysql_fetch_assoc($request);
		mysql_free_result($request);

		$group['type_txt'] = $txt['cffp_grouptype_' . $group['type']];

		ob_start();
		cffpSettingsShowFields($context['gid'], '', 'template');
		$list = ob_get_contents();
		ob_end_clean();

		$context['params'] = array(
			'group' => $group,
			'fields' => $list,
			'template_quick_input_regex' => $func['htmlspecialchars']($group['template_quick_input_regex'], ENT_QUOTES),
			'template_quick_input_params' => $group['template_quick_input_params'],
			'template_quick_input_help' => $group['template_quick_input_help'],
			'template_quick_input_show_modify' => $group['template_quick_input_show_modify'],
			'template_type' => $group['template_type'],
			'template_text' => $func['htmlspecialchars']($group['template_text'], ENT_QUOTES),
			'template_message_index_type' => $group['template_message_index_type'],
			'template_message_index_text' => $func['htmlspecialchars']($group['template_message_index_text'], ENT_QUOTES),
		);

		$context['settings_title'] = empty($context['params']['template_quick_input_regex']) && empty($context['params']['template_text']) && empty($context['params']['template_message_index_text']) ? $txt['cffp_add_templates'] : $txt['cffp_edit_templates'];
	}

	// Others data.
	$template_type_array = array('html', 'bbc', 'php');
	$context['params']['template_type_array'] = array();
	foreach ($template_type_array as $template_type)
		$context['params']['template_type_array'][$template_type] = $txt['cffp_template_type_' . $template_type];

	// Are we saving?
	if (isset($_POST['save']))
	{
		checkSession('get');

		// Validate inputs.
		if ('record' == $context['params']['group']['type'])
		{
			$template_quick_input_regex = $_POST['template_quick_input_regex'];

			$template_quick_input_params = array_map('trim', preg_split('~[\n\r]+~', $_POST['template_quick_input_params']));
			$template_quick_input_params = array_filter($template_quick_input_params, 'cffpIsNotEmptyString');
			$can_be_js = false;
			foreach ($template_quick_input_params as $k => $param)
			{
				if (is_numeric($param))
					$can_be_js = $param ? true : false;
				elseif ($can_be_js)
					$can_be_js = false;
				else
					unset($template_quick_input_params[$k]);
			}
			$template_quick_input_params = implode("\n", $template_quick_input_params);

			$template_quick_input_help = $func['htmlspecialchars']($_POST['template_quick_input_help'], ENT_QUOTES);
			$template_quick_input_show_modify = isset($_POST['template_quick_input_show_modify']) ? 1 : 0;
		}
		else
		{
			$template_quick_input_regex = '';
			$template_quick_input_params = '';
			$template_quick_input_help = '';
			$template_quick_input_show_modify = 0;
		}

		// Do the updates.
		if ($context['gid'])
		{
			db_query("
				UPDATE {$db_prefix}cffp_settings_groups
				SET
					template_quick_input_regex = '$template_quick_input_regex',
					template_quick_input_params = '$template_quick_input_params',
					template_quick_input_help = '$template_quick_input_help',
					template_quick_input_show_modify = $template_quick_input_show_modify,
					template_type = '" . $_POST['template_type'] . "',
					template_text = '" . $_POST['template_text'] . "',
					template_message_index_type = '" . $_POST['template_message_index_type'] . "',
					template_message_index_text = '" . $_POST['template_message_index_text'] . "'
				WHERE ID_GROUP = " . $context['gid'], __FILE__, __LINE__);
		}
	}

	if (isset($_POST['save']))
	{
		checkSession('get');
		redirectexit('action=featuresettings;sa=cffp' . $context['params_url']);
	}
}

// Admin area settings (show).
function cffpSettingsShow()
{
	global $txt, $scripturl, $context, $sourcedir;

	require_once ($sourcedir . '/Subs-CustomFieldFilterPost.php');

	loadTemplate('CustomFieldFilterPost');
	loadTemplate('GenericList-CustomFieldFilterPost');

	$context['sub_template'] = 'cffp_settings_show';
	$context['settings_title'] = $txt['cffp'];

	$base_href = $scripturl . '?action=featuresettings;sa=cffp';

	$get_count = list_cffpGetSettingsGroupsCount();
	$items_per_page = 25;

	$headers = array(
		'group_order' => array(
			'label' => $txt['cffp_order'],
			'style' => 'width: 5%; text-align: left;',
			'sort' => array(
				'default' => 'group_order',
				'reverse' => 'group_order DESC',
			),
		),
		'ID_GROUP' => array(
			'label' => $txt['cffp_id'],
			'style' => 'width: 5%; font-size: smaller;',
		),
		'name' => array(
			'label' => $txt['cffp_name'],
			'style' => 'width: 15%; text-align: left;',
			'sort' => array(
				'default' => 'name',
				'reverse' => 'name DESC',
			),
		),
		'type' => array(
			'label' => $txt['cffp_type'],
			'style' => 'width: 15%; text-align: left;',
			'sort' => array(
				'default' => 'type',
				'reverse' => 'type DESC',
			),
		),
		'active' => array(
			'label' => $txt['cffp_active'],
			'style' => 'width: 5%;',
			'sort' => array(
				'default' => 'active DESC',
				'reverse' => 'active',
			),
		),
		'fields' => array(
			'label' => $txt['cffp_fields'],
		),
		'option_modify' => array(
			'style' => 'width: 10%;',
		),
		'option_delete' => array(
			'style' => 'width: 10%;',
		),
	);

	$sort = isset($_REQUEST['sort']) && isset($headers[$_REQUEST['sort']]) && isset($headers[$_REQUEST['sort']]['sort']) ? $_REQUEST['sort'] : 'group_order';
	$sort_order = $headers[$sort]['sort'][isset($_REQUEST['desc']) ? 'reverse' : 'default'];
	$page_index = constructPageIndex($base_href . ';sort=' . $sort . (isset($_REQUEST['desc']) ? ';desc' : ''), $_REQUEST['start'], $get_count, $items_per_page);
	$start = $_REQUEST['start'];
	$params_url = ';start=' . $start . ';sort=' . $sort . (isset($_REQUEST['desc']) ? ';desc' : '');

	foreach ($headers as $col => $col_header)
	{
		if (isset($headers[$col]['sort']))
		{
			$headers[$col]['href'] = $base_href . ';sort=' . $col;
			if (!isset($_REQUEST['desc']) && $col == $sort)
				$headers[$col]['href'] .= ';desc';
			if ($col == $sort)
				$headers[$col]['sort_image'] = !isset($_REQUEST['desc']) ? 'down' : 'up';
			$headers[$col]['selected'] = $col == $sort;
		}
	}
	$num_columns = count($headers);

	$list = list_cffpGetSettingsGroups($start, $items_per_page, $sort_order);
	$rows = array();
	foreach ($list as $rowData)
	{

		if ('filters' == $rowData['type'])
		{
			ob_start();
			cffpSettingsShowFilters($rowData['show_filters']);
			$list_fields = ob_get_contents();
			ob_end_clean();

			$text_fields = '';
		}
		else
		{
			ob_start();
			cffpSettingsShowFields($rowData['ID_GROUP'], $params_url);
			$list_fields = ob_get_contents();
			ob_end_clean();

			$text_fields = '[<a href="' . $scripturl . '?action=featuresettings;sa=cffpedittemplate;gid=' . $rowData['ID_GROUP'] . $params_url . '">' . (empty($rowData['template_quick_input_regex']) && empty($rowData['template_text']) && empty($rowData['template_message_index_text']) ? $txt['cffp_add_templates'] : $txt['cffp_edit_templates']) . '</a>]';
		}
		$text_fields = '<div style="max-height: 200px; overflow: scroll;">' . $list_fields . '</div>' . $text_fields;

		$textKey = 'cffp_grouptype_' . $rowData['type'];

		$rows[] = array(
			'group_order' => array(
				'value' => $rowData['group_order'],
				'style' => 'text-align: center;',
			),
			'ID_GROUP' => array(
				'value' => $rowData['ID_GROUP'],
				'style' => 'text-align: center;',
				'class' => 'smalltext',
			),
			'name' => array(
				'value' => $rowData['name'] . (cffpIsNotEmptyString($rowData['description']) ? ('<br />/<br /><span class="smalltext">' . nl2br($rowData['description']) . '</span>') : ''),
			),
			'type' => array(
				'value' => isset($txt[$textKey]) ? $txt[$textKey] : $textKey,
			),
			'active' => array(
				'value' => $rowData['active'] ? $txt[163] : $txt[164],
				'style' => 'text-align: center;',
			),
			'fields' => array(
				'value' => $text_fields,
				'style' => 'text-align: right;',
			),
			'option_modify' => array(
				'value' => '<a href="' . $scripturl . '?action=featuresettings;sa=cffpeditgroup;gid=' . $rowData['ID_GROUP'] . $params_url . '">' . $txt[17] . '</a>',
				'style' => 'text-align: center;',
			),
			'option_delete' => array(
				'value' => '<a href="' . $scripturl . '?action=featuresettings;sa=cffpeditgroup;delete=1;gid=' . $rowData['ID_GROUP'] . $params_url . ';sesc=' . $context['session_id'] . '" onclick="return confirm(\'' . addslashes($txt['cffp_edit_delete_group_sure']) . '\');">' . $txt['smf138'] . '</a>',
				'style' => 'text-align: center;',
			),
		);
	}

	$context['cffp_settings_show_groups'] = array(
		'title' => $txt['cffp_group_of_fields'],
		'no_items_label' => $txt['cffp_none_group'],
		'items_per_page' => $items_per_page,
		'page_index' => $page_index,
		'num_columns' => $num_columns,
		'headers' => $headers,
		'rows' => $rows,
		'additional_rows' => array(
			'below_table_data' => array(
				array(
					'value' => '[<a href="' . $scripturl . '?action=featuresettings;sa=cffpeditgroup' . $params_url . '">' . $txt['cffp_add_group'] . '</a>]',
				),
			),
		),
	);
}

// Admin area settings (show fields of a group).
function cffpSettingsShowFields($id_group, $params_url, $type = '')
{
	global $txt, $scripturl, $context, $sourcedir;

	if (empty($id_group))
		return false;

	require_once ($sourcedir . '/Subs-CustomFieldFilterPost.php');

	loadTemplate('GenericList-CustomFieldFilterPost');

	$headers = array(
		array(
			'label' => $txt['cffp_order'],
			'style' => 'width: 5%; font-size: smaller;',
		),
		array(
			'label' => $txt['cffp_id'],
			'style' => 'width: 5%; font-size: smaller;',
		),
		array(
			'label' => $txt['cffp_name'],
			'style' => 'width: ' . (empty($type) ? 15 : 25) . '%; text-align: left; font-size: smaller;',
		),
		array(
			'label' => $txt['cffp_type'],
			'style' => 'width: 15%; text-align: left; font-size: smaller;',
		),
		array(
			'label' => $txt['cffp_more_info'],
			'style' => 'text-align: left; font-size: smaller;',
		),
	);
	if (empty($type))
	{
		$headers[] = array(
			'label' => $txt['cffp_active'],
			'style' => 'width: 5%; font-size: smaller;',
		);
		$headers[] = array(
			'style' => 'width: 8%;',
		);
		$headers[] = array(
			'style' => 'width: 8%;',
		);
	}

	$rows = array();
	if (!empty($type) && isset($context['cffp']['settings_groups'][$id_group]))
		$list = $context['cffp']['settings_groups'][$id_group]['settings_fields'];
	else
		$list = cffpGetListSettingsFields($id_group, !empty($type));
	foreach ($list as $rowData)
	{
		$text_name = $rowData['name'];
		if (cffpIsNotEmptyString($rowData['field_desc']))
			$text_name .= '<br /><span class="smalltext">' . $rowData['field_desc'] . '</span>';
		if (cffpIsNotEmptyString($rowData['description']))
			$text_name .= '<br />/<br /><span class="smalltext">' . nl2br($rowData['description']) . '</span>';

		$textKey = 'cffp_fieldtype_' . $rowData['type'];
		$row = array(
			array(
				'value' => $rowData['field_order'],
				'style' => 'text-align: center;',
				'class' => 'smalltext',
			),
			array(
				'value' => $rowData['ID_FIELD'] . (!empty($type) && $rowData['show_message_index'] ? ' <span class="smalltext">(*)</span>' : ''),
				'style' => 'text-align: center;',
				'class' => 'smalltext',
			),
			array(
				'value' => $text_name,
				'style' => 'text-align: left;',
				'class' => 'smalltext',
			),
			array(
				'value' => isset($txt[$textKey]) ? $txt[$textKey] : $textKey,
				'style' => 'text-align: left;',
				'class' => 'smalltext',
			),
			array(
				'value' => cffpGetMoreInfoField($rowData),
				'style' => 'text-align: left;',
				'class' => 'smalltext',
			),
		);
		if (empty($type))
		{
			$row[] = array(
				'value' => $rowData['active'] ? $txt[163] : $txt[164],
				'style' => 'text-align: center;',
				'class' => 'smalltext',
			);
			$row[] = array(
				'value' => '<a href="' . $scripturl . '?action=featuresettings;sa=cffpeditfield;gid=' . $id_group . ';fid=' . $rowData['ID_FIELD'] . $params_url . '">' . $txt[17] . '</a>',
				'style' => 'text-align: center;',
				'class' => 'smalltext',
			);
			$row[] = array(
				'value' => '<a href="' . $scripturl . '?action=featuresettings;sa=cffpeditfield;delete=1;gid=' . $id_group . ';fid=' . $rowData['ID_FIELD'] . $params_url . ';sesc=' . $context['session_id'] . '" onclick="return confirm(\'' . addslashes($txt['cffp_edit_delete_field_sure']) . '\');">' . $txt['smf138'] . '</a>',
				'style' => 'text-align: center;',
				'class' => 'smalltext',
			);
		}
		$rows[] = $row;
	}

	$additional_rows = array();
	if (empty($type))
	{
		$additional_rows['below_table_data'] = array(
			array(
				'value' => '[<a href="' . $scripturl . '?action=featuresettings;sa=cffpeditfield;gid=' . $id_group . $params_url . '">' . $txt['cffp_add_field'] . '</a>]',
				'class' => 'smalltext',
			),
		);
	}

	$context['cffp_settings_show_fields'] = array(
		'no_items_label' => $txt['cffp_none_field'],
		'num_columns' => count($headers),
		'headers' => $headers,
		'rows' => $rows,
		'additional_rows' => $additional_rows,
	);

	template_cffp_show_list('cffp_settings_show_fields');
}

// Admin area settings (show filters of a group).
function cffpSettingsShowFilters($filters)
{
	global $txt, $context, $modSettings, $sourcedir;

	require_once ($sourcedir . '/Subs-CustomFieldFilterPost.php');

	loadTemplate('GenericList-CustomFieldFilterPost');

	if (!empty($filters))
	{
		$settings_filters = cffpGetSettingsPredefinedFilters();
		$filters = explode(',', $filters);
	}
	else
		$filters = array();

	$headers = array(
		array(
			'label' => $txt['cffp_show_filters'],
			'style' => 'text-align: left; font-size: smaller;',
		),
		array(
		),
	);

	$rows = array();
	foreach ($filters as $filter)
	{
		$note_array = array();
		if ('sticky' == $filter && !$modSettings['enableStickyTopics'])
			$note_array[] = $txt['cffp_show_filters_sticky_disabled'];
		$note = !empty($note_array) ? ' <div class="smalltext">' . implode('<br />', $note_array) . '</div>' : '';

		$rows[] = array(
			array(
				'value' => $settings_filters[$filter]['name'][0] . (!empty($settings_filters[$filter]['field'][0]) ? $note : ''),
				'style' => 'text-align: left;',
				'class' => 'smalltext',
			),
			array(
				'value' => $settings_filters[$filter]['name'][1] . (!empty($settings_filters[$filter]['field'][1]) ? $note : ''),
				'style' => 'text-align: left;',
				'class' => 'smalltext',
			),
		);
	}

	$context['cffp_settings_show_fields'] = array(
		'no_items_label' => $txt['cffp_none_field'],
		'num_columns' => count($headers),
		'headers' => $headers,
		'rows' => $rows,
	);

	template_cffp_show_list('cffp_settings_show_fields');
}

// Edits membergroups of records.
function cffpMembergroupEdit()
{
	global $txt, $context, $func, $db_prefix, $sourcedir;

	require_once ($sourcedir . '/Subs-CustomFieldFilterPost.php');
	require_once ($sourcedir . '/Subs-Db-CustomFieldFilterPost-mysql.php');

	loadTemplate('CustomFieldFilterPost');

	// Sort out the context!
	$context['gid'] = isset($_GET['gid']) ? (int) $_GET['gid'] : 0;
	$context['mgid'] = isset($_GET['mgid']) ? (int) $_GET['mgid'] : 0;
	$context['page_title'] = $context['mgid'] ? $txt['cffp_edit_membergroup_title'] : $txt['cffp_add_membergroup_title'];
	$sort = isset($_GET['sort']) ? $_GET['sort'] : 'minGroups';
	$context['params_url'] = ';gid=' . $context['gid'] . ';sort=' . $sort . (isset($_GET['desc']) ? ';desc' : '');
	$context['admin_tabs']['tabs']['cffp']['is_selected'] = true;
	$context['sub_template'] = 'cffp_membergroup_edit';

	if ($context['gid'])
	{
		if ($context['mgid'])
		{
			$request = db_query("
				SELECT name, description, onlineColor, minGroups, stars, boards, child_boards
				FROM {$db_prefix}cffp_membergroups
				WHERE ID_MEMBERGROUP = " . $context['mgid'] . "
					AND ID_GROUP = " . $context['gid'], __FILE__, __LINE__);
			$row = mysql_fetch_assoc($request);
			mysql_free_result($request);

			if ($row)
			{
				$row['stars'] = explode('#', $row['stars']);

				$context['params'] = array(
					'name' => $row['name'],
					'description' => $row['description'],
					'onlineColor' => $row['onlineColor'],
					'minGroups' => $row['minGroups'],
					'star_count' => (int) $row['stars'][0],
					'star_image' => isset($row['stars'][1]) ? $row['stars'][1] : '',
					'boards' => explode(',', $row['boards']),
					'child_boards' => $row['child_boards'],
				);
			}
		}

		// Setup the default values as needed.
		if (empty($context['params']))
		{
			$context['params'] = array(
				'name' => '',
				'description' => '',
				'onlineColor' => '',
				'minGroups' => 0,
				'star_count' => 1,
				'star_image' => 'star.gif',
				'boards' => array(),
				'child_boards' => 0,
			);
		}

		// Others data.
		loadJumpTo();
		$context['params']['all_boards'] = $context['jump_to'];

		$request = db_query("
			SELECT name, active
			FROM {$db_prefix}cffp_settings_groups
			WHERE ID_GROUP = " . $context['gid'], __FILE__, __LINE__);
		$context['params']['group'] = mysql_fetch_assoc($request);
		mysql_free_result($request);
	}

	// Are we saving?
	if (isset($_POST['save']) && $context['gid'])
	{
		checkSession('get');

		// Everyone needs a name - even the (bracket) unknown...
		if ('' == trim($_POST['name']))
			fatal_lang_error('error_cffp_membergroup_need_name');

		// Validate inputs.
		$_POST['name'] = $func['htmlspecialchars']($_POST['name'], ENT_QUOTES);
		$_POST['description'] = $func['htmlspecialchars']($_POST['description'], ENT_QUOTES);

		$_POST['minGroups'] = (int) $_POST['minGroups'];

		$stars = empty($_POST['star_count']) || 0 > $_POST['star_count'] ? '' : min((int) $_POST['star_count'], 99) . '#' . $_POST['star_image'];

		if (empty($_POST['boards']))
			$boards = '';
		else
			$boards = in_array('all', $_POST['boards']) ? 'all' : implode(',', $_POST['boards']);
		if (is_numeric($_POST['child_boards']))
		{
			$db_type_child_boards = 'int';
			$child_boards = $_POST['child_boards'];
		}
		else
		{
			$db_type_child_boards = 'raw';
			$child_boards = 'NULL';
		}

		// Do the insertion/updates.
		if ($context['mgid'])
		{
			db_query("
				UPDATE {$db_prefix}cffp_membergroups
				SET
					name = '" . $_POST['name'] . "', description = '" . $_POST['description'] . "',
					onlineColor = '" . $_POST['onlineColor'] . "',
					minGroups = " . $_POST['minGroups'] . ", stars = '$stars',
					boards = '$boards', child_boards = $child_boards
				WHERE ID_MEMBERGROUP = " . $context['mgid'] . "
					AND ID_GROUP = " . $context['gid'], __FILE__, __LINE__);
		}
		else
		{
			cffp_db_insert('',
				'cffp_membergroups',
				array(
					'ID_GROUP' => 'int',
					'name' => 'string', 'description' => 'string',
					'onlineColor' => 'string',
					'minGroups' => 'int', 'stars' => 'string',
					'boards' => 'string', 'child_boards' => $db_type_child_boards,
				),
				array(
					$context['gid'],
					$_POST['name'], $_POST['description'],
					$_POST['onlineColor'],
					$_POST['minGroups'], $stars,
					$boards, $child_boards,
				)
			);
		}

		// Update membergroups of record in members.
		if ($context['params']['group']['active'])
			cffpUpdateMembergroupRecord($context['gid'], array(), true, true);
	}
	// Deleting?
	elseif (isset($_GET['delete']) && $context['gid'] && $context['mgid'])
	{
		checkSession('get');

		// Delete membergroup.
		db_query("
			DELETE FROM {$db_prefix}cffp_membergroups
			WHERE ID_MEMBERGROUP = " . $context['mgid'], __FILE__, __LINE__);

		// Update membergroups of record in members.
		if ($context['params']['group']['active'])
			cffpUpdateMembergroupRecord($context['gid'], array(), true, true);
	}

	if (isset($_GET['delete']) || isset($_POST['save']))
	{
		checkSession('get');
		redirectexit('action=membergroups;sa=cffp' . $context['params_url']);
	}
}

// Shows membergroups of records.
function cffpMembergroupsShow()
{
	global $txt, $scripturl, $context, $db_prefix, $settings;

	loadTemplate('CustomFieldFilterPost');
	loadTemplate('GenericList-CustomFieldFilterPost');

	$context['page_title'] = $txt['cffp_membergroups'];
	$context['sub_template'] = 'cffp_membergroups_show';

	$id_group = !empty($_REQUEST['gid']) ? $_REQUEST['gid']: 0;

	$request = db_query("
		SELECT ID_GROUP, name, counter_name, active
		FROM {$db_prefix}cffp_settings_groups
		WHERE type = 'record'
		ORDER BY name ASC", __FILE__, __LINE__);
	$context['list_menu'] = array();
	$records = array();
	while ($row = mysql_fetch_assoc($request))
	{
		if (empty($id_group))
			$id_group = $row['ID_GROUP'];

		if (!cffpIsNotEmptyString($row['counter_name']))
			$row['counter_name'] = $row['name'];

		$context['list_menu'][$row['ID_GROUP']] = array(
			'href' => $scripturl . '?action=membergroups;sa=cffp;gid=' . $row['ID_GROUP'],
			'label' => $row['counter_name'] . ($row['active'] ? '' : ' (' . $txt['cffp_inactive'] . ')'),
			'is_selected' => $id_group == $row['ID_GROUP'],
		);
		$records[$row['ID_GROUP']] = $row;
	}
	mysql_free_result($request);

	if (!empty($context['list_menu']))
	{
		if (!isset($context['list_menu'][$id_group]))
		{
			reset($context['list_menu']);
			list ($id_group, ) = each($context['list_menu']);
			$context['list_menu'][$id_group]['is_selected'] = true;
		}

		$headers = array(
			'name' => array(
				'label' => $txt['cffp_name'],
				'sort' => array(
					'default' => 'name',
					'reverse' => 'name DESC',
				),
			),
			'stars' => array(
				'label' => $txt['membergroups_stars'],
				'sort' => array(
					'default' => 'stars',
					'reverse' => 'stars DESC',
				),
			),
			'num_members' => array(
				'label' => $txt['membergroups_members_top'],
				'sort' => array(
					'default' => '1 DESC',
					'reverse' => '1',
				),
			),
			'minGroups' => array(
				'label' => $txt['cffp_min_groups'],
				'sort' => array(
					'default' => 'minGroups',
					'reverse' => 'minGroups DESC',
				),
			),
			'option_modify' => array(
				'style' => 'width: 10%;',
			),
			'option_delete' => array(
				'style' => 'width: 10%;',
			),
		);

		$base_href = $scripturl . '?action=membergroups;sa=cffp;gid=' . $id_group;
		$sort = isset($_REQUEST['sort']) && isset($headers[$_REQUEST['sort']]) && isset($headers[$_REQUEST['sort']]['sort']) ? $_REQUEST['sort'] : 'minGroups';
		$sort_order = $headers[$sort]['sort'][isset($_REQUEST['desc']) ? 'reverse' : 'default'];
		$params_url = ';gid=' . $id_group . ';sort=' . $sort . (isset($_REQUEST['desc']) ? ';desc' : '');

		foreach ($headers as $col => $col_header)
		{
			if (isset($headers[$col]['sort']))
			{
				$headers[$col]['href'] = $base_href . ';sort=' . $col;
				if (!isset($_REQUEST['desc']) && $col == $sort)
					$headers[$col]['href'] .= ';desc';
				if ($col == $sort)
					$headers[$col]['sort_image'] = !isset($_REQUEST['desc']) ? 'down' : 'up';
				$headers[$col]['selected'] = $col == $sort;
			}
		}
		$num_columns = count($headers);

		$list = list_cffpGetMembergroups($sort_order, $id_group, $records[$id_group]['active']);
		$rows = array();
		foreach ($list as $rowData)
		{
			$colorStyle = empty($rowData['onlineColor']) ? '' : ' style="color: ' . $rowData['onlineColor'] . ';"';
			$text_name = '<a href="' . $scripturl . '?action=membergroups;sa=cffpmembers;id_membergroup=' . $rowData['ID_MEMBERGROUP'] . '"' . $colorStyle . '>' . $rowData['name'] . '</a>';

			$stars = explode('#', $rowData['stars']);
			if (empty($stars[0]) || empty($stars[1]))
				$text_stars = '';
			else
			{
				$star_image = '<img src="' . $settings['images_url'] . '/' . str_replace('$language', $context['user']['language'], $stars[1]) . '" alt="*" border="0" />';
				$text_stars = str_repeat($star_image, $stars[0]);
			}

			$rows[] = array(
				'name' => array(
					'value' => $text_name,
				),
				'stars' => array(
					'value' => $text_stars,
				),
				'num_members' => array(
					'value' => $rowData['num_members'],
					'style' => 'text-align: center',
				),
				'minGroups' => array(
					'value' => $rowData['minGroups'],
					'style' => 'text-align: center',
				),
				'option_modify' => array(
					'value' => '<a href="' . $scripturl . '?action=membergroups;sa=cffpedit;mgid=' . $rowData['ID_MEMBERGROUP'] . $params_url . '">' . $txt[17] . '</a>',
					'style' => 'text-align: center;',
				),
				'option_delete' => array(
					'value' => '<a href="' . $scripturl . '?action=membergroups;sa=cffpedit;delete=1;mgid=' . $rowData['ID_MEMBERGROUP'] . $params_url . ';sesc=' . $context['session_id'] . '" onclick="return confirm(\'' . addslashes($txt['cffp_edit_delete_membergroup_sure']) . '\');">' . $txt['smf138'] . '</a>',
					'style' => 'text-align: center;',
				),
			);
		}

		$context['cffp_membergroups_show'] = array(
			'no_items_label' => sprintf($txt['cffp_none_membergroups'], $records[$id_group]['name']),
			'num_columns' => $num_columns,
			'headers' => $headers,
			'rows' => $rows,
			'additional_rows' => array(
				'below_table_data' => array(
					array(
						'value' => '[<a href="' . $scripturl . '?action=membergroups;sa=cffpedit' . $params_url . '">' . $txt['cffp_add_membergroup'] . '</a>]',
					),
				),
			),
		);
	}
}

// Display list of members of a membergroup of records.
function cffpMembergroupMembers()
{
	global $txt, $scripturl, $context, $db_prefix, $settings, $modSettings;

	loadTemplate('CustomFieldFilterPost');
	loadTemplate('GenericList-CustomFieldFilterPost');

	$context['page_title'] = $txt['membergroups_members_title'];
	$context['admin_tabs']['tabs']['cffp']['is_selected'] = true;
	$context['sub_template'] = 'cffp_membergroup_members';

	$id_membergroup = !empty($_REQUEST['id_membergroup']) ? $_REQUEST['id_membergroup']: 0;

	// Get the basic group data.
	$request = db_query("
		SELECT ID_MEMBERGROUP, cffpmg.ID_GROUP, cffpmg.name, cffpmg.description, onlineColor, minGroups, stars, cffpsg.name AS group_name, cffpsg.active
		FROM {$db_prefix}cffp_membergroups AS cffpmg
			INNER JOIN {$db_prefix}cffp_settings_groups AS cffpsg ON (cffpsg.ID_GROUP = cffpmg.ID_GROUP)
		WHERE ID_MEMBERGROUP = $id_membergroup", __FILE__, __LINE__);
	$row = mysql_fetch_assoc($request);
	mysql_free_result($request);

	if (!$row)
		fatal_lang_error('membergroup_does_not_exist');

	$context['membergroup'] = $row;
	$context['membergroup']['num_members'] = 0;
	$context['membergroup']['stars'] = explode('#', $context['membergroup']['stars']);
	$context['membergroup']['stars'] = !empty($context['membergroup']['stars'][0]) && !empty($context['membergroup']['stars'][1]) ? str_repeat('<img src="' . $settings['images_url'] . '/' . str_replace('$language', $context['user']['language'], $context['membergroup']['stars'][1]) . '" alt="*" border="0" />', $context['membergroup']['stars'][0]) : '';

	$context['page_title'] .= ': ' . $context['membergroup']['name'];

	// Get the amount of members.
	if ($context['membergroup']['active'])
	{
		$request = db_query("
			SELECT COUNT(*)
			FROM {$db_prefix}members
			WHERE cffp" . $context['membergroup']['ID_GROUP'] . "_ID_MEMBERGROUP = $id_membergroup", __FILE__, __LINE__);
		list ($context['membergroup']['num_members']) = mysql_fetch_row($request);
		mysql_free_result($request);

		// List of members.
		$base_href = $scripturl . '?action=membergroups;sa=cffpmembers;id_membergroup=' . $id_membergroup;

		$get_count = list_cffpGetMembergroupMembersCount($context['membergroup']['ID_GROUP'], $id_membergroup);
		$items_per_page = $modSettings['defaultMaxMembers'];

		$headers = array(
			'realName' => array(
				'label' => $txt[68],
				'sort' => array(
					'default' => 'realName',
					'reverse' => 'realName DESC',
				),
			),
			'emailAddress' => array(
				'label' => $txt[69],
				'sort' => array(
					'default' => 'emailAddress',
					'reverse' => 'emailAddress DESC',
				)
			),
			'lastLogin' => array(
				'label' => $txt['attachment_manager_last_active'],
				'sort' => array(
					'default' => 'lastLogin',
					'reverse' => 'lastLogin DESC',
				),
			),
			'dateRegistered' => array(
				'label' => $txt[233],
				'sort' => array(
					'default' => 'dateRegistered',
					'reverse' => 'dateRegistered DESC',
				),
			),
			'groups' => array(
				'label' => $txt['cffp_records'],
				'sort' => array(
					'default' => 'groups',
					'reverse' => 'groups DESC',
				),
			),
		);

		$sort = isset($_REQUEST['sort']) && isset($headers[$_REQUEST['sort']]) && isset($headers[$_REQUEST['sort']]['sort']) ? $_REQUEST['sort'] : 'realName';
		$sort_order = $headers[$sort]['sort'][isset($_REQUEST['desc']) ? 'reverse' : 'default'];
		$page_index = constructPageIndex($base_href . ';sort=' . $sort . (isset($_REQUEST['desc']) ? ';desc' : ''), $_REQUEST['start'], $get_count, $items_per_page);
		$start = $_REQUEST['start'];
		$params_url = ';id_membergroup=' . $id_membergroup . ';start=' . $start . ';sort=' . $sort . (isset($_REQUEST['desc']) ? ';desc' : '');

		foreach ($headers as $col => $col_header)
		{
			$headers[$col]['href'] = $base_href . ';sort=' . $col;
			if (!isset($_REQUEST['desc']) && $col == $sort)
				$headers[$col]['href'] .= ';desc';
			if ($col == $sort)
				$headers[$col]['sort_image'] = !isset($_REQUEST['desc']) ? 'down' : 'up';
			$headers[$col]['selected'] = $col == $sort;
		}
		$num_columns = count($headers);

		$list = list_cffpGetMembergroupMembers($start, $items_per_page, $sort_order, $context['membergroup']['ID_GROUP'], $id_membergroup);
		$rows = array();
		foreach ($list as $rowData)
		{
			$last_online = empty($rowData['lastLogin']) ? $txt['never'] : timeformat($rowData['lastLogin']);

			// Italicize the online note if they aren't activated.
			if ($rowData['is_activated'] % 10 != 1)
				$last_online = '<i title="' . $txt['not_activated'] . '">' . $last_online . '</i>';

			$rows[] = array(
				'realName' => array(
					'value' => '<a href="' . $scripturl . '?action=profile;u=' . $rowData['ID_MEMBER'] . '">' . $rowData['realName'] . '</a>',
				),
				'emailAddress' => array(
					'value' => '<a href="mailto:' . $rowData['emailAddress'] . '">' . $rowData['emailAddress'] . '</a>',
				),
				'lastLogin' => array(
					'value' => $last_online,
				),
				'dateRegistered' => array(
					'value' => timeformat($rowData['dateRegistered']),
				),
				'groups' => array(
					'value' => $rowData['groups'],
				),
			);
		}

		$context['cffp_membergroup_members'] = array(
			'title' => $txt['cffp_membergroups_members_group'],
			'no_items_label' => $txt['membergroups_members_no_members'],
			'items_per_page' => $items_per_page,
			'page_index' => $page_index,
			'num_columns' => $num_columns,
			'headers' => $headers,
			'rows' => $rows,
		);
	}
}

// Gets list of settings groups.
function list_cffpGetSettingsGroups($start, $items_per_page, $sort)
{
	global $db_prefix;

	if (!is_numeric($start) || empty($items_per_page) || empty($sort))
		return false;

	$request = db_query("
		SELECT
			ID_GROUP, name, description, group_order, type, show_filters, active,
			template_quick_input_regex, template_text, template_message_index_text
		FROM {$db_prefix}cffp_settings_groups
		ORDER BY $sort
		LIMIT $start, $items_per_page", __FILE__, __LINE__);
	$list = array();
	while ($row = mysql_fetch_assoc($request))
		$list[] = $row;
	mysql_free_result($request);

	return $list;
}

// Gets size of list of settings groups.
function list_cffpGetSettingsGroupsCount()
{
	global $db_prefix;

	$request = db_query("
		SELECT COUNT(*)
		FROM {$db_prefix}cffp_settings_groups", __FILE__, __LINE__);

	list ($size) = mysql_fetch_row($request);
	mysql_free_result($request);

	return $size;
}

// Gets list of member groups for a record.
function list_cffpGetMembergroups($sort, $id_group, $active)
{
	global $db_prefix;

	// Get the basic group data.
	$request = db_query("
		SELECT ID_MEMBERGROUP, name, description, onlineColor, minGroups, stars, 0 AS num_members
		FROM {$db_prefix}cffp_membergroups
		WHERE id_group = $id_group
		ORDER BY $sort", __FILE__, __LINE__);
	$groups = array();
	while ($row = mysql_fetch_assoc($request))
		$groups[$row['ID_MEMBERGROUP']] = $row;
	mysql_free_result($request);

	// If we found any membergroups, get the amount of members in them.
	if (!empty($groups) && $active)
	{
		$request = db_query("
			SELECT cffp{$id_group}_ID_MEMBERGROUP AS ID_MEMBERGROUP, COUNT(*) AS num_members
			FROM {$db_prefix}members
			WHERE cffp{$id_group}_ID_MEMBERGROUP <> 0
			GROUP BY cffp{$id_group}_ID_MEMBERGROUP", __FILE__, __LINE__);
		while ($row = mysql_fetch_assoc($request))
			$groups[$row['ID_MEMBERGROUP']]['num_members'] += $row['num_members'];
		mysql_free_result($request);
	}

	// Apply manual sorting if the 'number of members' column is selected.
	$sort_array = explode(' ', $sort);
	if ('1' == $sort_array[0])
	{
		$sort_array[0] = 'num_members';
		if (1 == count($sort_array))
			$sort_array[] = 'ASC';
		$flag_order = 'DESC' == strtoupper($sort_array[1]) ? SORT_DESC : SORT_ASC;
		$groups = cffp_array_column_sort($groups, $sort_array[0], $flag_order, SORT_REGULAR);
	}

	return $groups;
}

// Gets list of members of a membergroup.
function list_cffpGetMembergroupMembers($start, $items_per_page, $sort, $id_group, $id_membergroup)
{
	global $db_prefix;

	if (!is_numeric($start) || empty($items_per_page) || empty($sort) || empty($id_group) || empty($id_membergroup))
		return false;

	$request = db_query("
		SELECT ID_MEMBER, realName, emailAddress, dateRegistered, lastLogin,
			is_activated, cffp{$id_group}_groups AS groups
		FROM {$db_prefix}members
		WHERE cffp{$id_group}_ID_MEMBERGROUP = $id_membergroup
		ORDER BY $sort
		LIMIT $start, $items_per_page", __FILE__, __LINE__);
	$list = array();
	while ($row = mysql_fetch_assoc($request))
		$list[] = $row;
	mysql_free_result($request);

	return $list;
}

// Gets size of list of members of a membergroup.
function list_cffpGetMembergroupMembersCount($id_group, $id_membergroup)
{
	global $db_prefix;

	if (empty($id_group) || empty($id_membergroup))
		return false;

	$request = db_query("
		SELECT COUNT(*)
		FROM {$db_prefix}members
		WHERE cffp{$id_group}_ID_MEMBERGROUP = $id_membergroup", __FILE__, __LINE__);

	list ($size) = mysql_fetch_row($request);
	mysql_free_result($request);

	return $size;
}

/**
 * array_column_sort
 *
 * function to sort an "arrow of rows" by its columns
 * exracts the columns to be sorted and then
 * uses eval to flexibly apply the standard
 * array_multisort function
 *
 * uses a temporary copy of the array whith "_" prefixed to the keys
 * this makes sure that array_multisort is working with an associative
 * array with string type keys, which in turn ensures that the keys
 * will be preserved.
 *
 * TODO: find a way of modifying the keys of $array directly, without using
 * a copy of the array.
 *
 * flexible syntax:
 * $new_array = array_column_sort($array [, 'col1' [, SORT_FLAG [, SORT_FLAG]]]...);
 *
 * original code credited to Ichier (www.ichier.de) here:
 * http://uk.php.net/manual/en/function.array-multisort.php
 *
 * prefixing array indeces with "_" idea credit to steve at mg-rover dot org, also here:
 * http://uk.php.net/manual/en/function.array-multisort.php
 *
 */
function cffp_array_column_sort() {
	$args = func_get_args();
	$array = array_shift($args);

	// make a temporary copy of array for which will fix the
	// keys to be strings, so that array_multisort() doesn't
	// destroy them
	$array_mod = array();
	foreach ($array as $key => $value) {
		$array_mod['_' . $key] = $value;
	}

	$i = 0;
	$multi_sort_line = "return array_multisort(";
	foreach ($args as $arg) {
		$i++;
		if (is_string($arg)) {
			foreach ($array_mod as $row_key => $row) {
				$sort_array[$i][] = $row[$arg];
			}
		} else {
			$sort_array[$i] = $arg;
		}
		$multi_sort_line .= "\$sort_array[" . $i . "], ";
	}
	$multi_sort_line .= "\$array_mod);";

	eval($multi_sort_line);

	// now copy $array_mod back into $array, stripping off the "_"
	// that we added earlier.
	$array = array();
	foreach ($array_mod as $key => $value) {
		$array[substr($key, 1)] = $value;
	}
	return $array;
}

?>