/**
 * Custom fields and filters of post
 *
 * @package CustomFieldFilterPost
 * @version 2.0
 * @author davidhs
 * @copyright 2014-2018 by davidhs
 * @license Creative Commons Attribution 3.0 Unported License https://creativecommons.org/licenses/by/3.0/
 */

// Expand/collapse a block according to an image.
function cffpExpandCollapseWithImage(images_url, img_id, block_id, fieldset_id)
{
	var img_obj = document.getElementById(img_id);
	var block_obj = document.getElementById(block_id);
	var fieldset_obj = null;
	if ('' != fieldset_id)
		fieldset_obj = document.getElementById(fieldset_id);

	if ('none' == block_obj.style.display)
	{
		img_obj.src = images_url + '/collapse.gif';
		block_obj.style.display = '';
		if (fieldset_obj)
		{
			if (is_ie)
				fieldset_obj.style.border = '1px solid #d0d0bf';
			else
				fieldset_obj.style.border = '';
		}
	}
	else
	{
		img_obj.src = images_url + '/expand.gif';
		block_obj.style.display = 'none';
		if (fieldset_obj)
			fieldset_obj.style.border = 'none';
	}
}

// Expand/collapse a block according to a checkbox button.
function cffpExpandCollapseWithCheckbox(checkbox_obj, block_id)
{
	var block_obj = document.getElementById(block_id);

	block_obj.style.display = checkbox_obj.checked ? '' : 'none';
}

// Change value of input button when options are selected on select button.
function cffpOnchangeCombobox(input_obj, select_obj)
{
	var value;
	var i;

	if (select_obj.multiple)
	{
		value = '';
		for (i = 0; i < select_obj.length; i++)
			if (select_obj.options[i].selected)
			{
				if ('' != value)
					value += ',';
				value += select_obj.options[i].value;
			}
	}
	else
		value = select_obj.value;

	input_obj.value = value;
}

// Submit formulary of group
function cffpSubmitGroup(form_obj, cffp_action, start, id_msg, id_cffp, id_group, board, scripturl, session_var, session_id)
{
	var params;

	if ('add' == cffp_action)
		params = ';add=1';
	else if ('modify' == cffp_action)
		params = ';id_cffp=' + id_cffp;
	else if ('remove' == cffp_action)
		params = ';delete=1;id_cffp=' + id_cffp;
	else if ('move_to_first_post' == cffp_action)
		params = ';move=1;id_cffp=' + id_cffp;
	else if ('move_to_post' == cffp_action)
		params = ';move_to_post=1;id_cffp=' + id_cffp;
	else
		params = '';

	form_obj.onSubmit = '';
	form_obj.action = scripturl + '?action=cffpeditgroup' + params + ';start=' + start + ';id_msg=' + id_msg + ';id_group=' + id_group + ';board=' + board + ';' + session_var + '=' + session_id;
	form_obj.submit();

	return false;
}

// Applies quick input field in group fields.
function cffpApplyQuickInputField(id_group, regex, field_obj, params_array)
{
	var quick_input_field_obj = document.getElementById('cffp_' + id_group + '_quick_input_field');
	var field_value = quick_input_field_obj.value;
	var match = field_value.match(regex);
	var i, k, dim, js, value;
	var id_field = 0;

	if (null == match)
		return false;

	k = 0;
	for (i = 0, dim = params_array.length; i < dim; i++)
	{
		if ('number' == typeof params_array[i])
		{
			if (id_field && 'undefined' != typeof field_obj[id_field] && null != field_obj[id_field] && 'undefined' != typeof match[k])
				field_obj[id_field].value = cffpEval(field_obj, value, js);

			k++;
			id_field = params_array[i];
			if (id_field && 'undefined' != typeof field_obj[id_field] && null != field_obj[id_field] && 'undefined' != typeof match[k])
			{
				value = match[k];
				js = 'value = value;';
			}
		}
		else
			js = params_array[i];
	}
	if (id_field && 'undefined' != typeof field_obj[id_field] && null != field_obj[id_field] && 'undefined' != typeof match[k])
		field_obj[id_field].value = cffpEval(field_obj, value, js);

	return true;
}

// Evaluates JavaScript code that transforms a value.
function cffpEval(field_obj, value, cffp_eval)
{
	eval(cffp_eval);

	return value;
}
