<?php

/**
 * Custom fields and filters of post
 *
 * @package CustomFieldFilterPost
 * @version 2.0
 * @author davidhs
 * @copyright 2014-2018 by davidhs
 * @license Creative Commons Attribution 3.0 Unported License https://creativecommons.org/licenses/by/3.0/
 */

if (file_exists(dirname(__FILE__) . '/SSI.php') && !defined('SMF'))
	require_once (dirname(__FILE__) . '/SSI.php');
elseif (!defined('SMF'))
	exit('<b>Error:</b> Cannot install - please verify you put this in the same place as SMF\'s index.php.');

// Functions declared in Sources/Subs-Db-CustomFieldFilterPost-mysql.php.
if (!function_exists('cffp_db_create_table'))
{
	// Create a table.
	function cffp_db_create_table($table_name, $columns, $indexes = array(), $no_prefix = true, $if_exists = 'ignore')
	{
		global $db_prefix, $db_character_set;

		// Strip out the table name, we might not need it in some cases.
		$real_prefix = preg_match('~^(`?)(.+?)\\1\\.(.*?)$~', $db_prefix, $match) === 1 ? $match[3] : $db_prefix;

		// With or without the database name, the fullname looks like this.
		$full_table_name = $no_prefix ? $real_prefix . $table_name : $table_name;
		if ($no_prefix)
			$table_name = $db_prefix . $table_name;

		// Slightly easier on MySQL than the others...
		$tables = cffp_db_list_tables();
		if (in_array($full_table_name, $tables))
		{
			// This is a sad day... drop the table? If not, return false (error) by default.
			if ($if_exists == 'overwrite')
				cffp_db_drop_table($table_name);
			else
				return $if_exists == 'ignore';
		}

		// Righty - let's do the damn thing!
		$table_query = 'CREATE TABLE ' . $table_name . "\n" . '(';
		foreach ($columns as $column)
		{
			// Auto increment is easy here!
			if (!empty($column['auto']))
			{
				$default = 'auto_increment';
			}
			elseif (isset($column['default']) && $column['default'] !== null)
				$default = 'default \'' . addslashes($column['default']) . '\'';
			else
				$default = '';

			// Sort out the size... and stuff...
			$column['size'] = isset($column['size']) && is_numeric($column['size']) ? $column['size'] : null;
			list ($type, $size) = array($column['type'], $column['size']);

			// Allow unsigned integers (mysql only)
			$unsigned = in_array($type, array('int', 'tinyint', 'smallint', 'mediumint', 'bigint')) && !empty($column['unsigned']) ? 'unsigned ' : '';

			if ($size !== null)
				$type = $type . '(' . $size . ')';

			// Now just put it together!
			$table_query .= "\n\t`" .$column['name'] . '` ' . $type . ' ' . (!empty($unsigned) ? $unsigned : '') . (!empty($column['null']) ? '' : 'NOT NULL') . ' ' . $default . ',';
		}

		// Loop through the indexes next...
		foreach ($indexes as $index)
		{
			$columns = implode(',', $index['columns']);

			// Is it the primary?
			if (isset($index['type']) && $index['type'] == 'primary')
				$table_query .= "\n\t" . 'PRIMARY KEY (' . implode(',', $index['columns']) . '),';
			else
			{
				if (empty($index['name']))
					$index['name'] = implode('_', $index['columns']);
				$table_query .= "\n\t" . (isset($index['type']) && $index['type'] == 'unique' ? 'UNIQUE' : 'KEY') . ' ' . $index['name'] . ' (' . $columns . '),';
			}
		}

		// No trailing commas!
		if (substr($table_query, -1) == ',')
			$table_query = substr($table_query, 0, -1);

		$table_query .= ') ENGINE=MyISAM';
		if (!empty($db_character_set) && $db_character_set == 'utf8')
			$table_query .= ' DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci';

		// Create the table!
		db_query($table_query, __FILE__, __LINE__);
	}

	// Drop a table.
	function cffp_db_drop_table($table_name, $no_prefix = true)
	{
		global $db_prefix;

		// After stripping away the database name, this is what's left.
		$real_prefix = preg_match('~^(`?)(.+?)\\1\\.(.*?)$~', $db_prefix, $match) === 1 ? $match[3] : $db_prefix;

		// Get some aliases.
		$full_table_name = $no_prefix ? $real_prefix . $table_name : $table_name;
		if ($no_prefix)
			$table_name = $db_prefix . $table_name;

		// Does it exist?
		if (in_array($full_table_name, cffp_db_list_tables()))
		{
			db_query("DROP TABLE $table_name", __FILE__, __LINE__);

			return true;
		}

		// Otherwise do 'nout.
		return false;
	}

	// Add a column.
	function cffp_db_add_column($table_name, $column_info, $no_prefix = true, $if_exists = 'update')
	{
		global $db_prefix;

		if ($no_prefix)
			$table_name = $db_prefix . $table_name;

		// Does it exist - if so don't add it again!
		$columns = cffp_db_list_columns($table_name, false, false);
		foreach ($columns as $column)
			if ($column == $column_info['name'])
			{
				// If we're going to overwrite then use change column.
				if ($if_exists == 'update')
					return cffp_db_change_column($table_name, $column_info['name'], $column_info, false);
				else
					return false;
			}

		// Get the specifics...
		$column_info['size'] = isset($column_info['size']) && is_numeric($column_info['size']) ? $column_info['size'] : null;
		list ($type, $size) = array($column_info['type'], $column_info['size']);

		// Allow unsigned integers (mysql only).
		$unsigned = in_array($type, array('int', 'tinyint', 'smallint', 'mediumint', 'bigint')) && !empty($column_info['unsigned']) ? 'unsigned ' : '';

		if ($size !== null)
			$type = $type . '(' . $size . ')';

		// Now add the thing!
		db_query("
			ALTER TABLE $table_name
			ADD `" . $column_info['name'] . "` $type " . (!empty($unsigned) ? $unsigned : '') . (empty($column_info['null']) ? 'NOT NULL' : '') . ' ' .
				(!isset($column_info['default']) ? '' : 'default \'' . addslashes($column_info['default']) . '\'') . ' ' .
				(empty($column_info['auto']) ? '' : 'auto_increment primary key'), __FILE__, __LINE__);

		return true;
	}

	// Change a column.
	function cffp_db_change_column($table_name, $old_column, $column_info, $no_prefix = true)
	{
		global $db_prefix;

		if ($no_prefix)
			$table_name = $db_prefix . $table_name;

		// Check it does exist!
		$columns = cffp_db_list_columns($table_name, true, false);
		$old_info = null;
		foreach ($columns as $column)
			if ($column['name'] == $old_column)
				$old_info = $column;

		// Nothing?
		if ($old_info == null)
			return false;

		// Get the right bits.
		if (!isset($column_info['name']))
			$column_info['name'] = $old_column;
		if (!isset($column_info['default']))
			$column_info['default'] = $old_info['default'];
		if (!isset($column_info['null']))
			$column_info['null'] = $old_info['null'];
		if (!isset($column_info['auto']))
			$column_info['auto'] = $old_info['auto'];
		if (!isset($column_info['type']))
			$column_info['type'] = $old_info['type'];
		if (!isset($column_info['size']) || !is_numeric($column_info['size']))
			$column_info['size'] = $old_info['size'];
		if (!isset($column_info['unsigned']) || !in_array($column_info['type'], array('int', 'tinyint', 'smallint', 'mediumint', 'bigint')))
			$column_info['unsigned'] = '';

		list ($type, $size) = array($column_info['type'], $column_info['size']);

		// Allow for unsigned integers (mysql only).
		$unsigned = in_array($type, array('int', 'tinyint', 'smallint', 'mediumint', 'bigint')) && !empty($column_info['unsigned']) ? 'unsigned ' : '';

		if ($size !== null)
			$type = $type . '(' . $size . ')';

		db_query("
			ALTER TABLE $table_name
			CHANGE COLUMN `$old_column` `" . $column_info['name'] . "` $type " . (!empty($unsigned) ? $unsigned : '') . (empty($column_info['null']) ? 'NOT NULL' : '') . ' ' .
				(!isset($column_info['default']) ? '' : 'default \'' . addslashes($column_info['default']) . '\'') . ' ' .
				(empty($column_info['auto']) ? '' : 'auto_increment'), __FILE__, __LINE__);
	}

	// Add an index.
	function cffp_db_add_index($table_name, $index_info, $no_prefix = true, $if_exists = 'update')
	{
		global $db_prefix;

		if ($no_prefix)
			$table_name = $db_prefix . $table_name;

		// No columns = no index.
		if (empty($index_info['columns']))
			return false;
		$columns = implode(',', $index_info['columns']);

		// No name - make it up!
		if (empty($index_info['name']))
		{
			// No need for primary.
			if (isset($index_info['type']) && $index_info['type'] == 'primary')
				$index_info['name'] = '';
			else
				$index_info['name'] = implode('_', $index_info['columns']);
		}
		else
			$index_info['name'] = $index_info['name'];

		// Let's get all our indexes.
		$indexes = cffp_db_list_indexes($table_name, true, false);
		// Do we already have it?
		foreach ($indexes as $index)
		{
			if ($index['name'] == $index_info['name'] || ($index['type'] == 'primary' && isset($index_info['type']) && $index_info['type'] == 'primary'))
			{
				// If we want to overwrite simply remove the current one then continue.
				if ($if_exists != 'update' || $index['type'] == 'primary')
					return false;
				else
					cffp_db_remove_index($table_name, $index_info['name'], false);
			}
		}

		// If we're here we know we don't have the index - so just add it.
		if (!empty($index_info['type']) && $index_info['type'] == 'primary')
		{
			db_query("
				ALTER TABLE $table_name
				ADD PRIMARY KEY ($columns)", __FILE__, __LINE__);
		}
		else
		{
			db_query("
				ALTER TABLE $table_name
				ADD " . (isset($index_info['type']) && $index_info['type'] == 'unique' ? 'UNIQUE' : 'INDEX') . ' ' . $index_info['name'] . " ($columns)", __FILE__, __LINE__);
		}
	}

	// Remove an index.
	function cffp_db_remove_index($table_name, $index_name, $no_prefix = true)
	{
		global $db_prefix;

		if ($no_prefix)
			$table_name = $db_prefix . $table_name;

		// Better exist!
		$indexes = cffp_db_list_indexes($table_name, true, false);

		foreach ($indexes as $index)
		{
			// If the name is primary we want the primary key!
			if ($index['type'] == 'primary' && $index_name == 'primary')
			{
				// Dropping primary key?
				db_query("
					ALTER TABLE $table_name
					DROP PRIMARY KEY", __FILE__, __LINE__);

				return true;
			}
			if ($index['name'] == $index_name)
			{
				// Drop the bugger...
				db_query("
					ALTER TABLE $table_name
					DROP INDEX $index_name", __FILE__, __LINE__);

				return true;
			}
		}

		// Not to be found ;(
		return false;
	}

	// Return column information for a table.
	function cffp_db_list_columns($table_name, $detail = false, $no_prefix = true)
	{
		global $db_prefix;

		if ($no_prefix)
			$table_name = $db_prefix . $table_name;

		$request = db_query("
			SHOW FIELDS
			FROM " . (substr($table_name, 0, 1) == '`' ? $table_name : "`$table_name`"), __FILE__, __LINE__);
		$columns = array();
		while ($row = mysql_fetch_assoc($request))
		{
			if (!$detail)
			{
				$columns[] = $row['Field'];
			}
			else
			{
				// Is there an auto_increment?
				$auto = strpos($row['Extra'], 'auto_increment') !== false ? true : false;

				// Can we split out the size?
				if (preg_match('~(.+?)\s*\((\d+)\)(?:(?:\s*)?(unsigned))?~i', $row['Type'], $matches) === 1)
				{
					$type = $matches[1];
					$size = $matches[2];
					if (!empty($matches[3]) && $matches[3] == 'unsigned')
						$unsigned = true;
				}
				else
				{
					$type = $row['Type'];
					$size = null;
				}

				$columns[$row['Field']] = array(
					'name' => $row['Field'],
					'null' => $row['Null'] != 'YES' ? false : true,
					'default' => isset($row['Default']) ? $row['Default'] : null,
					'type' => $type,
					'size' => $size,
					'auto' => $auto,
				);

				if (isset($unsigned))
				{
					$columns[$row['Field']]['unsigned'] = $unsigned;
					unset($unsigned);
				}
			}
		}
		mysql_free_result($request);

		return $columns;
	}

	// What about some index information?
	function cffp_db_list_indexes($table_name, $detail = false, $no_prefix = true)
	{
		global $db_prefix;

		if ($no_prefix)
			$table_name = $db_prefix . $table_name;

		$request = db_query("
			SHOW KEYS
			FROM " . (substr($table_name, 0, 1) == '`' ? $table_name : "`$table_name`"), __FILE__, __LINE__);
		$indexes = array();
		while ($row = mysql_fetch_assoc($request))
		{
			if (!$detail)
				$indexes[] = $row['Key_name'];
			else
			{
				// What is the type?
				if ($row['Key_name'] == 'PRIMARY')
					$type = 'primary';
				elseif (empty($row['Non_unique']))
					$type = 'unique';
				elseif (isset($row['Index_type']) && $row['Index_type'] == 'FULLTEXT')
					$type = 'fulltext';
				else
					$type = 'index';

				// This is the first column we've seen?
				if (empty($indexes[$row['Key_name']]))
				{
					$indexes[$row['Key_name']] = array(
						'name' => $row['Key_name'],
						'type' => $type,
						'columns' => array(),
					);
				}

				// Is it a partial index?
				if (!empty($row['Sub_part']))
					$indexes[$row['Key_name']]['columns'][] = $row['Column_name'] . '(' . $row['Sub_part'] . ')';
				else
					$indexes[$row['Key_name']]['columns'][] = $row['Column_name'];
			}
		}
		mysql_free_result($request);

		return $indexes;
	}

	// List all the tables in the database.
	function cffp_db_list_tables($db = false, $filter = false)
	{
		global $db_name;

		$db = $db == false ? $db_name : $db;
		$db = trim($db);
		$filter = $filter == false ? '' : " LIKE '$filter'";

		$request = db_query("
			SHOW TABLES
			FROM `" . ($db[0] == '`' ? strtr($db, array('`' => '')) : $db) . "`
			$filter", __FILE__, __LINE__);
		$tables = array();
		while ($row = mysql_fetch_row($request))
			$tables[] = $row[0];
		mysql_free_result($request);

		return $tables;
	}
}

// Installs.
if (empty($context['uninstalling']))
{
	global $db_prefix;

	$tables = array(
		'cffp_fields' => array(
			'columns' => array(
				array(
					'name' => 'ID_CFFP',
					'type' => 'int',
					'size' => 10,
					'auto' => true,
				),
				array(
					'name' => 'ID_MSG',
					'type' => 'int',
					'size' => 10,
					'default' => '0',
				),
				array(
					'name' => 'ID_GROUP',
					'type' => 'int',
					'size' => 10,
					'default' => '0',
				),
				array(
					'name' => 'ID_MEMBER',
					'type' => 'int',
					'size' => 10,
					'default' => '0',
				),
				array(
					'name' => 'posterName',
					'type' => 'varchar',
					'size' => 255,
					'default' => '',
				),
				array(
					'name' => 'posterEmail',
					'type' => 'varchar',
					'size' => 255,
					'default' => '',
				),
				array(
					'name' => 'posterIP',
					'type' => 'varchar',
					'size' => 255,
					'default' => '',
				),
				array(
					'name' => 'posterTime',
					'type' => 'int',
					'size' => 10,
					'default' => '0',
				),
				array(
					'name' => 'modifiedName',
					'type' => 'varchar',
					'size' => 255,
					'default' => '',
				),
				array(
					'name' => 'modifiedTime',
					'type' => 'int',
					'size' => 10,
					'default' => '0',
				),
			),
			'indexes' => array(
				array(
					'type' => 'primary',
					'columns' => array('ID_CFFP'),
				),
			),
		),
		'cffp_membergroups' => array(
			'columns' => array(
				array(
					'name' => 'ID_MEMBERGROUP',
					'type' => 'int',
					'size' => 10,
					'auto' => true,
				),
				array(
					'name' => 'ID_GROUP',
					'type' => 'int',
					'size' => 10,
					'default' => '0',
				),
				array(
					'name' => 'name',
					'type' => 'varchar',
					'size' => 40,
					'default' => '',
				),
				array(
					'name' => 'description',
					'type' => 'varchar',
					'size' => 255,
					'default' => '',
				),
				array(
					'name' => 'onlineColor',
					'type' => 'varchar',
					'size' => 20,
					'default' => '',
				),
				array(
					'name' => 'minGroups',
					'type' => 'mediumint',
					'size' => 9,
					'default' => '0',
					'unsigned' => true,
				),
				array(
					'name' => 'stars',
					'type' => 'varchar',
					'size' => 255,
					'default' => '',
				),
				array(
					'name' => 'boards',
					'type' => 'text',
					'default' => '',
				),
				array(
					'name' => 'child_boards',
					'type' => 'tinyint',
					'size' => 3,
					'default' => '0',
					'null' => true,
				),
			),
			'indexes' => array(
				array(
					'type' => 'primary',
					'columns' => array('ID_MEMBERGROUP'),
				),
			),
		),
		'cffp_stats' => array(
			'columns' => array(
				array(
					'name' => 'ID_STATS',
					'type' => 'int',
					'size' => 10,
					'auto' => true,
				),
				array(
					'name' => 'ID_MSG',
					'type' => 'int',
					'size' => 10,
					'default' => '0',
				),
				array(
					'name' => 'ID_GROUP',
					'type' => 'int',
					'size' => 10,
					'default' => '0',
				),
				array(
					'name' => 'data',
					'type' => 'text',
					'default' => '',
				),
				array(
					'name' => 'options_text',
					'type' => 'text',
					'default' => '',
				),
				array(
					'name' => 'is_numeric',
					'type' => 'tinyint',
					'size' => 3,
					'default' => '0',
				),
				array(
					'name' => 'average',
					'type' => 'float',
					'size' => 10,
					'default' => null,
					'null' => true,
				),
				array(
					'name' => 'sum',
					'type' => 'float',
					'size' => 10,
					'default' => null,
					'null' => true,
				),
				array(
					'name' => 'max',
					'type' => 'float',
					'size' => 10,
					'default' => null,
					'null' => true,
				),
				array(
					'name' => 'min',
					'type' => 'float',
					'size' => 10,
					'default' => null,
					'null' => true,
				),
				array(
					'name' => 'highest',
					'type' => 'float',
					'size' => 10,
					'default' => null,
					'null' => true,
				),
				array(
					'name' => 'lowest',
					'type' => 'float',
					'size' => 10,
					'default' => null,
					'null' => true,
				),
				array(
					'name' => 'most_frequent',
					'type' => 'text',
					'default' => '',
				),
				array(
					'name' => 'least_frequent',
					'type' => 'text',
					'default' => '',
				),
				array(
					'name' => 'total',
					'type' => 'int',
					'size' => 10,
					'default' => '0',
				),
			),
			'indexes' => array(
				array(
					'type' => 'primary',
					'columns' => array('ID_STATS'),
				),
			),
		),
		'cffp_settings_fields' => array(
			'columns' => array(
				array(
					'name' => 'ID_FIELD',
					'type' => 'int',
					'size' => 10,
					'auto' => true,
				),
				array(
					'name' => 'ID_GROUP',
					'type' => 'int',
					'size' => 10,
					'default' => '0',
				),
				array(
					'name' => 'id_column',
					'type' => 'int',
					'size' => 10,
					'default' => '0',
				),
				array(
					'name' => 'name',
					'type' => 'varchar',
					'size' => 40,
					'default' => '',
				),
				array(
					'name' => 'field_desc',
					'type' => 'varchar',
					'size' => 255,
					'default' => '',
				),
				array(
					'name' => 'field_long_desc',
					'type' => 'text',
					'default' => '',
				),
				array(
					'name' => 'field_search_desc',
					'type' => 'varchar',
					'size' => 255,
					'default' => '',
				),
				array(
					'name' => 'field_search_long_desc',
					'type' => 'text',
					'default' => '',
				),
				array(
					'name' => 'description',
					'type' => 'varchar',
					'size' => 255,
					'default' => '',
				),
				array(
					'name' => 'field_order',
					'type' => 'tinyint',
					'size' => 3,
					'default' => '0',
				),
				array(
					'name' => 'type',
					'type' => 'varchar',
					'size' => 8,
					'default' => 'text',
				),
				array(
					'name' => 'type_filter',
					'type' => 'varchar',
					'size' => 15,
					'default' => 'text',
				),
				array(
					'name' => 'length',
					'type' => 'smallint',
					'size' => 5,
					'default' => '255',
				),
				array(
					'name' => 'options',
					'type' => 'text',
					'default' => '',
				),
				array(
					'name' => 'options_text',
					'type' => 'text',
					'default' => '',
				),
				array(
					'name' => 'mask',
					'type' => 'varchar',
					'size' => 255,
					'default' => '',
				),
				array(
					'name' => 'bbc',
					'type' => 'tinyint',
					'size' => 3,
					'default' => '0',
				),
				array(
					'name' => 'default_value',
					'type' => 'text',
					'default' => '',
				),
				array(
					'name' => 'text_previous',
					'type' => 'text',
					'default' => '',
				),
				array(
					'name' => 'text_next',
					'type' => 'text',
					'default' => '',
				),
				array(
					'name' => 'enclose_type',
					'type' => 'varchar',
					'size' => 4,
					'default' => 'html',
				),
				array(
					'name' => 'enclose_text',
					'type' => 'text',
					'default' => '',
				),
				array(
					'name' => 'show_message_index',
					'type' => 'tinyint',
					'size' => 3,
					'default' => '0',
				),
				array(
					'name' => 'show_filter_operator',
					'type' => 'tinyint',
					'size' => 3,
					'default' => '0',
				),
				array(
					'name' => 'hide_filter',
					'type' => 'varchar',
					'size' => 40,
					'default' => '',
				),
				array(
					'name' => 'show_filter_used_options',
					'type' => 'tinyint',
					'size' => 3,
					'default' => '0',
				),
				array(
					'name' => 'show_record_list_type',
					'type' => 'varchar',
					'size' => 5,
					'default' => 'no',
				),
				array(
					'name' => 'show_record_list_width',
					'type' => 'varchar',
					'size' => 7,
					'default' => '',
				),
				array(
					'name' => 'show_record_list_name',
					'type' => 'varchar',
					'size' => 40,
					'default' => '',
				),
				array(
					'name' => 'show_moderation_list_type',
					'type' => 'varchar',
					'size' => 5,
					'default' => 'no',
				),
				array(
					'name' => 'show_moderation_list_width',
					'type' => 'varchar',
					'size' => 7,
					'default' => '',
				),
				array(
					'name' => 'show_moderation_list_name',
					'type' => 'varchar',
					'size' => 40,
					'default' => '',
				),
				array(
					'name' => 'not_empty_value',
					'type' => 'tinyint',
					'size' => 3,
					'default' => '0',
				),
				array(
					'name' => 'unique_value',
					'type' => 'tinyint',
					'size' => 3,
					'default' => '0',
				),
				array(
					'name' => 'subset',
					'type' => 'varchar',
					'size' => 40,
					'default' => '',
				),
				array(
					'name' => 'permissions',
					'type' => 'text',
					'default' => '',
				),
				array(
					'name' => 'active',
					'type' => 'tinyint',
					'size' => 3,
					'default' => '1',
				),
			),
			'indexes' => array(
				array(
					'type' => 'primary',
					'columns' => array('ID_FIELD'),
				),
			),
		),
		'cffp_settings_groups' => array(
			'columns' => array(
				array(
					'name' => 'ID_GROUP',
					'type' => 'int',
					'size' => 10,
					'auto' => true,
				),
				array(
					'name' => 'name',
					'type' => 'varchar',
					'size' => 40,
					'default' => '',
				),
				array(
					'name' => 'description',
					'type' => 'varchar',
					'size' => 255,
					'default' => '',
				),
				array(
					'name' => 'group_order',
					'type' => 'tinyint',
					'size' => 3,
					'default' => '0',
				),
				array(
					'name' => 'text_input_before',
					'type' => 'text',
					'default' => '',
				),
				array(
					'name' => 'text_input',
					'type' => 'text',
					'default' => '',
				),
				array(
					'name' => 'text_output',
					'type' => 'text',
					'default' => '',
				),
				array(
					'name' => 'boards',
					'type' => 'text',
					'default' => '',
				),
				array(
					'name' => 'child_boards',
					'type' => 'tinyint',
					'size' => 3,
					'default' => '0',
					'null' => true,
				),
				array(
					'name' => 'type',
					'type' => 'varchar',
					'size' => 10,
					'default' => 'fields',
				),
				array(
					'name' => 'show_group_name_message_index',
					'type' => 'tinyint',
					'size' => 3,
					'default' => '0',
				),
				array(
					'name' => 'show_group_name_filter',
					'type' => 'tinyint',
					'size' => 3,
					'default' => '0',
				),
				array(
					'name' => 'show_group_name_search',
					'type' => 'tinyint',
					'size' => 3,
					'default' => '0',
				),
				array(
					'name' => 'show_fields_name',
					'type' => 'tinyint',
					'size' => 3,
					'default' => '1',
				),
				array(
					'name' => 'show_fields_set',
					'type' => 'tinyint',
					'size' => 3,
					'default' => '0',
				),
				array(
					'name' => 'show_empty_fields_post',
					'type' => 'tinyint',
					'size' => 3,
					'default' => '0',
				),
				array(
					'name' => 'show_empty_fields_message_index',
					'type' => 'tinyint',
					'size' => 3,
					'default' => '0',
				),
				array(
					'name' => 'show_poster_data',
					'type' => 'tinyint',
					'size' => 3,
					'default' => '1',
				),
				array(
					'name' => 'show_modified_data',
					'type' => 'tinyint',
					'size' => 3,
					'default' => '1',
				),
				array(
					'name' => 'show_actions_as',
					'type' => 'varchar',
					'size' => 10,
					'default' => 'icon_text',
				),
				array(
					'name' => 'show_member_expanded',
					'type' => 'tinyint',
					'size' => 3,
					'default' => '0',
				),
				array(
					'name' => 'show_member_expanded_data',
					'type' => 'tinyint',
					'size' => 3,
					'default' => '0',
				),
				array(
					'name' => 'show_member_expanded_data_recent',
					'type' => 'tinyint',
					'size' => 3,
					'default' => '0',
				),
				array(
					'name' => 'show_member_expanded_new',
					'type' => 'tinyint',
					'size' => 3,
					'default' => '0',
				),
				array(
					'name' => 'show_member_recent_at_top',
					'type' => 'tinyint',
					'size' => 3,
					'default' => '0',
				),
				array(
					'name' => 'member_max_height',
					'type' => 'smallint',
					'size' => 5,
					'default' => '0',
				),
				array(
					'name' => 'member_one_value',
					'type' => 'tinyint',
					'size' => 3,
					'default' => '0',
				),
				array(
					'name' => 'show_stats',
					'type' => 'tinyint',
					'size' => 3,
					'default' => '0',
				),
				array(
					'name' => 'show_stats_post_text',
					'type' => 'text',
					'default' => '',
				),
				array(
					'name' => 'show_stats_post_chart_average',
					'type' => 'varchar',
					'size' => 40,
					'default' => '',
				),
				array(
					'name' => 'show_stats_post_chart_hbars',
					'type' => 'varchar',
					'size' => 40,
					'default' => '',
				),
				array(
					'name' => 'show_stats_post_chart_vbars',
					'type' => 'varchar',
					'size' => 40,
					'default' => '',
				),
				array(
					'name' => 'show_stats_post_summary',
					'type' => 'varchar',
					'size' => 100,
					'default' => '',
				),
				array(
					'name' => 'show_stats_message_index_text',
					'type' => 'text',
					'default' => '',
				),
				array(
					'name' => 'show_stats_message_index_chart_average',
					'type' => 'varchar',
					'size' => 40,
					'default' => '',
				),
				array(
					'name' => 'show_stats_message_index_chart_hbars',
					'type' => 'varchar',
					'size' => 40,
					'default' => '',
				),
				array(
					'name' => 'show_stats_message_index_chart_vbars',
					'type' => 'varchar',
					'size' => 40,
					'default' => '',
				),
				array(
					'name' => 'show_stats_message_index_summary',
					'type' => 'varchar',
					'size' => 100,
					'default' => '',
				),
				array(
					'name' => 'show_stats_order_by',
					'type' => 'varchar',
					'size' => 100,
					'default' => '',
				),
				array(
					'name' => 'show_sticky',
					'type' => 'tinyint',
					'size' => 3,
					'default' => null,
					'null' => true,
				),
				array(
					'name' => 'show_locked',
					'type' => 'tinyint',
					'size' => 3,
					'default' => null,
					'null' => true,
				),
				array(
					'name' => 'show_first_post_topic',
					'type' => 'tinyint',
					'size' => 3,
					'default' => '0',
				),
				array(
					'name' => 'show_replies_topic',
					'type' => 'tinyint',
					'size' => 3,
					'default' => '0',
				),
				array(
					'name' => 'no_show_with_subject_prefix',
					'type' => 'text',
					'default' => '',
				),
				array(
					'name' => 'show_in_topic',
					'type' => 'varchar',
					'size' => 11,
					'default' => 'no',
				),
				array(
					'name' => 'show_filters',
					'type' => 'text',
					'default' => '',
				),
				array(
					'name' => 'show_filters_operator',
					'type' => 'text',
					'default' => '',
				),
				array(
					'name' => 'javascript',
					'type' => 'text',
					'default' => '',
				),
				array(
					'name' => 'search_page_name',
					'type' => 'text',
					'default' => '',
				),
				array(
					'name' => 'search_page_order_by',
					'type' => 'text',
					'default' => '',
				),
				array(
					'name' => 'counter_show',
					'type' => 'tinyint',
					'size' => 3,
					'default' => '0',
				),
				array(
					'name' => 'counter_name',
					'type' => 'varchar',
					'size' => 40,
					'default' => '',
				),
				array(
					'name' => 'counter_number_name',
					'type' => 'varchar',
					'size' => 40,
					'default' => '',
				),
				array(
					'name' => 'counter_total_name',
					'type' => 'varchar',
					'size' => 40,
					'default' => '',
				),
				array(
					'name' => 'counter_average_name',
					'type' => 'varchar',
					'size' => 40,
					'default' => '',
				),
				array(
					'name' => 'top_poster_show',
					'type' => 'tinyint',
					'size' => 3,
					'default' => '0',
				),
				array(
					'name' => 'top_poster_name',
					'type' => 'varchar',
					'size' => 40,
					'default' => '',
				),
				array(
					'name' => 'profile_list_show',
					'type' => 'tinyint',
					'size' => 3,
					'default' => '0',
				),
				array(
					'name' => 'profile_list_name',
					'type' => 'varchar',
					'size' => 40,
					'default' => '',
				),
				array(
					'name' => 'moderation_list_show',
					'type' => 'tinyint',
					'size' => 3,
					'default' => '0',
				),
				array(
					'name' => 'moderation_list_menu',
					'type' => 'tinyint',
					'size' => 3,
					'default' => '0',
				),
				array(
					'name' => 'moderation_list_name',
					'type' => 'varchar',
					'size' => 40,
					'default' => '',
				),
				array(
					'name' => 'moderation_list_order_by',
					'type' => 'varchar',
					'size' => 40,
					'default' => '',
				),
				array(
					'name' => 'permissions',
					'type' => 'text',
					'default' => '',
				),
				array(
					'name' => 'active',
					'type' => 'tinyint',
					'size' => 3,
					'default' => '1',
				),
				array(
					'name' => 'template_quick_input_regex',
					'type' => 'text',
					'default' => '',
				),
				array(
					'name' => 'template_quick_input_params',
					'type' => 'text',
					'default' => '',
				),
				array(
					'name' => 'template_quick_input_help',
					'type' => 'text',
					'default' => '',
				),
				array(
					'name' => 'template_quick_input_show_modify',
					'type' => 'tinyint',
					'size' => 3,
					'default' => '0',
				),
				array(
					'name' => 'template_type',
					'type' => 'varchar',
					'size' => 4,
					'default' => 'html',
				),
				array(
					'name' => 'template_text',
					'type' => 'text',
					'default' => '',
				),
				array(
					'name' => 'template_message_index_type',
					'type' => 'varchar',
					'size' => 4,
					'default' => 'html',
				),
				array(
					'name' => 'template_message_index_text',
					'type' => 'text',
					'default' => '',
				),
			),
			'indexes' => array(
				array(
					'type' => 'primary',
					'columns' => array('ID_GROUP'),
				),
			),
		),
	);

	$current_tables = cffp_db_list_tables(false, '%cffp%');
	$real_prefix = preg_match('~^(`?)(.+?)\\1\\.(.*?)$~', $db_prefix, $match) === 1 ? $match[3] : $db_prefix;

	// Loop through each table and do what needed.
	$delete_stats = false;
	$cffp_stats_columns = in_array(strtolower($real_prefix . 'cffp_stats'), array_map('strtolower', $current_tables)) ? cffp_db_list_columns('cffp_stats') : array();
	foreach ($tables as $table => $data)
	{
		if (in_array(strtolower($real_prefix . $table), array_map('strtolower', $current_tables)))
		{
			foreach ($data['columns'] as $column)
			{
				cffp_db_add_column($table, $column);

				if (!empty($cffp_stats_columns) && !$delete_stats && 'cffp_stats' == $table && !in_array($column['name'], $cffp_stats_columns))
					$delete_stats = true;
			}
			foreach ($data['indexes'] as $index)
				cffp_db_add_index($table, $index);
		}
		else
			cffp_db_create_table($table, $data['columns'], $data['indexes']);
	}

	// Deletes statistics.
	if ($delete_stats)
		db_query("
			TRUNCATE TABLE {$db_prefix}cffp_stats", __FILE__, __LINE__);

	// Upgrades from 1.5 to 2.0.
	$deprecated_table = 'cffp_count_groups';
	$members_columns = array(
		'groups' => array(
			'name' => '',
			'type' => 'mediumint',
			'size' => 8,
			'default' => '0',
			'unsigned' => true,
		),
		'ID_MEMBERGROUP' => array(
			'name' => '',
			'type' => 'int',
			'size' => 10,
			'default' => '0',
		),
	);
	if (in_array(strtolower($real_prefix . $deprecated_table), array_map('strtolower', $current_tables)))
	{
		$request = db_query("
			SELECT ID_GROUP
			FROM {$db_prefix}cffp_settings_groups
			WHERE type = 'record'
			ORDER BY ID_GROUP ASC", __FILE__, __LINE__);
		while ($row = mysql_fetch_assoc($request))
		{
			// Adds new colums.
			$members_columns['groups']['name'] = 'cffp' . $row['ID_GROUP'] . '_groups';
			$members_columns['ID_MEMBERGROUP']['name'] = 'cffp' . $row['ID_GROUP'] . '_ID_MEMBERGROUP';
			cffp_db_add_column('members', $members_columns['groups']);
			cffp_db_add_column('members', $members_columns['ID_MEMBERGROUP']);

			// Inserts values.
			$request2 = db_query("
				SELECT ID_MEMBER, groups
				FROM {$db_prefix}$deprecated_table
				WHERE ID_GROUP = " . $row['ID_GROUP'] . "
					AND groups > 0", __FILE__, __LINE__);
			while ($row2 = mysql_fetch_assoc($request2))
			{
				if (function_exists('apache_reset_timeout'))
					@apache_reset_timeout();

				db_query("
					UPDATE {$db_prefix}members
					SET " . $members_columns['groups']['name'] . " = " . $row2['groups'] . "
					WHERE ID_MEMBER = " . $row2['ID_MEMBER'], __FILE__, __LINE__);
			}
			mysql_free_result($request2);
		}
		mysql_free_result($request);

		// Deletes deprecated table.
		cffp_db_drop_table($deprecated_table);

		// Copies counter name in counter total/average name.
		db_query("
			UPDATE {$db_prefix}cffp_settings_groups
			SET counter_number_name = counter_name, counter_total_name = counter_name, counter_average_name = counter_name", __FILE__, __LINE__);
	}
}

?>