<?php

/**
 * Custom fields and filters of post
 *
 * @package CustomFieldFilterPost
 * @version 2.0.1
 * @author davidhs
 * @copyright 2014-2019 by davidhs
 * @license Creative Commons Attribution 3.0 Unported License https://creativecommons.org/licenses/by/3.0/
 */

if (!defined('SMF'))
	die('Hacking attempt...');

// Adds actions.
function cffp_actions(&$actionArray)
{
	$actionArray['cffpeditgroup'] = array('Subs-CustomFieldFilterPost.php', 'cffpEditGroup');
	$actionArray['cffpsearch'] = array('Subs-CustomFieldFilterPost.php', 'cffpSearch');
}

// Adds menu buttons.
function cffp_menu_buttons(&$buttons)
{
	global $context, $scripturl, $user_info;

	// Search pages.
	$button_search = array();
	foreach ($context['cffp']['search_pages'] as $k => $search)
	{
		$button_search['cffp_search_' . $k] = array(
			'title' => $search['title'],
			'href' => $search['href'],
			'show' => allowedTo('search_posts'),
		);
	}

	if (!empty($button_search))
	{
		end($button_search);
		$button_search[key($button_search)]['is_last'] = true;

		if (empty($buttons['search']['sub_buttons']))
			$buttons['search']['sub_buttons'] = $button_search;
		else
		{
			$sub_buttons = $buttons['search']['sub_buttons'];
			end($sub_buttons);
			$buttons['search']['sub_buttons'][key($sub_buttons)]['is_last'] = true;
			$buttons['search']['sub_buttons'] = array_merge($buttons['search']['sub_buttons'], $button_search);
		}
	}

	// Lists on Moderation Center.
	$button_moderate = array();
	foreach ($context['cffp']['moderation_list'] as $id_group => $moderation_list)
	{
		if (!$moderation_list['menu'])
			continue;

		$button_moderate['cffp_moderation_' . $id_group] = array(
			'title' => $moderation_list['name'],
			'href' => $scripturl . '?action=moderate;area=cffp;sa=' . $id_group,
			'show' => !empty($user_info['mod_cache']) && $user_info['mod_cache']['bq'] != '0=1',
		);
	}

	if (!empty($button_moderate))
	{
		end($button_moderate);
		$button_moderate[key($button_moderate)]['is_last'] = true;

		if (empty($buttons['moderate']['sub_buttons']))
			$buttons['moderate']['sub_buttons'] = $button_moderate;
		else
		{
			$sub_buttons = $buttons['moderate']['sub_buttons'];
			end($sub_buttons);
			$buttons['moderate']['sub_buttons'][key($sub_buttons)]['is_last'] = true;
			$buttons['moderate']['sub_buttons'] = array_merge($buttons['moderate']['sub_buttons'], $button_moderate);
		}
	}
}

// Adds profile areas.
function cffp_profile_areas(&$profile_areas)
{
	global $context;

	foreach ($context['cffp']['settings_groups'] as $id_group => $settings_group)
	{
		if ('record' == $settings_group['type'] && $settings_group['profile_list_show'])
			$profile_areas['info']['areas']['showposts']['subsections']['cffp_record_' . $id_group] = array(cffpIsNotEmptyString($settings_group['profile_list_name']) ? $settings_group['profile_list_name'] : $settings_group['name'], array('profile_view_own', 'profile_view_any'));
	}
}

// Adds Moderation Center areas.
function cffpAddModerationAreas(&$moderation_areas)
{
	global $context, $scripturl, $txt;

	$subsections = array();
	foreach ($context['cffp']['settings_groups'] as $id_group => $settings_group)
	{
		if (!$settings_group['moderation_list_show'])
			continue;

		$subsections[$id_group] = array(cffpIsNotEmptyString($settings_group['moderation_list_name']) ? $settings_group['moderation_list_name'] : $settings_group['name']);
	}

	if (!empty($subsections))
	{
		$moderation_areas['posts']['areas']['cffp'] = array(
			'label' => $txt['cffp_moderation_area'],
			'enabled' => $context['can_moderate_boards'],
			'file' => 'Subs-CustomFieldFilterPost.php',
			'function' => 'cffpModerationList',
			'custom_url' => $scripturl . '?action=moderate;area=cffp',
			'subsections' => $subsections,
		);
	}
}

// Adds profile tabs.
function cffpAddProfileTabs()
{
	global $context;

	foreach ($context['cffp']['settings_groups'] as $id_group => $settings_group)
	{
		if ('record' == $settings_group['type'] && $settings_group['profile_list_show'])
			$context[$context['profile_menu_name']]['tab_data']['tabs']['cffp_record_' . $id_group] = array();
	}
}

// Returns boards and children boards.
function cffpAddChildrenBoards($board_array, $child_boards)
{
	global $smcFunc;

	if ((!is_numeric($child_boards) || !empty($child_boards)) && !empty($board_array))
	{
		$request = $smcFunc['db_query']('', '
			SELECT id_board
			FROM {db_prefix}boards
			WHERE id_parent IN ({array_int:id_parent_array})',
			array('id_parent_array' => $board_array)
		);
		$child_array = array();
		while ($row = $smcFunc['db_fetch_assoc']($request))
			$child_array[] = $row['id_board'];
		$smcFunc['db_free_result']($request);

		if (is_numeric($child_boards))
			$child_boards--;
		$child_array = cffpAddChildrenBoards($child_array, $child_boards);
		$board_array = array_merge($board_array, $child_array);
	}
	return $board_array;
}

// Edits custom post fields of a group type.
function cffpEditGroup()
{
	global $context, $smcFunc, $txt, $user_info, $modSettings, $sourcedir;

	loadTemplate('CustomFieldFilterPost');

	if (empty($_GET['id_msg']) || empty($_GET['id_group']) || !isset($_GET['board']))
		fatal_lang_error('error_cffp_params');

	// Sort out the context!
	$context['start'] = (int) $_GET['start'];
	$context['id_msg'] = (int) $_GET['id_msg'];
	$context['id_group'] = (int) $_GET['id_group'];
	$context['board'] = (int) $_GET['board'];
	$context['id_cffp'] = !empty($_GET['id_cffp']) ? (int) $_GET['id_cffp'] : 0;

	$request = $smcFunc['db_query']('', '
		SELECT ' . implode(', ', $context['cffp']['list_columns']['fields']) . '
		FROM {db_prefix}cffp_fields
		WHERE id_cffp = {int:id_cffp}',
		array(
			'id_cffp' => $context['id_cffp'],
		)
	);
	$fields_value = $smcFunc['db_fetch_assoc']($request);
	$smcFunc['db_free_result']($request);

	$user = cffpGetTypeUser();
	$owner = cffpIsOwner(empty($context['id_cffp']), !empty($fields_value) ? $fields_value['id_member'] : $context['user']['id']);

	$data_msg = cffpGetDataMsgTopic($context['id_msg']);
	$group_array = cffpGetGroupsAndFieldsOfGroup($data_msg, $context['id_group'], 'edit', $context['board'], $user, $owner);
	if (empty($group_array))
		fatal_lang_error('error_cffp_params');

	$context['params']['group'] = $group_array[$context['id_group']];

	if (isset($_GET['move_to_post']))
		$title = 'no' == $context['params']['group']['show_in_topic'] ? $txt['cffp_move_group_to_other_post'] : $txt['cffp_move_group_to_other_topic'];
	else
		$title = $txt['cffp_edit_group'];

	$context['page_title'] = sprintf($title, $context['params']['group']['name']);
	$context['sub_template'] = isset($_GET['move_to_post']) ? 'cffp_move_group' : 'cffp_edit_group';

	foreach ($context['params']['group']['settings_fields'] as $row)
	{
		$name_field = 'column_' . $row['id_column'];
		$value = isset($fields_value[$name_field]) ? $fields_value[$name_field] : '';

		$row['show_filter_used_options'] = 0;
		$context['params']['field_array'][$row['id_field']] = array(
			'name' => $row['name'],
			'field_desc' => $row['field_desc'],
			'field_long_desc' => parse_bbc(censorText($row['field_long_desc'])),
			'html' => cffpHtmlOfField($row, $value, false, $context['id_group'], $name_field),
		);
	}

	// Are we saving?
	if ((isset($_POST['save']) && cffpHavePermission($user, $owner, 'modify', $context['params']['group']['permissions'])) || (isset($_GET['add']) && cffpHavePermission($user, $owner, 'add', $context['params']['group']['permissions'])))
	{
		checkSession('get');

		// Validate inputs.
		if (isset($_GET['add']))
		{
			$infix = $context['id_msg'] . '_group' . $context['id_group'];
			$_REQUEST['cffp'] = array($context['id_group'] => $_REQUEST['cffp'][$infix]);
		}

		$post_errors = cffpValidateRequest('member', 0, 'return_post_errors');

		// Wrong verification code?
		if (isset($_GET['add']) && !$user_info['is_admin'] && !$user_info['is_mod'] && !empty($modSettings['posts_require_captcha']) && ($user_info['posts'] < $modSettings['posts_require_captcha'] || ($user_info['is_guest'] && $modSettings['posts_require_captcha'] == -1)))
		{
			require_once($sourcedir . '/Subs-Editor.php');
			$verificationOptions = array(
				'id' => 'cffp_' . $infix,
			);
			$context['require_verification'] = create_control_verification($verificationOptions, true);
			if (is_array($context['require_verification']))
				$post_errors = array_merge($post_errors, $context['require_verification']);
		}

		if ($post_errors)
		{
			loadLanguage('Errors');

			$error = array();
			foreach ($post_errors as $post_error)
			{
				if ('cffp_' == substr($post_error, 0, 5))
				{
					list ($post_error2, ) = explode(',', $post_error);
					$eval = '$txt[\'error_' . $post_error . '\'] = sprintf($txt[\'error_' . $post_error2 . '\'], \'' . implode('\', \'', $context['cffp']['validate'][$post_error]) . '\');';
					eval($eval);
				}
				$error[] = $txt['error_' . $post_error];
			}
			$txt['error_cffp_all_txt'] = implode('<br />', $error);
			fatal_lang_error('error_cffp_all_txt');
		}

		// Do the insertion/updates.
		if (isset($_GET['add']))
			$context['id_cffp'] = cffpInsertFields('member', $context['id_msg'], $context['id_group'], $context['params']['group']['settings_fields']);
		else
			cffpUpdateFields($context['id_cffp'], $context['id_group'], $context['params']['group']['settings_fields']);

		if ('member' == $context['params']['group']['type'])
			cffpUpdateStats($context['id_group'], $context['id_msg']);
	}
	// Deleting?
	elseif (isset($_GET['delete']) && cffpHavePermission($user, $owner, 'remove', $context['params']['group']['permissions']) && $context['id_cffp'])
	{
		checkSession('get');

		cffpDeleteFields($context['id_cffp']);

		if ('member' == $context['params']['group']['type'])
			cffpUpdateStats($context['id_group'], $context['id_msg']);
	}
	// Moving?
	elseif (isset($_REQUEST['move']) && cffpHavePermission($user, $owner, 'move', $context['params']['group']['permissions']) && $context['id_msg'] && $context['id_group'])
	{
		checkSession('get');

		if (isset($_POST['msg_to']))
		{
			$msg_to = (int) $_POST['msg_to'];

			if ('no' != $context['params']['group']['show_in_topic'])
			{
				$request = $smcFunc['db_query']('', '
					SELECT id_first_msg
					FROM {db_prefix}topics
					WHERE id_topic = {int:id_topic}',
					array(
						'id_topic' => $msg_to,
					)
				);
				$row = $smcFunc['db_fetch_assoc']($request);
				$smcFunc['db_free_result']($request);

				if ($row)
					$msg_to = $row['id_first_msg'];
				else
					fatal_lang_error('error_cffp_move_missing_topic');
			}

			$request = $smcFunc['db_query']('', '
				SELECT id_board
				FROM {db_prefix}messages
				WHERE id_msg = {int:id_msg}',
				array(
					'id_msg' => $msg_to,
				)
			);
			$row = $smcFunc['db_fetch_assoc']($request);
			$smcFunc['db_free_result']($request);

			if ($row)
			{
				if (!cffpIsGroupAllowInBoard($context['params']['group']['boards'], $context['params']['group']['child_boards'], $row['id_board']))
					fatal_lang_error('no' == $context['params']['group']['show_in_topic'] ? 'error_cffp_move_board_post_not_allowed' : 'error_cffp_move_board_topic_not_allowed');
			}
			else
				fatal_lang_error('error_cffp_move_missing_post');
		}
		else
		{
			$request = $smcFunc['db_query']('', '
				SELECT t.id_first_msg
				FROM {db_prefix}messages AS m
					INNER JOIN {db_prefix}topics AS t ON (t.id_topic = m.id_topic)
				WHERE m.id_msg = {int:id_msg}',
				array(
					'id_msg' => $context['id_msg'],
				)
			);
			$row = $smcFunc['db_fetch_assoc']($request);
			$smcFunc['db_free_result']($request);

			$msg_to = $row ? $row['id_first_msg'] : 0;
		}

		if ($msg_to && $msg_to != $context['id_msg'])
		{
			$data_msg_to = cffpGetDataMsgTopic($msg_to);
			$error_cffp_move_not_allowed = 'no' == $context['params']['group']['show_in_topic'] ? 'error_cffp_move_post_not_allowed' : 'error_cffp_move_topic_not_allowed';

			if (is_numeric($context['params']['group']['show_sticky']) && $context['params']['group']['show_sticky'] != $data_msg_to['sticky'])
				fatal_lang_error($error_cffp_move_not_allowed);

			if (is_numeric($context['params']['group']['show_locked']) && $context['params']['group']['show_locked'] != $data_msg_to['locked'])
				fatal_lang_error($error_cffp_move_not_allowed);

			if (!$context['params']['group']['show_first_post_topic'] && $data_msg_to['first_msg'])
				fatal_lang_error($error_cffp_move_not_allowed);

			if (!$context['params']['group']['show_replies_topic'] && !$data_msg_to['first_msg'])
				fatal_lang_error($error_cffp_move_not_allowed);

			if (!empty($context['params']['group']['no_show_with_subject_prefix']))
			{
				$subject_prefix = explode(',', $context['params']['group']['no_show_with_subject_prefix']);
				foreach ($subject_prefix as $pre)
				{
					if (0 === $smcFunc['strpos']($data_msg_to['subject'], $pre))
						fatal_lang_error($error_cffp_move_not_allowed);
				}
			}

			if ('member' == $context['params']['group']['type'])
			{
				if ($context['params']['group']['member_one_value'] && !empty($fields_value['id_member']))
				{
					$request = $smcFunc['db_query']('', '
						SELECT id_cffp
						FROM {db_prefix}cffp_fields
						WHERE id_msg = {int:id_msg}
							AND id_group = {int:id_group}
							AND id_member = {int:id_member}',
						array(
							'id_msg' => $msg_to,
							'id_group' => $context['id_group'],
							'id_member' => $fields_value['id_member'],
						)
					);
					$num_rows = $smcFunc['db_num_rows']($request);
					$smcFunc['db_free_result']($request);

					if (0 < $num_rows)
					{
						$error_cffp_move_already_has_data = 'no' == $context['params']['group']['show_in_topic'] ? 'error_cffp_move_post_already_has_data' : 'error_cffp_move_topic_already_has_data';
						fatal_lang_error($error_cffp_move_already_has_data);
					}
				}
			}
			else
			{
				$request = $smcFunc['db_query']('', '
					SELECT id_cffp
					FROM {db_prefix}cffp_fields
					WHERE id_msg = {int:id_msg}
						AND id_group = {int:id_group}',
					array(
						'id_msg' => $msg_to,
						'id_group' => $context['id_group'],
					)
				);
				$num_rows = $smcFunc['db_num_rows']($request);
				$smcFunc['db_free_result']($request);

				if (0 < $num_rows)
					fatal_lang_error('error_cffp_move_already_has_data');
			}

			cffpMoveFields($context['id_group'], $context['id_cffp'], $context['id_msg'], $msg_to);

			if ('member' == $context['params']['group']['type'])
			{
				cffpUpdateStats($context['id_group'], $context['id_msg']);
				cffpUpdateStats($context['id_group'], $msg_to);
			}
		}
	}

	if (isset($_GET['delete']) || isset($_GET['add']) || isset($_POST['save']) || isset($_REQUEST['move']))
	{
		checkSession('get');

		$request = $smcFunc['db_query']('', '
			SELECT id_topic
			FROM {db_prefix}messages
			WHERE id_msg = {int:id_msg}',
			array(
				'id_msg' => $context['id_msg'],
			)
		);
		$row = $smcFunc['db_fetch_assoc']($request);
		$smcFunc['db_free_result']($request);

		if ('member' == $context['params']['group']['type'])
		{
			$page = 'no' == $context['params']['group']['show_in_topic'] ? '.msg' . $context['id_msg'] : '.' . $context['start'];
			$params = ';cffp_show=' . $context['id_msg'] . ',' . $context['id_group'] . '#cffp' . (isset($_GET['delete']) || isset($_REQUEST['move']) ? '_msg' . $context['id_msg'] . '_group' . $context['id_group'] : $context['id_cffp']);
		}
		else
		{
			$page = '.msg' . $context['id_msg'];
			$params = '#msg' . $context['id_msg'];
		}

		redirectexit('topic=' . $row['id_topic'] . $page . $params);
	}
}

// Gets data of message and topic.
function cffpGetDataMsgTopic($id_msg = 0, $id_topic = 0)
{
	global $context;

	if (!empty($id_msg))
	{
		if (!isset($context['cffp']['data_msg_topic'][$id_msg]))
			cffpLoadDataMsgTopic($id_msg);
		$data = $context['cffp']['data_msg_topic'][$id_msg];
	}
	elseif (!empty($id_topic))
	{
		if (!isset($context['cffp']['data_msg_topic']['topic_' . $id_topic]))
			cffpLoadDataMsgTopic(array(), $id_topic);
		$data = $context['cffp']['data_msg_topic']['topic_' . $id_topic];
	}
	else
		$data = $context['cffp']['data_msg_topic']['topic_new'];

	return $data;
}

// Gets groups and fields of each group.
function cffpGetGroupsAndFieldsOfGroup($data_msg, $id_group, $cffp_action, $current_board = 0, $user = '', $owner = false, $search_page_name = false, $only_type_array = array())
{
	global $context, $smcFunc, $txt;

	// Filter by group.
	if (empty($id_group))
		$group_array = $context['cffp']['settings_groups'];
	elseif (is_array($id_group))
	{
		$group_array = array();
		foreach ($context['cffp']['settings_groups'] as $id => $settings_group)
		{
			if (in_array($id, $id_group))
				$group_array[$id] = $settings_group;
		}
	}
	else // Integer.
		$group_array = array($id_group => $context['cffp']['settings_groups'][$id_group]);

	// More filters.
	$filters = false;
	$user_gr = $user;
	$owner_gr = $owner;
	$show_in_topic = '';
	if ('search_posts' != $cffp_action)
		$search_page_name = false;

	$action_message_index = 'message_index' == substr($cffp_action, 0, 13);
	$action_search = 'search' == substr($cffp_action, 0, 6);
	$action_readonly = '_readonly' == substr($cffp_action, -9);
	$action_display_member_topic = 'display_member_topic_' == substr($cffp_action, 0, 21);

	switch ($cffp_action)
	{
		case 'display':
		case 'display_readonly':
		case 'message_index':
		{
			$type_array = array('fields', 'record');
			$perm_group_array = array('view');
			$perm_field_array = array('view');
			break;
		}
		case 'display_member':
		case 'display_member_topic_begin_all':
		case 'display_member_topic_end_all':
		case 'display_member_topic_begin_first':
		case 'display_member_topic_end_first':
		{
			$type_array = array('member');
			$user_gr = '';
			$owner_gr = false;
			$perm_group_array = array('view', 'stats', 'add', 'modify', 'remove');
			$perm_field_array = array('view');

			if ($action_display_member_topic)
				$show_in_topic = substr($cffp_action, 21);
			else
				$show_in_topic = 'no';
			break;
		}
		case 'display_member_readonly':
		case 'display_member_topic_begin_all_readonly':
		case 'display_member_topic_end_all_readonly':
		case 'display_member_topic_begin_first_readonly':
		case 'display_member_topic_end_first_readonly':
		case 'message_index_member':
		{
			$type_array = array('member');
			$user_gr = '';
			$owner_gr = false;
			$perm_group_array = array('view', 'stats');
			$perm_field_array = array('view');

			if ($action_display_member_topic)
				$show_in_topic = substr($cffp_action, 21, -9);
			elseif ('display_member_readonly' == $cffp_action)
				$show_in_topic = 'no';
			break;
		}
		case 'post':
		{
			$type_array = array('fields', 'record');
			$perm_group_array = array('add', 'modify', 'remove');
			$perm_field_array = array('view');
			break;
		}
		case 'edit':
		{
			$type_array = array('fields', 'record', 'member');
			$perm_group_array = array('add', 'modify', 'remove', 'move');
			$perm_field_array = array('view');
			break;
		}
		case 'order_by':
		{
			$type_array = array('fields', 'record', 'member');
			$perm_group_array = array('view', 'stats');
			$perm_field_array = array('sort');
			break;
		}
		case 'filters':
		{
			$type_array = array('fields', 'record', 'member');
			$filters = true;
			$perm_group_array = array('view');
			$perm_field_array = array('filter');
			break;
		}
		case 'search_posts':
		case 'search_posts_menu':
		{
			$type_array = array('fields', 'member');
			$filters = true;
			$perm_group_array = array('search');
			$perm_field_array = array('search');
			break;
		}
		case 'search_records':
		case 'search_records_menu':
		{
			$type_array = array('record');
			$perm_group_array = array('search');
			$perm_field_array = array('search');
			break;
		}
	}

	if (!empty($only_type_array))
		$type_array = array_intersect($type_array, $only_type_array);

	foreach ($group_array as $id_group => $settings_group)
	{
		if (!(in_array($settings_group['type'], $type_array) || ($filters && 'filters' == $settings_group['type'] && !empty($settings_group['show_filters']))))
		{
			unset($group_array[$id_group]);
			continue;
		}

		if (!cffpHavePermission($user_gr, $owner_gr, $perm_group_array, $settings_group['permissions']))
		{
			unset($group_array[$id_group]);
			continue;
		}

		if (!empty($show_in_topic) && $show_in_topic != $settings_group['show_in_topic'])
		{
			unset($group_array[$id_group]);
			continue;
		}

		if (is_string($search_page_name))
		{
			if (cffpIsNotEmptyString($search_page_name) xor cffpIsNotEmptyString($settings_group['search_page_name']))
			{
				unset($group_array[$id_group]);
				continue;
			}
			elseif (cffpIsNotEmptyString($search_page_name) && cffpIsNotEmptyString($settings_group['search_page_name']))
			{
				$search_page_name_array = explode(',', $settings_group['search_page_name']);
				if (!in_array($search_page_name, $search_page_name_array))
				{
					unset($group_array[$id_group]);
					continue;
				}
			}
		}

		if ($action_search)
		{
			if (empty($settings_group['boards']))
			{
				unset($group_array[$id_group]);
				continue;
			}
		}
		else
		{
			if (!('all' === $current_board) && !cffpIsGroupAllowInBoard($settings_group['boards'], $settings_group['child_boards'], $current_board))
			{
				unset($group_array[$id_group]);
				continue;
			}

			if ('filters' != $cffp_action && 'filters' != $settings_group['type'] && !empty($data_msg))
			{
				if (is_numeric($data_msg['sticky']) && is_numeric($settings_group['show_sticky']) && $settings_group['show_sticky'] != $data_msg['sticky'])
				{
					unset($group_array[$id_group]);
					continue;
				}

				if (is_numeric($data_msg['locked']) && is_numeric($settings_group['show_locked']) && $settings_group['show_locked'] != $data_msg['locked'])
				{
					unset($group_array[$id_group]);
					continue;
				}

				if (is_numeric($data_msg['first_msg']))
				{
					if (empty($settings_group['show_first_post_topic']) && $data_msg['first_msg'])
					{
						unset($group_array[$id_group]);
						continue;
					}

					if (empty($settings_group['show_replies_topic']) && !$data_msg['first_msg'])
					{
						unset($group_array[$id_group]);
						continue;
					}
				}

				if ('post' != $cffp_action && 0 < strlen($data_msg['subject']) && !empty($settings_group['no_show_with_subject_prefix']))
				{
					$subject_prefix = explode(',', $settings_group['no_show_with_subject_prefix']);
					foreach ($subject_prefix as $pre)
					{
						if (0 === $smcFunc['strpos']($data_msg['subject'], $pre))
						{
							unset($group_array[$id_group]);
							continue 2; // Exit foreach $subject_prefix and continue foreach $group_array.
						}
					}
				}
			}
		}

		if ('filters' == $settings_group['type'])
		{
			$settings_filters = cffpGetSettingsPredefinedFilters();
			$filters = explode(',', $settings_group['show_filters']);
			$show_filters_operator = explode(',', $settings_group['show_filters_operator']);

			foreach ($filters as $filter)
			{
				$group_array[$id_group]['settings_fields'][$filter] = $settings_filters[$filter];
				$group_array[$id_group]['settings_fields'][$filter]['show_filter_operator'] = in_array($filter, $show_filters_operator) ? 1 : 0;
			}
		}
		else
		{
			foreach ($settings_group['settings_fields'] as $id_field => $settings_field)
			{
				if ($action_message_index && !$settings_field['show_message_index'])
				{
					unset($group_array[$id_group]['settings_fields'][$id_field]);
					continue;
				}

				if (!cffpHavePermission($user_gr, $owner_gr, $perm_field_array, $settings_field['permissions']))
				{
					unset($group_array[$id_group]['settings_fields'][$id_field]);
					continue;
				}

				if ($user)
					$group_array[$id_group]['settings_fields'][$id_field]['can_view'] = cffpHavePermission($user, $owner, 'view', $settings_field['permissions']);
			}

			if (empty($group_array[$id_group]['settings_fields']) && ('order_by' == $cffp_action || !in_array('stats', $perm_group_array)))
			{
				unset($group_array[$id_group]);
				continue;
			}
		}

		if (!empty($settings_group['text_input_before']))
			$group_array[$id_group]['text_input_before'] = parse_bbc(censorText($settings_group['text_input_before']));
		if (!empty($settings_group['text_input']))
			$group_array[$id_group]['text_input'] = parse_bbc(censorText($settings_group['text_input']));
		if (!empty($settings_group['text_output']))
			$group_array[$id_group]['text_output'] = parse_bbc(censorText($settings_group['text_output']));
		if (!empty($settings_group['show_stats_post_text']))
			$group_array[$id_group]['show_stats_post_text'] = parse_bbc(censorText($settings_group['show_stats_post_text']));
		if (!empty($settings_group['show_stats_message_index_text']))
			$group_array[$id_group]['show_stats_message_index_text'] = parse_bbc(censorText($settings_group['show_stats_message_index_text']));

		if (!empty($settings_group['template_quick_input_regex']))
		{
			if (!empty($settings_group['template_quick_input_help']))
				$group_array[$id_group]['template_quick_input_help'] = parse_bbc(censorText($settings_group['template_quick_input_help']));
			else
			{
				$bbc = '[code]' . $group_array[$id_group]['template_quick_input_regex'] . '[/code]';
				$group_array[$id_group]['template_quick_input_help'] =
					'<span class="smalltext">' . $txt['cffp_quick_input_field_desc'] . '</span><br />' .
					parse_bbc(censorText($bbc));
			}
		}

		if ($user)
		{
			$group_array[$id_group]['can_view'] = cffpHavePermission($user, $owner, 'view', $settings_group['permissions']);
			$group_array[$id_group]['can_add'] = !$action_readonly && cffpHavePermission($user, $owner, 'add', $settings_group['permissions']);
			$group_array[$id_group]['can_modify'] = !$action_readonly && cffpHavePermission($user, $owner, 'modify', $settings_group['permissions']);
			$group_array[$id_group]['can_remove'] = !$action_readonly && cffpHavePermission($user, $owner, 'remove', $settings_group['permissions']);
			$group_array[$id_group]['can_move'] = !$action_readonly && cffpHavePermission($user, $owner, 'move', $settings_group['permissions']);
		}

		if ('search_records' == $cffp_action)
		{
			$show_record_list = array();
			$column_list = array();
			foreach ($context['cffp']['settings_groups'][$id_group]['settings_fields'] as $id_field => $settings_field)
			{
				if ('no' == $settings_field['show_record_list_type'] || !cffpHavePermission($user, $owner, 'view', $settings_field['permissions']))
					continue;

				$settings_field2 = array(
					'id_field' => $settings_field['id_field'],
					'id_column' => $settings_field['id_column'],
					'show_record_list_type' => $settings_field['show_record_list_type'],
					'show_record_list_width' => $settings_field['show_record_list_width'],
					'show_record_list_name' => $settings_field['show_record_list_name'],
				);
				if ('multi' == $settings_field['show_record_list_type'])
				{
					$column = (!empty($settings_field['show_record_list_width']) ? $settings_field['show_record_list_width'] : 'auto') . ',' . $settings_field['show_record_list_name'];
					if (!isset($column_list[$column]))
					{
						$column_list[$column] = 'column_' . $settings_field['id_column'];
						$show_record_list[$column_list[$column]] = array($settings_field2, array());
					}
					$show_record_list[$column_list[$column]][1][] = $settings_field['id_field'];
				}
				else // one.
					$show_record_list[$settings_field['id_field']] = $settings_field2;
			}
			$group_array[$id_group]['show_record_list'] = $show_record_list;
		}
	}

	return $group_array;
}

// Gets options of combobox.
function cffpGetOptionsCombobox($id_group, $name_field, $option_array = array())
{
	global $smcFunc, $context, $txt;

	if (empty($id_group) || empty($name_field))
		return false;

	if (!isset($context['cffp']['options_combobox'][$id_group][$name_field]))
	{
		$option_array2 = array();

		// Get options with one or more than one values.
		$request = $smcFunc['db_query']('', '
			SELECT DISTINCT {raw:name_field}
			FROM {db_prefix}cffp_fields
			WHERE id_group = {int:id_group}
				AND {raw:name_field} <> {string:empty}',
			array(
				'id_group' => $id_group,
				'name_field' => $name_field,
				'empty' => '',
			)
		);
		while ($row = $smcFunc['db_fetch_assoc']($request))
		{
			if (strpos($row[$name_field], ','))
				$option_array2 = array_merge($option_array2, explode(',', $row[$name_field]));
			else
				$option_array2[] = $row[$name_field];
		}
		$smcFunc['db_free_result']($request);

		// Remove duplicate options.
		$option_array2 = array_unique($option_array2);
		sort($option_array2);

		$context['cffp']['options_combobox'][$id_group][$name_field] = $option_array2;
	}

	$add_empty_value = !empty($option_array) && strtr($txt['cffp_field_empty_value'], array(',' => '')) == $option_array[0];
	if ($add_empty_value)
		array_shift($option_array);
	$option_array = array_merge($option_array, $context['cffp']['options_combobox'][$id_group][$name_field]);

	// Remove duplicate options.
	$option_array = array_unique($option_array);
	sort($option_array);

	if ($add_empty_value)
		array_unshift($option_array, strtr($txt['cffp_field_empty_value'], array(',' => '')));

	return $option_array;
}

// Gets options used of select or combobox.
function cffpGetOptionsUsed($id_group, $name_field, $board, $add_empty_value = false)
{
	global $smcFunc, $context, $txt;

	if (empty($id_group) || empty($name_field) || empty($board))
		return false;

	if (!isset($context['cffp']['options_used'][$id_group][$name_field][$board]))
	{
		$option_array = array();

		// Get options with one or more than one values.
		$request = $smcFunc['db_query']('', '
			SELECT DISTINCT {raw:name_field}
			FROM {db_prefix}cffp_fields AS cffpf
				INNER JOIN {db_prefix}messages AS m ON (m.id_msg = cffpf.id_msg)
			WHERE id_group = {int:id_group}
				AND id_board = {int:id_board}
				AND {raw:name_field} <> {string:empty}',
			array(
				'id_group' => $id_group,
				'id_board' => $board,
				'name_field' => $name_field,
				'empty' => '',
			)
		);
		while ($row = $smcFunc['db_fetch_assoc']($request))
		{
			if (strpos($row[$name_field], ','))
				$option_array = array_merge($option_array, explode(',', $row[$name_field]));
			else
				$option_array[] = $row[$name_field];
		}
		$smcFunc['db_free_result']($request);

		// Remove duplicate options.
		$option_array = array_unique($option_array);
		sort($option_array);

		$context['cffp']['options_used'][$id_group][$name_field][$board] = $option_array;
	}

	$option_array = $context['cffp']['options_used'][$id_group][$name_field][$board];
	if ($add_empty_value)
		array_unshift($option_array, strtr($txt['cffp_field_empty_value'], array(',' => '')));

	return $option_array;
}

// Gets value of field on $_REQUEST['cffp'][$id_group] array.
function cffpGetRequestValueOfField($id_group, $name_field, $type, $length, $options, $default_value = '')
{
	global $smcFunc;

	if (empty($id_group) || empty($name_field))
		return false;

	if (isset($_REQUEST['cffp'][$id_group]))
	{
		$value = isset($_REQUEST['cffp'][$id_group][$name_field]) ? $_REQUEST['cffp'][$id_group][$name_field] : '';
		if (is_array($value))
			$value = implode(',', $value);
		$value = $smcFunc['htmlspecialchars']($value, ENT_QUOTES);

		if ($type == 'boolean')
			$value = !empty($value) ? 1 : 0;
		elseif (cffpIsNotEmptyString($value))
		{
			switch ($type)
			{
				case 'text':
				case 'textarea':
				{
					if ($length)
						$value = $smcFunc['substr']($value, 0, $length);
					break;
				}
				case 'combobox':
				{
					list ($select_multiple, ) = explode(',', $options);
					if ($select_multiple)
					{
						$value_array = explode(',', $value);
						$value2_array = array();
						foreach ($value_array as $v)
						{
							$v = trim($v);
							if (cffpIsNotEmptyString($v))
							{
								if ($length)
									$v = $smcFunc['substr']($v, 0, $length);
								$value2_array[] = $v;
							}
						}
						$value = implode(',', array_unique($value2_array));
					}
					else
					{
						$value = strtr($value, array(',' => ''));
						$value = trim($value);
						if ($length)
							$value = $smcFunc['substr']($value, 0, $length);
					}
					break;
				}
			}
		}
	}
	else
		$value = $default_value;

	return $value;
}

// Gets filter operator on $_REQUEST['cffp_operator'][$id_group] array.
function cffpGetRequestFilterOperator($id_group, $name_field, $type)
{
	if (empty($id_group) || empty($name_field) || empty($type) || 'empty_non_empty' == $type)
		return false;

	if (in_array($name_field, array('date_topic', 'date_replies')))
		$default_operator = '<=';
	elseif ('subject_start' == $name_field || 'select' == $type)
		$default_operator = '=';
	else
		$default_operator = 'LIKE';

	return isset($_REQUEST['cffp_operator'][$id_group][$name_field]) ? $_REQUEST['cffp_operator'][$id_group][$name_field] : $default_operator;
}

// Gets settings for predefined filters.
function cffpGetSettingsPredefinedFilters()
{
	global $txt, $user_info, $board, $modSettings;

	$settings_filters = array();

	$settings_filters['boards'] = array(
		'id' => 'boards',
		'name' => array('', $txt['cffp_show_filters_boards']),
		'field' => array('', 'm.id_board'),
		'table_as' => array(array(), array('m')),
		'type' => 'select',
		'size' => 0,
		'length' => 0,
		'options' => '1,5',
		'options_text' => '',
		'mask' => '',
		'subset' => '',
		'text_previous' => '',
		'text_next' => '',
		'show_filter_operator' => 0,
	);

	$data_msg = array(
		'sticky' => null,
		'locked' => null,
		'id_msg' => null,
		'first_msg' => 1,
		'subject' => '',
	);
	$user = cffpGetTypeUser();
	$group_array = cffpGetGroupsAndFieldsOfGroup($data_msg, 0, 'message_index', 'all', $user, true, false, array('record'));
	foreach ($group_array as $group)
		$settings_filters['record_' . $group['id_group']] = array(
			'id' => 'record_' . $group['id_group'],
			'name' => array($group['name'], $group['name']),
			'field' => array('cffpf' . $group['id_group'] . '.id_group', 'cffpf' . $group['id_group'] . '.id_group'),
			'table_as' => array(array(), array()),
			'type' => 'select',
			'size' => 0,
			'length' => 0,
			'options' => '0,1,1,0',
			'options_text' => strtr($txt['yes'], array(',' => '')) . ',' . strtr($txt['no'], array(',' => '')),
			'mask' => '',
			'subset' => '',
			'text_previous' => '',
			'text_next' => '',
			'show_filter_operator' => 0,
		);

	$settings_filters['subject'] = array(
		'id' => 'subject',
		'name' => array($txt['subject'], $txt['subject']),
		'field' => array('mf.subject', 'm.subject'),
		'table_as' => array(array('mf'), array('m')),
		'type' => 'text',
		'size' => 15,
		'length' => 0,
		'options' => '',
		'options_text' => '',
		'mask' => 'nohtml',
		'subset' => '',
		'text_previous' => '',
		'text_next' => '',
		'show_filter_operator' => 1,
	);

	$settings_filters['subject_start'] = array(
		'id' => 'subject_start',
		'name' => array($txt['cffp_show_filters_subject_start'], $txt['cffp_show_filters_subject_start']),
		'field' => array('mf.subject', 'm.subject'),
		'table_as' => array(array('mf'), array('m')),
		'type' => 'text',
		'size' => 15,
		'length' => 0,
		'options' => '',
		'options_text' => '',
		'mask' => 'nohtml',
		'subset' => '',
		'text_previous' => '',
		'text_next' => '',
		'show_filter_operator' => 1,
	);

	$options = '0,1,symbol,number';
	$options_text = strtr($txt['cffp_symbol'], array(',' => '')) . ',' . strtr($txt['cffp_number'], array(',' => ''));
	if (!empty($txt['cffp_alphabet_letters']))
	{
		$options .= ',' . $txt['cffp_alphabet_letters'];
		$options_text .= ',' . $txt['cffp_alphabet_letters'];
	}
	$settings_filters['subject_first_char'] = array(
		'id' => 'subject_first_char',
		'name' => array($txt['cffp_show_filters_subject_first_char'], $txt['cffp_show_filters_subject_first_char']),
		'field' => array('mf.subject', 'm.subject'),
		'table_as' => array(array('mf'), array('m')),
		'type' => 'select',
		'size' => 0,
		'length' => 0,
		'options' => $options,
		'options_text' => $options_text,
		'mask' => '',
		'subset' => '',
		'text_previous' => '',
		'text_next' => '',
		'show_filter_operator' => 1,
	);

	$settings_filters['date_topic'] = array(
		'id' => 'date_topic',
		'name' => array($txt['cffp_show_filters_date_topic'], $txt['cffp_poster_time']),
		'field' => array('mf.poster_time', 'm.poster_time'),
		'table_as' => array(array('mf'), array('m')),
		'type' => 'text',
		'size' => 4,
		'length' => 0,
		'options' => '',
		'options_text' => '',
		'mask' => 'number',
		'subset' => '',
		'text_previous' => '',
		'text_next' => '',
		'show_filter_operator' => 1,
	);

	$settings_filters['date_replies'] = array(
		'id' => 'date_replies',
		'name' => array($txt['cffp_show_filters_date_replies'], $txt['cffp_poster_time']),
		'field' => array('ml.poster_time', 'm.poster_time'),
		'table_as' => array(array('ml'), array('m')),
		'type' => 'text',
		'size' => 4,
		'length' => 0,
		'options' => '',
		'options_text' => '',
		'mask' => 'number',
		'subset' => '',
		'text_previous' => '',
		'text_next' => '',
		'show_filter_operator' => 1,
	);

	$settings_filters['posted_by'] = array(
		'id' => 'posted_by',
		'name' => array($txt['posted_by'], $txt['posted_by']),
		'field' => array(
			'IFNULL(memf.real_name, mf.poster_name)',
			'IFNULL(mem.real_name, m.poster_name)',
		),
		'table_as' => array(array('memf', 'mf'), array('mem', 'm')),
		'type' => 'text',
		'size' => 15,
		'length' => 0,
		'options' => '',
		'options_text' => '',
		'mask' => 'nohtml',
		'subset' => '',
		'text_previous' => '',
		'text_next' => '',
		'show_filter_operator' => 1,
	);

	$settings_filters['new'] = array(
		'id' => 'new',
		'name' => array($txt['cffp_show_filters_topic_new_posts'], $txt['new']),
		'field' => array(
			($user_info['is_guest'] ? '0' : 'IFNULL(lt.id_msg, IFNULL(lmr.id_msg, -1)) + 1') . ' <= ml.id_msg_modified',
			($user_info['is_guest'] ? '0' : 'IFNULL(lt.id_msg, IFNULL(lmr.id_msg, 0))') . ' < m.id_msg',
		),
		'table_as' => array(array('lt', 'lmr', 'ml'), array('lt', 'lmr', 'm')),
		'type' => 'select',
		'size' => 0,
		'length' => 0,
		'options' => '0,1,1,0',
		'options_text' => strtr($txt['yes'], array(',' => '')) . ',' . strtr($txt['no'], array(',' => '')),
		'mask' => '',
		'subset' => '',
		'text_previous' => '',
		'text_next' => '',
		'show_filter_operator' => 0,
	);

	$settings_filters['sticky'] = array(
		'id' => 'sticky',
		'name' => array($txt['cffp_show_filters_sticky'], $txt['cffp_show_filters_sticky']),
		'field' => array('t.is_sticky', 't.is_sticky'),
		'table_as' => array(array('t'), array('t')),
		'type' => 'select',
		'size' => 0,
		'length' => 0,
		'options' => '0,1,1,0',
		'options_text' => strtr($txt['yes'], array(',' => '')) . ',' . strtr($txt['no'], array(',' => '')),
		'mask' => '',
		'subset' => '',
		'text_previous' => '',
		'text_next' => '',
		'show_filter_operator' => 0,
	);

	$settings_filters['locked'] = array(
		'id' => 'locked',
		'name' => array($txt['cffp_show_filters_locked'], $txt['cffp_show_filters_locked']),
		'field' => array('t.locked', 't.locked'),
		'table_as' => array(array('t'), array('t')),
		'type' => 'select',
		'size' => 0,
		'length' => 0,
		'options' => '0,1,1,0',
		'options_text' => strtr($txt['yes'], array(',' => '')) . ',' . strtr($txt['no'], array(',' => '')),
		'mask' => '',
		'subset' => '',
		'text_previous' => '',
		'text_next' => '',
		'show_filter_operator' => 0,
	);

	$settings_filters['replies'] = array(
		'id' => 'replies',
		'name' => array($txt['replies'], ''),
		'field' => array('t.num_replies', ''),
		'table_as' => array(array('t'), array()),
		'type' => 'select',
		'size' => 0,
		'length' => 0,
		'options' => '0,1,no,normal,hot,very_hot',
		'options_text' =>
			strtr($txt['cffp_topic_without_replies'], array(',' => '')) . ',' .
			strtr($txt['normal_topic'], array(',' => '')) . ',' .
			strtr(sprintf($txt['hot_topics'], $modSettings['hotTopicPosts']), array(',' => '')) . ',' .
			strtr(sprintf($txt['very_hot_topics'], $modSettings['hotTopicVeryPosts']), array(',' => '')),
		'mask' => '',
		'subset' => '',
		'text_previous' => '',
		'text_next' => '',
		'show_filter_operator' => 1,
	);

	$settings_filters['poll'] = array(
		'id' => 'poll',
		'name' => array($txt['poll'], ''),
		'field' => array('t.id_poll', ''),
		'table_as' => array(array('t'), array()),
		'type' => 'select',
		'size' => 0,
		'length' => 0,
		'options' => '0,1,1,0',
		'options_text' => strtr($txt['yes'], array(',' => '')) . ',' . strtr($txt['no'], array(',' => '')),
		'mask' => '',
		'subset' => '',
		'text_previous' => '',
		'text_next' => '',
		'show_filter_operator' => 0,
	);

	return $settings_filters;
}

// Gets tokens from a string to search.
function cffpGetTokensSearch($string)
{
	$string = strtr($string, array('&quot;' => '"'));

	preg_match_all('~(\s+OR\s+)?(\s*-)?("([^"]+)"|([^\s]+))~', $string, $match);

	foreach ($match[0] as $i => $m)
	{
		if ('' == $match[5][$i])
			$match[5][$i] = $match[4][$i];
		if ('-' == trim($match[2][$i]))
			$match[5][$i] = '-' . $match[5][$i];
		if ('OR' == trim($match[1][$i]))
			$match[5][$i] = 'OR' . $match[5][$i];

		$match[5][$i] = strtr($match[5][$i], array('"' => '&quot;'));
	}

	return array_filter($match[5], 'cffpIsNotEmptyString');
}

// Gets SQL parameters of filter.
function cffpGetSqlFilter($cffp_action)
{
	global $context, $txt, $modSettings, $smcFunc;

	$filter_index = 'filters' == $cffp_action ? 0 : 1;
	$group_checked = 'filters' == $cffp_action && !empty($context['cffp']['order_by']['field_array']['group_checked']) ? $context['cffp']['order_by']['field_array']['group_checked'] : 0;
	$record_checked_array = array();

	$from = '';
	$from_join_cffp_fields = array();
	$cond = '';
	$cond_array = array();
	$query_parameters = array();
	$query_table_as = array();

	$not_operator = array(
		'=' => '<>',
		'<>' => '=',
		'<' => '>=',
		'<=' => '>',
		'>' => '<=',
		'>=' => '<',
		'LIKE' => 'NOT LIKE',
		'NOT LIKE' => 'LIKE',
	);

	if (!empty($context['cffp'][$cffp_action]) && isset($_REQUEST['cffp']) && is_array($_REQUEST['cffp']))
	{
		$attr_id_msg = 'filters' == $cffp_action ? 't.id_first_msg' : 'm.id_msg';
		foreach ($context['cffp'][$cffp_action]['group_array'] as $id_group => $group)
		{
			if (empty($_REQUEST['cffp'][$id_group]))
				continue;

			$table_as = 'cffpf' . $id_group;
			$cond_group_array =array();

			if ('filters' == $group['type'])
			{
				foreach ($group['settings_fields'] as $settings_field)
				{
					$name_field = $settings_field['id'];
					if (!isset($_REQUEST['cffp'][$id_group][$name_field]) || !cffpIsNotEmptyString($_REQUEST['cffp'][$id_group][$name_field]))
						continue;

					$value_field_query = 'cffp_value_' . $id_group . '_' . $name_field;
					$value = $smcFunc['htmlspecialchars']($_REQUEST['cffp'][$id_group][$name_field], ENT_QUOTES);
					$value_slash = strtr($value, array('%' => '\%', '_' => '\_'));

					$operator = cffpGetRequestFilterOperator($id_group, $name_field, $settings_field['type']);
					$operator_like = !(false === strpos($operator, 'LIKE'));
					$operator_not = '<>' == $operator;

					switch ($settings_field['id'])
					{
						case 'boards':
						{
							$cond_array[] = $settings_field['field'][$filter_index] . ' IN ({array_int:' . $value_field_query . '})';
							$query_parameters[$value_field_query] = explode(',', $value);
							break;
						}
						case 'subject':
						case 'posted_by':
						{
							if ($operator_like)
							{
								$tokens = cffpGetTokensSearch($value_slash);

								$cond_group_prev = '1=0';
								foreach ($tokens as $k => $token)
								{
									$cond_or = 'OR' == substr($token, 0, 2);
									if ($cond_or)
										$token = substr($token, 2);
									$op_not = '-' == substr($token, 0, 1);
									if ($op_not)
										$token = substr($token, 1);
									if (!cffpIsNotEmptyString($token))
										continue;

									$cond_token = $settings_field['field'][$filter_index] . ' ' . ($op_not ? $not_operator[$operator] : $operator) . ' {string:' . $value_field_query . '_' . $k . '}';

									if ($cond_or)
										$cond_group_prev .= ' OR ' . $cond_token;
									else
									{
										if ('1=0' != $cond_group_prev)
											$cond_group_array[] = '(' . $cond_group_prev . ')';
										$cond_group_prev = $cond_token;
									}

									$query_parameters[$value_field_query . '_' . $k] = '%' . $token . '%';
								}
								if ('1=0' != $cond_group_prev)
									$cond_group_array[] = '(' . $cond_group_prev . ')';
							}
							else // Operators =, <>
							{
								$cond_group_array[] = $settings_field['field'][$filter_index] . ' ' . $operator . ' {string:' . $value_field_query . '}';
								$query_parameters[$value_field_query] = $value;
							}
							break;
						}
						case 'subject_start':
						{
							$cond_group_array[] = $settings_field['field'][$filter_index] . ($operator_not ? ' NOT' : '') . ' LIKE {string:' . $value_field_query . '}';
							$query_parameters[$value_field_query] = $value_slash . '%';
							break;
						}
						case 'subject_first_char':
						{
							if ('symbol' == $value)
							{
								$i = 1;
								$cond_group_array2 = array();
								$cond_group_array2[] = $settings_field['field'][$filter_index] . ($operator_not ? '' : ' NOT') . ' RLIKE {string:' . $value_field_query . '_' . $i . '}';
								$query_parameters[$value_field_query . '_' . $i] = '^[0-9]';
								if (!empty($txt['cffp_alphabet_letters']))
								{
									$letters = explode(',', $txt['cffp_alphabet_letters']);
									foreach ($letters as $letter)
									{
										$i++;
										$cond_group_array2[] = $settings_field['field'][$filter_index] . ($operator_not ? '' : ' NOT') . ' LIKE {string:' . $value_field_query . '_' . $i . '}';
										$query_parameters[$value_field_query . '_' . $i] = strtr($letter, array('%' => '\%', '_' => '\_')) . '%';
									}
								}

								if ($operator_not)
									$cond_group_array[] = '(' . implode(' OR ', $cond_group_array2) . ')';
								else
									$cond_group_array = array_merge($cond_group_array, $cond_group_array2);
							}
							elseif ('number' == $value)
							{
								$cond_group_array[] = $settings_field['field'][$filter_index] . ($operator_not ? ' NOT' : '') . ' RLIKE {string:' . $value_field_query . '}';
								$query_parameters[$value_field_query] = '^[0-9]';
							}
							else
							{
								$cond_group_array[] = $settings_field['field'][$filter_index] . ($operator_not ? ' NOT' : '') . ' LIKE {string:' . $value_field_query . '}';
								$query_parameters[$value_field_query] = $value_slash . '%';
							}
							break;
						}
						case 'date_topic':
						case 'date_replies':
						{
							$cond_group_array[] = '((UNIX_TIMESTAMP() - ' . $settings_field['field'][$filter_index] . ') / 86400 ' . $operator . ' {int:' . $value_field_query . '})';
							$query_parameters[$value_field_query] = $value;
							break;
						}
						case 'new':
						{
							$cond_group_array[] = $value ? $settings_field['field'][$filter_index] : 'NOT (' . $settings_field['field'][$filter_index] . ')';
							break;
						}
						case 'sticky':
						case 'locked':
						{
							$cond_group_array[] = $settings_field['field'][$filter_index] . ' = {int:' . $value_field_query . '}';
							$query_parameters[$value_field_query] = $value;
							break;
						}
						case 'replies':
						{
							switch ($value)
							{
								case 'no':
								{
									$cond_group_array[] = $settings_field['field'][$filter_index] . ' ' . $operator . ' {int:' . $value_field_query . '}';
									$query_parameters[$value_field_query] = 0;
									break;
								}
								case 'normal':
								{
									if ($operator_not)
										$cond_group_array[] = '(' . $settings_field['field'][$filter_index] . ' = {int:' . $value_field_query . '_1} OR ' . $settings_field['field'][$filter_index] . ' >= {int:' . $value_field_query . '_2})';
									else
									{
										$cond_group_array[] = $settings_field['field'][$filter_index] . ' > {int:' . $value_field_query . '_1}';
										$cond_group_array[] = $settings_field['field'][$filter_index] . ' < {int:' . $value_field_query . '_2}';
									}
									$query_parameters[$value_field_query . '_1'] = 0;
									$query_parameters[$value_field_query . '_2'] = $modSettings['hotTopicPosts'];
									break;
								}
								case 'hot':
								{
									if ($operator_not)
										$cond_group_array[] = '(' . $settings_field['field'][$filter_index] . ' < {int:' . $value_field_query . '_1} OR ' . $settings_field['field'][$filter_index] . ' >= {int:' . $value_field_query . '_2})';
									else
									{
										$cond_group_array[] = $settings_field['field'][$filter_index] . ' >= {int:' . $value_field_query . '_1}';
										$cond_group_array[] = $settings_field['field'][$filter_index] . ' < {int:' . $value_field_query . '_2}';
									}
									$query_parameters[$value_field_query . '_1'] = $modSettings['hotTopicPosts'];
									$query_parameters[$value_field_query . '_2'] = $modSettings['hotTopicVeryPosts'];
									break;
								}
								case 'very_hot':
								{
									$cond_group_array[] = $settings_field['field'][$filter_index] . ' ' . ($operator_not ? '<' : '>=') . ' {int:' . $value_field_query . '}';
									$query_parameters[$value_field_query] = $modSettings['hotTopicVeryPosts'];
									break;
								}
							}
							break;
						}
						case 'poll':
						{
							$cond_group_array[] = $settings_field['field'][$filter_index] . ' ' . (empty($value) ? '=' : '<>') . ' {int:' . $value_field_query . '}';
							$query_parameters[$value_field_query] = 0;
							break;
						}
						default: // record_ID
						{
							$id_group2 = substr($settings_field['id'], 7);
							if ($value)
							{
								$record_checked_array[] = $id_group2;
								$cond_group_array[] = $settings_field['field'][$filter_index] . ' = {int:' . $value_field_query . '}';
								$query_parameters[$value_field_query] = $id_group2;
							}
							else
							{
								$request = $smcFunc['db_query']('', '
									SELECT DISTINCT m.id_msg
									FROM {db_prefix}messages AS m
										INNER JOIN {db_prefix}cffp_fields AS cffpf ON (cffpf.id_msg = m.id_msg)
									WHERE id_group = {int:id_group}',
									array(
										'id_group' => $id_group2,
									)
								);
								$message_with_group = array();
								while ($row = $smcFunc['db_fetch_assoc']($request))
									$message_with_group[] = $row['id_msg'];
								$smcFunc['db_free_result']($request);

								$settings_field['table_as'] = array(array('mf'), array('m'));
								$cond_group_array[] = $settings_field['table_as'][$filter_index][0] . '.id_msg NOT IN ({array_int:' . $value_field_query . '})';
								$query_parameters[$value_field_query] = $message_with_group;
							}
							break;
						}
					}

					$query_table_as = array_merge($query_table_as, $settings_field['table_as'][$filter_index]);
				}
			}
			else
			{
				foreach ($group['settings_fields'] as $settings_field)
				{
					$name_field = 'column_' . $settings_field['id_column'];
					if (!isset($_REQUEST['cffp'][$id_group][$name_field]) || !cffpIsNotEmptyString($_REQUEST['cffp'][$id_group][$name_field]))
						continue;

					$name_field_query = 'cffp_name_' . $name_field;
					$value_field_query = 'cffp_value_' . $id_group . '_' . $name_field;
					$value = $smcFunc['htmlspecialchars']($_REQUEST['cffp'][$id_group][$name_field], ENT_QUOTES);
					$value_slash = strtr($value, array('%' => '\%', '_' => '\_'));
					$operator = cffpGetRequestFilterOperator($id_group, $name_field, $settings_field['type_filter']);
					$operator_like = !(false === strpos($operator, 'LIKE'));
					$query_parameters[$name_field_query] = $name_field;
					switch ($settings_field['type_filter'])
					{
						case 'empty_non_empty':
						{
							if ('empty' == $value)
								$cond_group_array[] = '(' . $table_as . '.{raw:' . $name_field_query . '} = {string:' . $value_field_query . '} OR ' . $table_as . '.{raw:' . $name_field_query . '} IS NULL)';
							else
								$cond_group_array[] = '(' . $table_as . '.{raw:' . $name_field_query . '} <> {string:' . $value_field_query . '} AND ' . $table_as . '.{raw:' . $name_field_query . '} IS NOT NULL)';
							$query_parameters[$value_field_query] = '';
							break;
						}
						case 'select':
						case 'combobox':
						{
							if (strtr($txt['cffp_field_empty_value'], array(',' => '')) == $value)
							{
								$cond_group_array[] = '(' . $table_as . '.{raw:' . $name_field_query . '} = {string:' . $value_field_query . '} OR ' . $table_as . '.{raw:' . $name_field_query . '} IS NULL)';
								$query_parameters[$value_field_query] = '';
							}
							else
							{
								$options = explode(',', $settings_field['options']);
								$select_multiple = array_shift($options);
								if ($select_multiple && !$operator_like)
								{
									$cond_group_array[] =
										('=' == $operator ? '' : 'NOT ') .
										'(' .
											$table_as . '.{raw:' . $name_field_query . '} = {string:' . $value_field_query . '_1}' .
											' OR ' . $table_as . '.{raw:' . $name_field_query . '} LIKE {string:' . $value_field_query . '_2}' .
											' OR ' . $table_as . '.{raw:' . $name_field_query . '} LIKE {string:' . $value_field_query . '_3}' .
											' OR ' . $table_as . '.{raw:' . $name_field_query . '} LIKE {string:' . $value_field_query . '_4}' .
										')';
									$query_parameters[$value_field_query . '_1'] = $value;
									$query_parameters[$value_field_query . '_2'] = '%,' . $value_slash . ',%';
									$query_parameters[$value_field_query . '_3'] = $value_slash . ',%';
									$query_parameters[$value_field_query . '_4'] = '%,' . $value_slash;
								}
								else
								{
									$cond_group_array[] = $table_as . '.{raw:' . $name_field_query . '} ' . $operator . ' {string:' . $value_field_query . '}';
									$query_parameters[$value_field_query] = $operator_like ? '%' . $value_slash . '%' : $value;
								}
							}
							break;
						}
						default: // text.
						{
							if ($operator_like)
							{
								$tokens = cffpGetTokensSearch($value_slash);

								$cond_group_prev = '1=0';
								foreach ($tokens as $k => $token)
								{
									$cond_or = 'OR' == substr($token, 0, 2);
									if ($cond_or)
										$token = substr($token, 2);
									$op_not = '-' == substr($token, 0, 1);
									if ($op_not)
										$token = substr($token, 1);
									if (!cffpIsNotEmptyString($token))
										continue;

									$cond_token = $table_as . '.{raw:' . $name_field_query . '} ' . ($op_not ? $not_operator[$operator] : $operator) . ' {string:' . $value_field_query . '_' . $k . '}';

									if ($cond_or)
										$cond_group_prev .= ' OR ' . $cond_token;
									else
									{
										if ('1=0' != $cond_group_prev)
											$cond_group_array[] = '(' . $cond_group_prev . ')';
										$cond_group_prev = $cond_token;
									}

									$query_parameters[$value_field_query . '_' . $k] = '%' . $token . '%';
								}
								if ('1=0' != $cond_group_prev)
									$cond_group_array[] = '(' . $cond_group_prev . ')';
							}
							else
							{
								$cond_group_array[] = $table_as . '.{raw:' . $name_field_query . '} ' . $operator . ' {string:' . $value_field_query . '}';
								$query_parameters[$value_field_query] = $value;
							}
							break;
						}
					}
				}

				if (!empty($cond_group_array) || $group_checked == $id_group)
				{
					if ($group_checked == $id_group)
						$group_checked = 0;

					$query_parameters['cffp_id_group_' . $id_group] = $id_group;

					$from .= '
			LEFT JOIN {db_prefix}cffp_fields AS ' . $table_as . ' ON (' . $table_as . '.id_msg = ' . $attr_id_msg . ' AND ' . $table_as . '.id_group = {int:cffp_id_group_' . $id_group . '})';
					$from_join_cffp_fields[] = $id_group;
				}
			}

			$cond_array = array_merge($cond_array, $cond_group_array);
		}

		if (!empty($cond_array))
			$cond = implode(' AND ', $cond_array);
		$query_table_as = array_unique($query_table_as);
	}

	if (!empty($group_checked))
	{
		$attr_id_msg = 't.id_first_msg';
		if (is_numeric($group_checked))
		{
			$id_group = $group_checked;
			$table_as = 'cffpf' . $id_group;

			$from .= '
			LEFT JOIN {db_prefix}cffp_fields AS ' . $table_as . ' ON (' . $table_as . '.id_msg = ' . $attr_id_msg . ' AND ' . $table_as . '.id_group = {int:cffp_id_group_' . $id_group . '})';
			$from_join_cffp_fields[] = $id_group;
		}
		else
		{
			$id_group = substr($group_checked, 6);
			$table_as = 'cffps' . $id_group;

			$from .= '
			LEFT JOIN {db_prefix}cffp_stats AS ' . $table_as . ' ON (' . $table_as . '.id_msg = ' . $attr_id_msg . ' AND ' . $table_as . '.id_group = {int:cffp_id_group_' . $id_group . '})';
		}
		$query_parameters['cffp_id_group_' . $id_group] = $id_group;
	}

	$record_checked_array = array_diff($record_checked_array, $from_join_cffp_fields);
	foreach ($record_checked_array as $id_group)
	{
		$attr_id_msg = 't.id_first_msg';
		$table_as = 'cffpf' . $id_group;
		$from .= '
			LEFT JOIN {db_prefix}cffp_fields AS ' . $table_as . ' ON (' . $table_as . '.id_msg = ' . $attr_id_msg . ')';
	}

	return compact('from', 'cond', 'query_parameters', 'query_table_as');
}

// Gets SQL parameters of search.
function cffpGetSqlSearch($cffp_action)
{
	global $modSettings, $context, $user_info;

	$params = cffpGetSqlFilter($cffp_action);

	$cond_array = array();
	$query_parameters = array();
	$from = '{db_prefix}messages AS m';
	if ('search_posts' == $cffp_action)
	{
		$from .= '
			INNER JOIN {db_prefix}topics AS t ON (t.id_topic = m.id_topic)
			LEFT JOIN {db_prefix}members AS mem ON (mem.id_member = m.id_member)
			LEFT JOIN {db_prefix}log_topics AS lt ON (lt.id_topic = t.id_topic AND lt.id_member = {int:id_member})
			LEFT JOIN {db_prefix}log_mark_read AS lmr ON (lmr.id_board = t.id_board AND lmr.id_member = {int:id_member})';
		$query_parameters['id_member'] = $user_info['id'];

		$board_array = array();
		$all_boards = true;
		$show_sticky = array(false, false);
		$show_locked = array(false, false);
		$show_first_post_topic = true;
		$show_replies_topic = true;
		$no_show_with_subject_prefix = array();
		foreach ($context['cffp']['search_posts']['group_array'] as $id_group => $group)
		{
			if ('filters' != $group['type'])
			{
				if (is_numeric($group['show_sticky']))
					$show_sticky[(int) $group['show_sticky']] = true;
				if (is_numeric($group['show_locked']))
					$show_locked[(int) $group['show_locked']] = true;
				if (!$group['show_first_post_topic'])
					$show_first_post_topic = false;
				if (!$group['show_replies_topic'])
					$show_replies_topic = false;
				if (cffpIsNotEmptyString($group['no_show_with_subject_prefix']))
					$no_show_with_subject_prefix = array_merge($no_show_with_subject_prefix, explode(',', $group['no_show_with_subject_prefix']));
			}

			if ('all' != $group['boards'])
			{
				$current_board_array = is_numeric($group['boards']) ? array($group['boards']) : explode(',', $group['boards']);
				$current_board_array = cffpAddChildrenBoards($current_board_array, $group['child_boards']);

				if ($all_boards)
				{
					$board_array = $current_board_array;
					$all_boards = false;
				}
				else
					$board_array = array_intersect($board_array, $current_board_array);
			}
		}
		if (!$all_boards)
		{
			if (empty($board_array))
				$board_array = array(0);

			$cond_array[] = 'm.id_board IN ({array_int:id_board_array})';
			$query_parameters['id_board_array'] = $board_array;
		}
		if ($show_sticky[0])
		{
			$cond_array[] = 't.is_sticky = {int:show_sticky_0}';
			$query_parameters['show_sticky_0'] = 0;
		}
		if ($show_sticky[1])
		{
			$cond_array[] = 't.is_sticky = {int:show_sticky_1}';
			$query_parameters['show_sticky_1'] = 1;
		}
		if ($show_locked[0])
		{
			$cond_array[] = 't.locked = {int:show_locked_0}';
			$query_parameters['show_locked_0'] = 0;
		}
		if ($show_locked[1])
		{
			$cond_array[] = 't.locked = {int:show_locked_1}';
			$query_parameters['show_locked_1'] = 1;
		}
		if (!$show_first_post_topic)
			$cond_array[] = 'm.id_msg <> t.id_first_msg';
		if (!$show_replies_topic)
			$cond_array[] = 'm.id_msg = t.id_first_msg';
		if (!empty($no_show_with_subject_prefix))
		{
			$no_show_with_subject_prefix = array_unique($no_show_with_subject_prefix);
			foreach ($no_show_with_subject_prefix as $k => $prefix)
			{
				$cond_array[] = 'm.subject NOT LIKE {string:prefix_' . $k . '}';
				$query_parameters['prefix_' . $k] = $prefix . '%';
			}
		}
	}
	else // search_records.
	{
		$from .= '
			INNER JOIN {db_prefix}cffp_fields AS cffpf0 ON (cffpf0.id_msg = m.id_msg)
			LEFT JOIN {db_prefix}members AS mem ON (mem.id_member = cffpf0.id_member)';
		$cond_array[] = 'cffpf0.id_group = {int:id_group}';
		$query_parameters['id_group'] = $context['id_group'];
	}

	if ($modSettings['postmod_active'] && !allowedTo('approve_posts'))
	{
		if ('search_posts' != $cffp_action)
			$from .= '
			INNER JOIN {db_prefix}topics AS t ON (t.id_topic = m.id_topic)';
		$cond_array[] = '(t.approved = {int:is_approved}' . ($user_info['is_guest'] ? '' : ' OR t.id_member_started = {int:current_member}') . ')';
		$query_parameters['is_approved'] = 1;
		$query_parameters['current_member'] = $user_info['id'];
	}
	if (!empty($params['from']))
		$from .= $params['from'];
	if (!empty($params['cond']))
		$cond_array[] = $params['cond'];
	if (!empty($params['query_parameters']))
		$query_parameters = array_merge($query_parameters, $params['query_parameters']);

	$cond = (!empty($cond_array)) ? implode(' AND ', $cond_array) : '';

	return compact('from', 'cond', 'query_parameters');
}

// Gets number total of topics on current board. Assumes !empty($context['cffp_filters']['sql']['from']) || !empty($context['cffp_filters']['sql']['cond']).
function cffpGetTotalTopics()
{
	global $context, $smcFunc, $user_info, $modSettings, $board;

	$request = $smcFunc['db_query']('', '
		SELECT DISTINCT t.id_topic
		FROM {db_prefix}topics AS t
			INNER JOIN {db_prefix}messages AS ml ON (ml.id_msg = t.id_last_msg)
			INNER JOIN {db_prefix}messages AS mf ON (mf.id_msg = t.id_first_msg)
			LEFT JOIN {db_prefix}members AS meml ON (meml.id_member = ml.id_member)
			LEFT JOIN {db_prefix}members AS memf ON (memf.id_member = mf.id_member)' . ($user_info['is_guest'] ? '' : '
			LEFT JOIN {db_prefix}log_topics AS lt ON (lt.id_topic = t.id_topic AND lt.id_member = {int:current_member})
			LEFT JOIN {db_prefix}log_mark_read AS lmr ON (lmr.id_board = {int:current_board} AND lmr.id_member = {int:current_member})') . $context['cffp_filters']['sql']['from'] . '
		WHERE t.id_board = {int:current_board}' . (!$modSettings['postmod_active'] || allowedTo('approve_posts') ? '' : '
			AND (t.approved = {int:is_approved}' . ($user_info['is_guest'] ? '' : ' OR t.id_member_started = {int:current_member}') . ')') . (empty($context['cffp_filters']['sql']['cond']) ? '' : '
			AND ' . $context['cffp_filters']['sql']['cond']),
		array_merge($context['cffp_filters']['sql']['query_parameters'], array(
			'current_board' => $board,
			'current_member' => $user_info['id'],
			'is_approved' => 1,
		))
	);
	$num_rows = $smcFunc['db_num_rows']($request);
	$smcFunc['db_free_result']($request);

	return $num_rows;
}

// Gets type of user.
function cffpGetTypeUser()
{
	global $context;

	if ($context['user']['is_admin'])
		$user = 'admin';
	elseif (allowedTo('moderate_forum'))
		$user = 'globalmod';
	elseif ($context['user']['is_mod'])
		$user = 'localmod';
	elseif ($context['user']['is_logged'])
		$user = 'user';
	elseif ($context['user']['is_guest'])
		$user = 'guest';

	return $user;
}

// Gets value of fields by post.
function cffpGetValueOfFieldsByMsg($id_msg, $member)
{
	global $context;

	if (empty($id_msg))
		return array();

	if (!isset($context['cffp']['fields_value'][$id_msg]))
		cffpLoadValueOfFieldsByMsg($id_msg);

	return $context['cffp']['fields_value'][$id_msg][$member ? 1 : 0];
}

// Indicates if user have permission.
function cffpHavePermission($user, $owner, $perm, $permissions)
{
	if (!is_array($permissions))
		$permissions = explode(',', $permissions);

	if (!is_array($perm))
		$perm = array($perm);

	$have = false;

	if (empty($user))
	{
		foreach ($perm as $p)
		{
			foreach ($permissions as $p2)
			{
				if (strpos($p2, ':' . $p))
				{
					$have = true;
					break 2; // Exit foreach $perm.
				}
			}
		}
	}
	else
	{
		foreach ($perm as $p)
		{
			$have = in_array($user . ':' . $p, $permissions);
			if ($have)
				break; // Exit foreach.
		}
	}

	if (!$have && $owner)
	{
		foreach ($perm as $p)
		{
			$have = in_array('owner:' . $p, $permissions);
			if ($have)
				break; // Exit foreach.
		}
	}

	return $have;
}

// Returns HTML code (input or ouput) of field.
// If $output then $id is $id_msg else $id is $id_cffp_group.
function cffpHtmlOfField($field, $value, $output = true, $id = 0, $name_field = '')
{
	global $txt;

	if ($output)
	{
		$id_msg = $id;
		$html = $value;
		switch ($field['type'])
		{
			case 'text':
			{
				if ($field['bbc'])
					$html = parse_bbc(censorText($html));
				break;
			}
			case 'textarea':
			{
				if ($field['bbc'])
					$html = parse_bbc(censorText($html));
				else
					$html = strtr($html, array("\n" => '<br />'));
				break;
			}
			case 'select':
			case 'combobox':
			{
				$options = explode(',', $field['options']);
				$select_multiple = array_shift($options);
				if ('combobox' != $field['type'] && cffpIsNotEmptyString($field['options_text']))
				{
					array_shift($options);
					$trans = array_combine($options, explode(',', $field['options_text']));
					if (!empty($trans))
					{
						$value_array = $select_multiple ? explode(',', $value) : array($value);
						foreach ($value_array as $k => $v)
						{
							if (isset($trans[$v]))
								$value_array[$k] = $trans[$v];
						}
						$html = implode(',', $value_array);
					}
				}
				if ($select_multiple)
					$html = str_replace(',', ', ', $html);
				break;
			}
			case 'radio':
			case 'check':
			{
				if (cffpIsNotEmptyString($field['options']) && cffpIsNotEmptyString($field['options_text']))
				{
					$trans = array_combine(explode(',', $field['options']), explode(',', $field['options_text']));
					if (!empty($trans))
					{
						$value_array = 'check' == $field['type'] ? explode(',', $value) : array($value);
						foreach ($value_array as $k => $v)
						{
							if (isset($trans[$v]))
								$value_array[$k] = $trans[$v];
						}
						$html = implode(',', $value_array);
					}
				}

				if ('check' == $field['type'])
					$html = str_replace(',', ', ', $html);
				break;
			}
			case 'boolean':
			{
				$html = $html ? $txt['yes'] : $txt['no'];
				break;
			}
		}

		// Enclosing the output text within some other text.
		if (cffpIsNotEmptyString($html))
		{
			if (!empty($field['enclose_text']))
			{
				$trans = array(
					'{INPUT}' => $html,
					'{ID_MSG}' => $id_msg,
					'{TEXT_PREVIOUS}' => $field['text_previous'],
					'{TEXT_NEXT}' => $field['text_next'],
				);
				$html = cffpParseTemplate($field['enclose_text'], $field['enclose_type'], $trans);
			}
			else
			{
				if (cffpIsNotEmptyString($field['text_previous']))
					$html = $field['text_previous'] . ' ' . $html;
				if (cffpIsNotEmptyString($field['text_next']))
					$html .= ' ' . $field['text_next'];
			}
		}
	}
	else // Input.
	{
		$id_cffp_group = $id;
		$size_max = 100;
		$width_total = 'width: 100%; max-width: 100%; min-width: 100%;';
		$width_max = 'max-width: 100%;';

		$name = 'cffp[' . $id_cffp_group . '][' . $name_field . ']';
		$id = 'cffp_' . $id_cffp_group . '_' . $name_field;
		$id2 = 'cffp_' . $id_cffp_group . '_' . $name_field;
		$js = '';
		switch ($field['type'])
		{
			case 'text':
			{
				if (isset($field['size']))
					$size = $field['size'] ? $field['size'] : $size_max;
				else
					$size = $field['length'] ? (int) ceil($field['length'] / 10) * 10 : $size_max;
				$size = min($size_max, $size);
				$width = $size_max == $size ? $width_total : $width_max;
				$html = '<input type="text" name="' . $name . '" id="' . $id . '" value="' . $value . '"' . (!empty($field['length']) ? ' maxlength="' . $field['length'] . '"' : '') . ' size="' . $size . '" style="' . $width . '" class="input_text" />';
				break;
			}
			case 'textarea':
			{
				list ($rows, $cols) = explode(',', $field['options']);
				$width = empty($cols) || $size_max <= $cols ? $width_total : $width_max;
				$html = '<textarea name="' . $name . '" id="' . $id . '"' . (!empty($rows) ? ' rows="' . $rows . '"' : '') . (!empty($cols) ? ' cols="' . $cols . '"' : '') . ' style="' . $width . '">' . $value . '</textarea>';
				break;
			}
			case 'select':
			case 'combobox':
			{
				$options = explode(',', $field['options']);
				$trans = array();
				$select_multiple = array_shift($options);
				$select_size = array_shift($options);
				$options_group = !empty($field['options_group']) ? $field['options_group'] : array();
				if (!empty($options_group))
				{
					$options = array();
					$field['options_text'] = '';
				}

				if ('select' == $field['type'])
				{
					$html = '<select name="' . $name . ($select_multiple ? '[]' : '') . '" id="' . $id . '"' . ($select_multiple ? ' multiple="multiple"' : '') . (1 < $select_size ? ' size="' . $select_size . '"' : '') . ' style="' . $width_max . '">';
					if (cffpIsNotEmptyString($field['options_text']))
						$trans = array_combine($options, explode(',', $field['options_text']));
					if (empty($options_group) && !empty($field['show_filter_used_options']))
					{
						$add_empty_value = !empty($options) && strtr($txt['cffp_field_empty_value'], array(',' => '')) == $options[0];
						$options = cffpGetOptionsUsed($field['id_group'], $name_field, $field['show_filter_used_options'], $add_empty_value);
					}
				}
				// combobox.
				else
				{
					if (empty($options_group))
					{
						if (empty($field['show_filter_used_options']))
							$options = cffpGetOptionsCombobox($field['id_group'], $name_field, $options);
						else
						{
							$add_empty_value = !empty($options) && strtr($txt['cffp_field_empty_value'], array(',' => '')) == $options[0];
							$options = cffpGetOptionsUsed($field['id_group'], $name_field, $field['show_filter_used_options'], $add_empty_value);
						}
					}

					if (isset($field['size']))
						$size = $field['size'] ? $field['size'] : $size_max;
					else
						$size = $field['length'] ? (int) ceil($field['length'] / 10) * 10 : $size_max;
					$size = min($size_max, $size);
					$width = $size_max == $size ? $width_total : 'width: ' . ($size * 6) . 'px; ' . $width_max;

					$select_name = 'cffp[' . $id_cffp_group . '][' . $name_field . '_select]';
					$select_id = 'cffp_' . $id_cffp_group . '_' . $name_field . '_select';

					if (!empty($field['dropdown']))
					{
						$select_multiple = 0;
						$select_size = 1;
						$select_width = $width_max;
						$br = '';
					}
					else
					{
						$select_width = $width;
						$br = '<br />';
					}

					$html =
						'<input type="text" name="' . $name . '" id="' . $id . '" value="' . $value . '"' . (!empty($field['length']) && !$select_multiple ? ' maxlength="' . $field['length'] . '"' : '') . ' size="' . $size . '" style="' . $width . '" class="input_text" />' .
						$br .
						'<select name="' . $select_name . ($select_multiple ? '[]' : '') . '" id="' . $select_id . '"' . ($select_multiple ? ' multiple="multiple"' : '') . (1 < $select_size ? ' size="' . $select_size . '"' : '') . ' style="' . $select_width . '">';
				}

				if ($select_multiple)
				{
					$value_array = cffpIsNotEmptyString($value) ? explode(',', $value) : array();
					$no_checked_array = array_flip($value_array);
					if (empty($options_group))
					{
						foreach ($options as $v)
						{
							$checked = !empty($value_array) && cffpIsNotEmptyString($v) && in_array($v, $value_array);
							$html .= '<option value="' . $v . '"' . ($checked ? ' selected="selected"' : '') . '>' . (isset($trans[$v]) ? $trans[$v] : $v) . '</option>';
							if ($checked)
								unset($no_checked_array[$v]);
						}
					}
					else
					{
						if (isset($options_group[0]['settings']))
						{
							if (!empty($options_group[0]['settings']['show_empty_option']))
							{
								$checked = empty($value_array) || in_array('', $value_array);
								if ($checked)
								{
									$value_array = array();
									$no_checked_array = array();
								}
								if ($checked || !$field['not_empty_value'])
									$html .= '<option value=""' . ($checked ? ' selected="selected"' : '') . '>' . (isset($field['empty_option_text']) ? $field['empty_option_text'] : '') . '</option>';
							}
							unset($options_group[0]);
						}

						foreach ($options_group as $option_group)
						{
							if (isset($option_group['name']))
								$html .= '<optgroup label="' . $option_group['name'] . '">';
							foreach ($option_group['options'] as $op)
							{
								$checked = !empty($value_array) && in_array($op['value'], $value_array);
								$html .= '<option value="' . $op['value'] . '"' . ($checked ? ' selected="selected"' : '') . '>' . $op['text'] . '</option>';
								if ($checked)
									unset($no_checked_array[$op['value']]);
							}
							if (isset($option_group['name']))
								$html .= '</optgroup>';
						}
					}
					foreach ($no_checked_array as $v => $i)
						$html .= '<option value="' . $v . '" selected="selected">' . $v . '</option>';
				}
				else // No select multiple.
				{
					$checked = !cffpIsNotEmptyString($value);
					$no_checked = !$checked;
					if ($checked || !$field['not_empty_value'])
						$html .= '<option value=""' . ($checked ? ' selected="selected"' : '') . '>' . (isset($field['empty_option_text']) ? $field['empty_option_text'] : '') . '</option>';
					if (empty($options_group))
					{
						foreach ($options as $v)
						{
							$checked = cffpIsNotEmptyString($v) && cffpIsNotEmptyString($value) && $value == $v;
							$html .= '<option value="' . $v . '"' . ($checked ? ' selected="selected"' : '') . '>' . (isset($trans[$v]) ? $trans[$v] : $v) . '</option>';
							if ($checked)
								$no_checked = false;
						}
					}
					else
					{
						foreach ($options_group as $option_group)
						{
							if (isset($option_group['name']))
								$html .= '<optgroup label="' . $option_group['name'] . '">';
							foreach ($option_group['options'] as $op)
							{
								$checked = cffpIsNotEmptyString($value) && $value == $op['value'];
								$html .= '<option value="' . $op['value'] . '"' . ($checked ? ' selected="selected"' : '') . '>' . $op['text'] . '</option>';
								if ($checked)
									$no_checked = false;
							}
							if (isset($option_group['name']))
								$html .= '</optgroup>';
						}
					}
					if ($no_checked)
						$html .= '<option value="' . $value . '" selected="selected">' . $value . '</option>';
				}
				$html .= '</select>';

				if ('combobox' == $field['type'])
				{
					$js = 'document.getElementById(\'' . $select_id . '\').onchange = function (event)
{
	cffpOnchangeCombobox(document.getElementById(\'' . $id . '\'), this);
}';
				}
				break;
			}
			case 'radio':
			{
				$id = '';
				$options = explode(',', $field['options']);
				$trans = cffpIsNotEmptyString($field['options_text']) ? array_combine($options, explode(',', $field['options_text'])) : array();
				$html = '';
				$no_checked = true;
				foreach ($options as $v)
				{
					$checked = cffpIsNotEmptyString($v) && cffpIsNotEmptyString($value) && $value == $v;
					$html .= '<label><input type="radio" name="' . $name . '" value="' . $v . '"' . ($checked ? ' checked="checked"' : '') . ' class="input_radio" /> ' . (isset($trans[$v]) ? $trans[$v] : $v) . '</label><br />';
					if ($checked)
						$no_checked = false;
				}
				if ($no_checked && cffpIsNotEmptyString($value))
					$html .= '<label><input type="radio" name="' . $name . '" value="' . $value . '" checked="checked" class="input_radio" /> ' . $value . '</label>';
				break;
			}
			case 'check':
			{
				$id = '';
				$options = explode(',', $field['options']);
				$trans = cffpIsNotEmptyString($field['options_text']) ? array_combine($options, explode(',', $field['options_text'])) : array();
				$value_array = cffpIsNotEmptyString($value) ? explode(',', $value) : array();
				$html = '';
				$no_checked_array = array_flip($value_array);
				foreach ($options as $v)
				{
					$checked = !empty($value_array) && cffpIsNotEmptyString($v) && in_array($v, $value_array);
					$html .= '<label><input type="checkbox" name="' . $name . '[]" value="' . $v . '"' . ($checked ? ' checked="checked"' : '') . ' class="input_check" /> ' . (isset($trans[$v]) ? $trans[$v] : $v) . '</label><br />';
					if ($checked)
						unset($no_checked_array[$v]);
				}
				foreach ($no_checked_array as $v => $i)
					$html .= '<label><input type="checkbox" name="' . $name . '[]" value="' . $v . '" checked="checked" class="input_check" /> ' . $v . '</label><br />';
				break;
			}
			case 'boolean':
			{
				$html = '<input type="checkbox" name="' . $name . '" id="' . $id . '" value="1"' . ($value ? ' checked="checked"' : '') . ' class="input_check" />';
				break;
			}
		}

		// Enclosing the input button within some other text.
		if (cffpIsNotEmptyString($html))
		{
			if (cffpIsNotEmptyString($field['text_previous']))
				$html = $field['text_previous'] . ' ' . $html;
			if (cffpIsNotEmptyString($field['text_next']))
				$html .= ' ' . $field['text_next'];
		}
	}

	return $output ? $html : compact('html', 'id', 'id2', 'js');
}

// Returns HTML code of filter operator.
function cffpHtmlOfFilterOperator($select_multiple, $value, $id_group, $name_field)
{
	global $smcFunc, $txt;

	switch ($name_field)
	{
		case 'subject': // Predefined filters type text.
		case 'posted_by':
		{
			$operators = array('=', '<>', 'LIKE', 'NOT LIKE');
			break;
		}
		case 'subject_start': // Predefined filter type text.
		case 'subject_first_char': // Predefined filters type select.
		case 'replies':
		{
			$operators = array('=', '<>');
			break;
		}
		case 'date_topic': // Predefined filters type text (number of days).
		case 'date_replies':
		{
			$operators = array('<=', '>');
			break;
		}
		default: // Filters of fields.
		{
			$operators = $select_multiple ? array('=', '<>', 'LIKE', 'NOT LIKE') : array('=', '<>', '<', '<=', '>', '>=', 'LIKE', 'NOT LIKE');
			break;
		}
	}
	$trans = array(
		'LIKE' => $txt['cffp_show_filter_operator_like'],
		'NOT LIKE' => $txt['cffp_show_filter_operator_not_like'],
	);

	$name = 'cffp_operator[' . $id_group . '][' . $name_field . ']';
	$id = 'cffp_operator_' . $id_group . '_' . $name_field;

	$html = '<select name="' . $name . '" id="' . $id . '">';
	foreach ($operators as $v)
		$html .= '<option value="' . $v . '"' . ($value == $v ? ' selected="selected"' : '') . '>' . $smcFunc['htmlspecialchars'](isset($trans[$v]) ? $trans[$v] : $v, ENT_QUOTES) . '</option>';
	$html .= '</select>';

	return $html;
}

// Indicates if group (with $boards and $child_boards attributes) is allowed in current board.
function cffpIsGroupAllowInBoard($boards, $child_boards, $current_board = 0)
{
	if (empty($boards))
		$allow = false;
	elseif ('all' == $boards)
		$allow = true;
	else
	{
		if (empty($current_board))
		{
			global $board;
			$current_board = $board;
		}
		$board_array = is_numeric($boards) ? array($boards) : explode(',', $boards);
		$board_array = cffpAddChildrenBoards($board_array, $child_boards);
		$allow = in_array($current_board, $board_array);
	}
	return $allow;
}

// Indicates if is not an empty string.
function cffpIsNotEmptyString($string)
{
	return !empty($string) || is_numeric($string);
}

// Indicates if user is owner of message.
function cffpIsOwner($new_msg, $id_member_poster)
{
	global $context;

	return !$context['user']['is_guest'] && ($new_msg || $context['user']['id'] == $id_member_poster);
}

// Loads custom post fields of a post.
function cffpLoad($id_msg, $id_member_poster, $cffp_action, $current_board = 0, $id_group = 0, $search_page_name = false)
{
	global $context, $txt, $settings, $boardurl, $scripturl, $modSettings, $user_info, $sourcedir;

	$user = cffpGetTypeUser();

	$action_readonly = '_readonly' == substr($cffp_action, -9);

	if (in_array($cffp_action, array('order_by', 'filters')) || 'search' == substr($cffp_action, 0, 6))
		$data_msg = array();
	else
	{
		if (empty($id_msg))
			global $topic;

		$owner = cffpIsOwner(empty($id_msg), $id_member_poster);
		$data_msg = cffpGetDataMsgTopic($id_msg, !empty($topic) ? $topic : 0);
	}
	$context['cffp'][$cffp_action] = array();

	$group_array = array();
	$field_array = array();
	switch ($cffp_action)
	{
		case 'display':
		case 'display_readonly':
		case 'message_index':
		{
			$fields_value = cffpGetValueOfFieldsByMsg($id_msg, false);
			if (!empty($id_group))
				$fields_value = isset($fields_value[$id_group]) ? array($id_group => $fields_value[$id_group]) : array();
			if (empty($fields_value))
				return;

			$group_array = cffpGetGroupsAndFieldsOfGroup($data_msg, array_keys($fields_value), $cffp_action, $current_board, $user, $owner);
			foreach ($fields_value as $id_group => $fv)
			{
				if (!isset($group_array[$id_group]))
					continue;

				$group_array[$id_group]['poster_name'] = $fv['poster_name'];
				$group_array[$id_group]['poster_time'] = $fv['poster_time'];
				$group_array[$id_group]['modified_name'] = $fv['modified_name'];
				$group_array[$id_group]['modified_time'] = $fv['modified_time'];

				foreach ($group_array[$id_group]['settings_fields'] as $row)
				{
					$name_field = 'column_' . $row['id_column'];
					$value = $fv[$name_field];

					$html = cffpHtmlOfField($row, $value, true, $id_msg);
					if ((('message_index' == $cffp_action && !$group_array[$id_group]['show_empty_fields_message_index']) || ('message_index' != $cffp_action && !$group_array[$id_group]['show_empty_fields_post'])) && !cffpIsNotEmptyString($html))
					{
						unset($group_array[$id_group]['settings_fields'][$row['id_field']]);
						continue;
					}

					$field_array[$id_group][$row['id_field']] = array(
						'name' => $row['name'],
						'html' => $html,
					);
				}

				if (empty($field_array[$id_group]))
					unset($group_array[$id_group]);
			}
			break;
		}
		case 'display_member':
		case 'display_member_topic_begin_all':
		case 'display_member_topic_end_all':
		case 'display_member_topic_begin_first':
		case 'display_member_topic_end_first':
		case 'display_member_readonly':
		case 'display_member_topic_begin_all_readonly':
		case 'display_member_topic_end_all_readonly':
		case 'display_member_topic_begin_first_readonly':
		case 'display_member_topic_end_first_readonly':
		{
			$group_array = cffpGetGroupsAndFieldsOfGroup($data_msg, 0, $cffp_action, $current_board);
			if (empty($group_array))
				return;

			$fields_value = cffpGetValueOfFieldsByMsg($id_msg, true);

			foreach ($group_array as $id_group => $group)
			{
				$fvm = isset($fields_value[$id_group]) ? $fields_value[$id_group] : array();
				$group_fields_own = array();
				$group_fields_array = array();
				foreach ($fvm as $fv)
				{
					$owner2 = cffpIsOwner(false, $fv['id_member']);
					$can_view = cffpHavePermission($user, $owner2, 'view', $group['permissions']);
					$can_modify = !$action_readonly && cffpHavePermission($user, $owner2, 'modify', $group['permissions']);
					$can_remove = !$action_readonly && cffpHavePermission($user, $owner2, 'remove', $group['permissions']);
					$can_move = !$action_readonly && cffpHavePermission($user, $owner2, 'move', $group['permissions']);
					if (!$can_view && !$can_modify && !$can_remove)
						continue;

					$field_array2 = array();
					if ($can_view)
					{
						foreach ($group['settings_fields'] as $row)
						{
							$can_view2 = cffpHavePermission($user, $owner2, 'view', $row['permissions']);
							if (!$can_view2)
								continue;

							$name_field = 'column_' . $row['id_column'];
							$value = $fv[$name_field];

							$html = cffpHtmlOfField($row, $value, true, $id_msg);
							if (!$group['show_empty_fields_post'] && !cffpIsNotEmptyString($html))
								continue;

							$field_array2[$row['id_field']] = array(
								'name' => $row['name'],
								'html' => $html,
							);
						}
					}
					if (empty($field_array2))
						continue;

					$group_fields = array(
						'id_cffp' => $fv['id_cffp'],
						'id_member' => $fv['id_member'],
						'poster_name' => $fv['poster_name'],
						'poster_time' => $fv['poster_time'],
						'modified_name' => $fv['modified_name'],
						'modified_time' => $fv['modified_time'],
						'can_view' => $can_view,
						'can_modify' => $can_modify,
						'can_remove' => $can_remove,
						'can_move' => $can_move,
						'field_array' => $field_array2,
					);
					if ($owner2 && $group['member_one_value'])
						$group_fields_own = $group_fields;
					else
						$group_fields_array[] = $group_fields;
				}

				$owner2 = cffpIsOwner(true, $context['user']['id']);
				$can_add = !$action_readonly && cffpHavePermission($user, $owner2, 'add', $group['permissions']) && empty($group_fields_own);
				$add_group_fields = array();
				if ($can_add)
				{
					foreach ($group['settings_fields'] as $row)
					{
						$can_view2 = cffpHavePermission($user, $owner2, 'view', $row['permissions']);
						if (!$can_view2)
							continue;

						$name_field = 'column_' . $row['id_column'];
						$value = $row['default_value'];

						$row['show_filter_used_options'] = 0;
						$add_group_fields['field_array'][$row['id_field']] = array(
							'name' => $row['name'],
							'field_desc' => $row['field_desc'],
							'field_long_desc' => parse_bbc(censorText($row['field_long_desc'])),
							'html' => cffpHtmlOfField($row, $value, false, $id_msg . '_group' . $id_group, $name_field),
						);
					}

					if (!empty($add_group_fields))
					{
						if ($context['user']['is_guest'])
						{
							$add_group_fields['guest_name'] = isset($_SESSION['guest_name']) ? $_SESSION['guest_name'] : '';

							if (empty($modSettings['guest_post_no_email']))
								$add_group_fields['guest_email'] = isset($_SESSION['guest_email']) ? $_SESSION['guest_email'] : '';
						}

						// Do we need to show the visual verification image?
						$context['require_verification'] = !$user_info['is_mod'] && !$user_info['is_admin'] && !empty($modSettings['posts_require_captcha']) && ($user_info['posts'] < $modSettings['posts_require_captcha'] || ($user_info['is_guest'] && $modSettings['posts_require_captcha'] == -1));
						if ($context['require_verification'])
						{
							require_once($sourcedir . '/Subs-Editor.php');
							$verificationOptions = array(
								'id' => 'cffp_' . $id_msg . '_group' . $id_group,
							);
							$context['require_verification'] = create_control_verification($verificationOptions);
							$context['visual_verification_id'] = $verificationOptions['id'];
						}
					}
				}

				$stats = cffpLoadStatsOfGroupByMsg($id_group, $id_msg, true);
				$can_stats = cffpHavePermission($user, false, 'stats', $group['permissions']);
				$stats = !empty($stats) && $can_stats ? cffpGetStats($stats[$id_msg], $group, 'post') : array();

				if (empty($group_fields_array) && empty($group_fields_own) && empty($add_group_fields) && empty($stats))
				{
					unset($group_array[$id_group]);
					continue;
				}

				if (!empty($group['show_member_expanded_data_recent']) && !empty($group_fields_array) && count($group_fields_array) > $group['show_member_expanded_data_recent'])
				{
					$group_fields_previous_array = $group_fields_array;
					$group_fields_recent_array = array();
					for ($k = 0; $k < $group['show_member_expanded_data_recent']; $k++)
					{
						$recent = array_pop($group_fields_previous_array);
						if (empty($recent))
							break; // Exit for.

						$group_fields_recent_array[] = $recent;
					}

					if ($group['show_member_recent_at_top'])
						$group_fields_previous_array = array_reverse($group_fields_previous_array);
					else
						$group_fields_recent_array = array_reverse($group_fields_recent_array);
				}
				else
				{
					$group_fields_previous_array = array();
					$group_fields_recent_array = $group_fields_array;

					if ($group['show_member_recent_at_top'])
						$group_fields_recent_array = array_reverse($group_fields_recent_array);
				}

				if (!empty($group_fields_own))
				{
					if ($group['show_member_recent_at_top'])
						array_unshift($group_fields_recent_array, $group_fields_own);
					else
						$group_fields_recent_array[] = $group_fields_own;
				}

				$field_array[$id_group] = array(
					'stats' => $stats,
					'group_fields_array' => array(
						'previous' => $group_fields_previous_array,
						'recent' => $group_fields_recent_array,
					),
					'add_group_fields' => $add_group_fields,
				);
			}

			if (isset($_GET['cffp_show']))
			{
				list ($show_msg, $show_group) = explode(',', $_GET['cffp_show']);
				if ($id_msg == $show_msg && isset($group_array[$show_group]))
				{
					if (0 == $group_array[$show_group]['show_member_expanded'])
						$group_array[$show_group]['show_member_expanded'] = 1;
					if (0 == $group_array[$show_group]['show_member_expanded_data'])
						$group_array[$show_group]['show_member_expanded_data'] = 1;
				}
			}
			break;
		}
		case 'message_index_member':
		{
			$group_array = cffpGetGroupsAndFieldsOfGroup($data_msg, 0, $cffp_action, $current_board);
			if (empty($group_array))
				return;

			$fields_value = cffpGetValueOfFieldsByMsg($id_msg, true);

			foreach ($group_array as $id_group => $group)
			{
				$fvm = isset($fields_value[$id_group]) ? $fields_value[$id_group] : array();
				$group_fields_array = array();
				foreach ($fvm as $fv)
				{
					$owner2 = cffpIsOwner(false, $fv['id_member']);
					$can_view = cffpHavePermission($user, $owner2, 'view', $group['permissions']);
					if (!$can_view)
						continue;

					$field_array2 = array();
					foreach ($group['settings_fields'] as $row)
					{
						$can_view2 = cffpHavePermission($user, $owner2, 'view', $row['permissions']);
						if (!$can_view2)
							continue;

						$name_field = 'column_' . $row['id_column'];
						$value = $fv[$name_field];

						$html = cffpHtmlOfField($row, $value, true, $id_msg);
						if (!$group['show_empty_fields_message_index'] && !cffpIsNotEmptyString($html))
							continue;

						$field_array2[$row['id_field']] = array(
							'name' => $row['name'],
							'html' => $html,
						);
					}
					if (empty($field_array2))
						continue;

					$group_fields_array[] = array(
						'id_cffp' => $fv['id_cffp'],
						'id_member' => $fv['id_member'],
						'poster_name' => $fv['poster_name'],
						'poster_time' => $fv['poster_time'],
						'modified_name' => $fv['modified_name'],
						'modified_time' => $fv['modified_time'],
						'field_array' => $field_array2,
					);
				}

				$stats = cffpLoadStatsOfGroupByMsg($id_group, $id_msg, true);
				$can_stats = cffpHavePermission($user, false, 'stats', $group['permissions']);
				$stats = !empty($stats) && $can_stats ? cffpGetStats($stats[$id_msg], $group, 'message_index') : array();

				if (empty($group_fields_array) && empty($stats))
				{
					unset($group_array[$id_group]);
					continue;
				}

				if (!empty($group_fields_array) && $group['show_member_recent_at_top'])
					$group_fields_array = array_reverse($group_fields_array);

				$field_array[$id_group] = array(
					'stats' => $stats,
					'group_fields_array' => $group_fields_array,
				);
			}
			break;
		}
		case 'post':
		{
			$group_array = cffpGetGroupsAndFieldsOfGroup($data_msg, 0, $cffp_action, $current_board, $user, $owner);
			if (empty($group_array))
				return;

			$fields_value = cffpGetValueOfFieldsByMsg($id_msg, false);

			foreach ($group_array as $id_group => $group)
			{
				$fv = isset($fields_value[$id_group]) ? $fields_value[$id_group] : array();
				if ((empty($fv) && !$group['can_add'] && 'record' == $group['type']) || (!empty($fv) && !$group['can_modify'] && !$group['can_remove']))
				{
					unset($group_array[$id_group]);
					continue;
				}

				$group_array[$id_group]['id_cffp'] = !empty($fv) ? $fv['id_cffp'] : 0;
				$group_array[$id_group]['set'] = isset($_REQUEST['cffp'][$id_group]) && ((empty($fv) && $group['can_add']) || (!empty($fv) && $group['can_remove'])) ? isset($_REQUEST['cffp'][$id_group]['set']) : !empty($fv);

				if (!empty($id_msg) && empty($fv) && $context['user']['is_guest'])
				{
					if (isset($_SESSION['guest_name']))
						$group_array[$id_group]['guest_name'] = $_SESSION['guest_name'];
					else
						$group_array[$id_group]['guest_name'] = isset($_REQUEST['cffp'][$id_group]['guest_name']) ? $_REQUEST['cffp'][$id_group]['guest_name'] : '';

					if (empty($modSettings['guest_post_no_email']))
					{
						if (isset($_SESSION['guest_email']))
							$group_array[$id_group]['guest_email'] = $_SESSION['guest_email'];
						else
							$group_array[$id_group]['guest_email'] = isset($_REQUEST['cffp'][$id_group]['guest_email']) ? $_REQUEST['cffp'][$id_group]['guest_email'] : '';
					}
				}

				$group_array[$id_group]['can_modify_fields'] = empty($fv) || $group['can_modify'];
				$group_array[$id_group]['can_view_fields'] = !$group_array[$id_group]['can_modify_fields'] && $group['can_remove'] && $group['can_view'];
				if (!$group_array[$id_group]['can_modify_fields'] && !$group_array[$id_group]['can_view_fields'])
					continue;

				foreach ($group['settings_fields'] as $row)
				{
					$name_field = 'column_' . $row['id_column'];

					if ($group_array[$id_group]['can_modify_fields'])
					{
						$value = cffpGetRequestValueOfField($id_group, $name_field, $row['type'], $row['length'], $row['options'], !empty($fv) ? $fv[$name_field] : $row['default_value']);
						$row['show_filter_used_options'] = 0;
						$html = cffpHtmlOfField($row, $value, false, $id_group, $name_field);
					}
					else // $group_array[$id_group]['can_view_fields'].
					{
						$value = !empty($fv) ? $fv[$name_field] : $row['default_value'];
						$html = cffpHtmlOfField($row, $value, true, $id_msg);
					}

					$field_array[$id_group][$row['id_field']] = array(
						'name' => $row['name'],
						'field_desc' => $row['field_desc'],
						'field_long_desc' => parse_bbc(censorText($row['field_long_desc'])),
						'html' => $html,
					);
				}
			}
			break;
		}
		case 'order_by':
		{
			$group_array = cffpGetGroupsAndFieldsOfGroup(array(), $id_group, $cffp_action, 0, $user);
			if (empty($group_array))
				return;

			if (!empty($_REQUEST['cffp_order_by']))
				$cffp_order_by = $_REQUEST['cffp_order_by'];
			elseif (!empty($_REQUEST['sort']))
				$cffp_order_by = $_REQUEST['sort'] . ' ' . (isset($_REQUEST['desc']) ? 'desc' : 'asc');
			else
				$cffp_order_by = '';
			$sort_array = array();
			$option_array = array();
			$no_checked = true;
			$group_checked = 0;
			foreach ($group_array as $id_group => $group)
			{
				if ($group['show_stats'])
				{
					$group = $context['cffp']['settings_groups'][$id_group];
					$show_stats = 1 == count($group['settings_fields']) && !empty($group['show_stats_order_by']);
					if (!$show_stats)
					{
						unset($group_array[$id_group]);
						continue;
					}

					$settings_field = current($group['settings_fields']);
					switch ($settings_field['type'])
					{
						case 'select':
						{
							$options = explode(',', $settings_field['options']);
							array_shift($options);
							array_shift($options);
							break;
						}
						case 'radio':
						case 'check':
						{
							$options = explode(',', $settings_field['options']);
							break;
						}
						default:
						{
							$show_stats = 0;
							break;
						}
					}
					if (!$show_stats)
					{
						unset($group_array[$id_group]);
						continue;
					}

					$is_numeric = true;
					foreach ($options as $value)
					{
						if (!is_numeric($value))
						{
							$is_numeric = false;
							break; // Exit foreach.
						}
					}

					$show_stats_order_by = explode(',', $group['show_stats_order_by']);
					if (!$is_numeric)
					{
						$show_stats_order_by = array_diff($show_stats_order_by, array('average', 'sum', 'highest', 'lowest'));
						if (empty($show_stats_order_by))
						{
							unset($group_array[$id_group]);
							continue;
						}
					}

					foreach ($show_stats_order_by as $order_by)
					{
						$sort = 'cffps' . $id_group . '.' . $order_by;
						$value_asc = $sort . ' asc';
						$value_desc = $sort . ' desc';
						$checked_asc = $cffp_order_by == $value_asc;
						$checked_desc = $cffp_order_by == $value_desc;
						if ($checked_asc || $checked_desc)
						{
							$no_checked = false;
							$group_checked = 'stats_' . $id_group;
						}

						$sort_array[] = $sort;
						$option_array[$id_group][] = array(
							'value' => $value_asc,
							'checked' => $checked_asc,
							'text' => $txt['cffp_show_stats_summary_' . $order_by] . ' (' . $txt['cffp_order_by_asc'] . ')',
						);
						$option_array[$id_group][] = array(
							'value' => $value_desc,
							'checked' => $checked_desc,
							'text' => $txt['cffp_show_stats_summary_' . $order_by] . ' (' . $txt['cffp_order_by_desc'] . ')',
						);
					}
				}
				else
				{
					foreach ($group['settings_fields'] as $row)
					{
						$sort = 'cffpf' . $id_group . '.column_' . $row['id_column'];
						$value_asc = $sort . ' asc';
						$value_desc = $sort . ' desc';
						$checked_asc = $cffp_order_by == $value_asc;
						$checked_desc = $cffp_order_by == $value_desc;
						if ($checked_asc || $checked_desc)
						{
							$no_checked = false;
							$group_checked = $id_group;
						}

						$sort_array[] = $sort;
						$option_array[$id_group][] = array(
							'value' => $value_asc,
							'checked' => $checked_asc,
							'text' => $row['name'] . ' (' . $txt['cffp_order_by_asc'] . ')',
						);
						$option_array[$id_group][] = array(
							'value' => $value_desc,
							'checked' => $checked_desc,
							'text' => $row['name'] . ' (' . $txt['cffp_order_by_desc'] . ')',
						);
					}
				}

				if (empty($option_array[$id_group]))
					unset($group_array[$id_group]);
			}

			if (empty($sort_array))
				return;

			$field_array = array(
				'sort_array' => $sort_array,
				'option_array' => $option_array,
				'no_checked' => $no_checked,
				'group_checked' => $group_checked,
			);
			break;
		}
		case 'filters':
		case 'search_posts':
		case 'search_posts_menu':
		case 'search_records':
		case 'search_records_menu':
		{
			$group_array = cffpGetGroupsAndFieldsOfGroup(array(), $id_group, $cffp_action, 0, $user, false, $search_page_name);
			if (empty($group_array))
				return;

			$size = 'filters' == $cffp_action ? 15 : 0;
			$filter_index = 'filters' == $cffp_action ? 0 : 1;
			$dropdown = 'filters' == $cffp_action;
			$action_menu = '_menu' == substr($cffp_action, -5);
			foreach ($group_array as $id_group => $group)
			{
				if ('filters' == $group['type'])
				{
					foreach ($group['settings_fields'] as $row)
					{
						if (('sticky' == $row['id'] && !$modSettings['enableStickyTopics']) || empty($row['field'][$filter_index]))
						{
							unset($group_array[$id_group]['settings_fields'][$row['id']]);
							continue;
						}

						if ('filters' != $cffp_action)
						{
							if (('date_topic' == $row['id'] && isset($field_array[$id_group]['date_replies'])) || ('date_replies' == $row['id'] && isset($field_array[$id_group]['date_topic'])))
							{
								unset($group_array[$id_group]['settings_fields'][$row['id']]);
								continue;
							}
						}

						if (!$action_menu)
						{
							$name_field = $row['id'];
							switch ($row['id'])
							{
								case 'boards':
								{
									require_once ($sourcedir . '/Subs-MessageIndex.php');
									$boardListOptions = array(
										'not_redirection' => true,
									);
									if ('all' != $group['boards'])
									{
										$board_array = is_numeric($group['boards']) ? array($group['boards']) : explode(',', $group['boards']);
										$boardListOptions['included_boards'] = cffpAddChildrenBoards($board_array, $group['child_boards']);
									}
									$all_boards = getBoardList($boardListOptions);

									$options_group = array(
										array(
											'settings' => array(
												'show_empty_option' => true,
											),
										),
									);

									foreach ($all_boards as $category)
									{
										$options = array();
										foreach ($category['boards'] as $board)
										{
											$options[] = array(
												'value' => $board['id'],
												'text' => ($board['child_level'] > 0 ? str_repeat('==', $board['child_level'] - 1) . '=&gt;' : '') . ' ' . $board['name'],
											);
										}
										$options_group[] = array(
											'name' => $category['name'],
											'options' => $options,
										);
									}
									$row['options_group'] = $options_group;
									break;
								}
								case 'subject':
								case 'subject_start':
								case 'posted_by':
								{
									$row['size'] = $size;
									break;
								}
								default:
								{
									// VOID
									break;
								}
							}
							$row['show_filter_used_options'] = 0;
							$row['not_empty_value'] = 0;
							$row['empty_option_text'] = $txt['all'];
							$value = cffpGetRequestValueOfField($id_group, $name_field, $row['type'], $row['length'], $row['options']);
							$error_code = cffpValidateMask($row['mask'], $value);
							if (!empty($error_code) || ('date_' == substr($row['id'], 0, 5) && empty($value)) || ('boards' == $row['id'] && (empty($value) || ',' == substr($value, 0, 1))))
							{
								$_REQUEST['cffp'][$id_group][$name_field] = '';
								$value = '';
							}
							elseif ('boards' == $row['id'])
								$_REQUEST['cffp'][$id_group][$name_field] = $value;

							$html = cffpHtmlOfField($row, $value, false, $id_group, $name_field);
							if ('date_' == substr($row['id'], 0, 5))
								$html['html'] = sprintf($txt['cffp_show_filters_date_lastdays'], $html['html']);

							if ($row['show_filter_operator'])
							{
								$operator = cffpGetRequestFilterOperator($id_group, $name_field, $row['type']);
								$html_operator = cffpHtmlOfFilterOperator(false, $operator, $id_group, $name_field);
							}
							else
								$html_operator = '';

							$field_array[$id_group][$row['id']] = array(
								'name' => $row['name'][$filter_index],
								'field_desc' => '',
								'field_long_desc' => '',
								'html' => $html,
								'html_operator' => $html_operator,
							);
						}
					}

					if (!$action_menu && empty($field_array[$id_group]))
					{
						unset($group_array[$id_group]);
						continue;
					}
				}
				elseif (!$action_menu)
				{
					foreach ($group['settings_fields'] as $row)
					{
						$name_field = 'column_' . $row['id_column'];
						$select_multiple = false;
						switch ($row['type_filter'])
						{
							case 'text': // type IN (text, textarea, select, combobox, radio, check).
							{
								if ('textarea' == $row['type'])
									list ($rows, $size_orig) = explode(',', $row['options']);
								elseif (!empty($row['length']))
									$size_orig = (int) ceil($row['length'] / 10) * 10;
								else
									$size_orig = 0;

								if (!empty($size_orig))
									$row['size'] = !empty($size) ? min($size, $size_orig) : $size_orig;
								else
									$row['size'] = $size;

								$row['length'] = 0;
								break;
							}
							case 'select': // type IN (select, combobox, radio, check, boolean).
							case 'combobox': // type IN (select, combobox, radio, check).
							{
								if ('combobox' == $row['type_filter'])
								{
									if (0 < strlen($row['options_text']))
										$row['type_filter'] = 'select';
									else
									{
										$row['dropdown'] = $dropdown;

										if ('combobox' == $row['type_filter'] && !empty($row['length']))
										{
											$size_orig = (int) ceil($row['length'] / 10) * 10;
											$row['size'] = !empty($size) ? min($size, $size_orig) : $size_orig;
										}
										else
											$row['size'] = $size;

										$row['length'] = 0;
									}
								}

								if (in_array($row['type'], array('radio', 'check')))
								{
									if ($row['not_empty_value'] || !$group['can_modify'])
										$row['options'] = '0,1' . (0 < strlen($row['options']) ? ',' . $row['options'] : '');
									else
									{
										$row['options'] = '0,1,' . strtr($txt['cffp_field_empty_value'], array(',' => '')) . (0 < strlen($row['options']) ? ',' . $row['options'] : '');
										if (0 < strlen($row['options_text']))
											$row['options_text'] = strtr($txt['cffp_field_empty_value'], array(',' => '')) . ',' . $row['options_text'];
									}
									$select_multiple = 'check' == $row['type'];
								}
								elseif ('boolean' == $row['type'])
								{
									$row['options'] = '0,1,1,0';
									$row['options_text'] = strtr($txt['yes'], array(',' => '')) . ',' . strtr($txt['no'], array(',' => ''));
								}
								else // select, combobox.
								{
									$options = explode(',', $row['options']);
									$select_multiple = array_shift($options);
									array_shift($options);
									if ('combobox' == $row['type'])
										$options = cffpGetOptionsCombobox($id_group, $name_field, $options);

									if ($row['not_empty_value'] || !$group['can_modify'])
										$row['options'] = '0,1' . (!empty($options) ? ',' . implode(',', $options) : '');
									else
									{
										$row['options'] = '0,1,' . strtr($txt['cffp_field_empty_value'], array(',' => '')) . (!empty($options) ? ',' . implode(',', $options) : '');
										if (0 < strlen($row['options_text']))
											$row['options_text'] = strtr($txt['cffp_field_empty_value'], array(',' => '')) . ',' . $row['options_text'];
									}
								}
								break;
							}
							case 'empty_non_empty': // type IN (text, textarea, select, combobox, radio, check).
							{
								$row['type_filter'] = 'select';
								$row['options'] = '0,1,empty,non-empty';
								$row['options_text'] = strtr($txt['cffp_field_empty_value'], array(',' => '')) . ',' . strtr($txt['cffp_field_non_empty_value'], array(',' => ''));
								break;
							}
						}
						$row['type'] = $row['type_filter'];
						if ('filters' != $cffp_action)
							$row['show_filter_used_options'] = 0;
						elseif ($row['show_filter_used_options'])
						{
							global $board;
							$row['show_filter_used_options'] = $board;
						}
						$row['not_empty_value'] = 0;
						$row['empty_option_text'] = $txt['all'];
						$value = cffpGetRequestValueOfField($id_group, $name_field, $row['type'], $row['length'], $row['options']);
						$operator = cffpGetRequestFilterOperator($id_group, $name_field, $row['type']);

						$field_array[$id_group][$row['id_field']] = array(
							'name' => $row['name'],
							'field_desc' => 'filters' != $cffp_action ? $row['field_search_desc'] : '',
							'field_long_desc' => 'filters' != $cffp_action ? parse_bbc(censorText($row['field_search_long_desc'])) : '',
							'html' => cffpHtmlOfField($row, $value, false, $id_group, $name_field),
							'html_operator' => $row['show_filter_operator'] ? cffpHtmlOfFilterOperator($select_multiple, $operator, $id_group, $name_field) : '',
						);
					}

					if ('filters' == $cffp_action)
					{
						$group_array[$id_group]['filter_tags'] = array();
						foreach ($group['settings_fields'] as $row)
							$group_array[$id_group]['filter_tags'][$row['hide_filter']][] = $row['id_field'];
					}
				}
			}
			break;
		}
	}

	$context['cffp'][$cffp_action] = array(
		'id_msg' => $id_msg,
		'sticky' => !empty($data_msg) ? $data_msg['sticky'] : null,
		'locked' => !empty($data_msg) ? $data_msg['locked'] : null,
		'first_msg' => !empty($data_msg) ? $data_msg['first_msg'] : null,
		'subject' => !empty($data_msg) ? $data_msg['subject'] : '',
		'id_member_poster' => $id_member_poster,
		'user' => $user,
		'group_array' => $group_array,
		'field_array' => $field_array,
	);
}

// Loads data of message and topic.
function cffpLoadDataMsgTopic($id_msg_array, $id_topic_array = array())
{
	global $smcFunc, $context;

	if (!empty($id_msg_array))
	{
		if (!is_array($id_msg_array))
			$id_msg_array = array($id_msg_array);

		foreach ($id_msg_array as $k => $id_msg)
		{
			if (isset($context['cffp']['data_msg_topic'][$id_msg]))
				unset($id_msg_array[$k]);
			else
				$context['cffp']['data_msg_topic'][$id_msg] = array();
		}

		if (empty($id_msg_array))
			return;

		$request = $smcFunc['db_query']('', '
			SELECT t.is_sticky AS sticky, t.locked, m.id_msg, (t.id_first_msg = m.id_msg) AS first_msg, m.subject
			FROM {db_prefix}topics AS t
				INNER JOIN {db_prefix}messages AS m ON (m.id_topic = t.id_topic)
			WHERE id_msg IN ({array_int:id_msg_array})',
			array(
				'id_msg_array' => $id_msg_array,
			)
		);
		while ($row = $smcFunc['db_fetch_assoc']($request))
			$context['cffp']['data_msg_topic'][$row['id_msg']] = $row;
		$smcFunc['db_free_result']($request);
	}
	elseif (!empty($id_topic_array))
	{
		if (!is_array($id_topic_array))
			$id_topic_array = array($id_topic_array);

		foreach ($id_topic_array as $k => $id_topic)
		{
			if (isset($context['cffp']['data_msg_topic']['topic_' . $id_topic]))
				unset($id_topic_array[$k]);
			else
				$context['cffp']['data_msg_topic']['topic_' . $id_topic] = array();
		}

		if (empty($id_topic_array))
			return;

		$request = $smcFunc['db_query']('', '
			SELECT is_sticky AS sticky, locked, id_topic
			FROM {db_prefix}topics
			WHERE id_topic IN ({array_int:id_topic_array})',
			array(
				'id_topic_array' => $id_topic_array,
			)
		);
		while ($row = $smcFunc['db_fetch_assoc']($request))
		{
			$context['cffp']['data_msg_topic']['topic_' . $row['id_topic']] = $row;
			$context['cffp']['data_msg_topic']['topic_' . $row['id_topic']]['first_msg'] = 0;
			$context['cffp']['data_msg_topic']['topic_' . $row['id_topic']]['subject'] = '';
		}
		$smcFunc['db_free_result']($request);
	}
}

// Loads value of fields by post.
function cffpLoadValueOfFieldsByMsg($id_msg_array, $reload = false)
{
	global $smcFunc, $context, $memberContext;

	if (empty($id_msg_array))
		return;

	if (!is_array($id_msg_array))
		$id_msg_array = array($id_msg_array);

	foreach ($id_msg_array as $k => $id_msg)
	{
		if (isset($context['cffp']['fields_value'][$id_msg]) && !$reload)
			unset($id_msg_array[$k]);
		else
			$context['cffp']['fields_value'][$id_msg] = array(array(), array());
	}

	if (empty($id_msg_array))
		return;

	$request = $smcFunc['db_query']('', '
		SELECT cffpsg.type, cffpf.' . implode(', cffpf.', $context['cffp']['list_columns']['fields']) . '
		FROM {db_prefix}cffp_fields AS cffpf
			INNER JOIN {db_prefix}cffp_settings_groups AS cffpsg ON (cffpsg.id_group = cffpf.id_group)
		WHERE id_msg IN ({array_int:id_msg_array})
		ORDER BY cffpf.poster_time ASC',
		array(
			'id_msg_array' => $id_msg_array,
		)
	);
	while ($row = $smcFunc['db_fetch_assoc']($request))
	{
		if (loadMemberData($row['id_member']) && loadMemberContext($row['id_member']))
		{
			$row['poster_name'] = $memberContext[$row['id_member']]['name'];
			$row['poster_email'] = $memberContext[$row['id_member']]['email'];
			$row['poster_ip'] = $memberContext[$row['id_member']]['ip'];
		}

		// Fields of groups type member may have more than one value.
		if ('member' == $row['type'])
			$context['cffp']['fields_value'][$row['id_msg']][1][$row['id_group']][] = $row;
		// Fields of others group have one value.
		else
			$context['cffp']['fields_value'][$row['id_msg']][0][$row['id_group']] = $row;
	}
	$smcFunc['db_free_result']($request);
}

// Loads statistics of a group by post.
function cffpLoadStatsOfGroupByMsg($id_group, $id_msg_array, $reload = false)
{
	global $smcFunc, $context;

	if (empty($id_group) || empty($id_msg_array) || !$context['cffp']['settings_groups'][$id_group]['show_stats'])
		return array();

	if (!is_array($id_msg_array))
		$id_msg_array = array($id_msg_array);

	$request = $smcFunc['db_query']('', '
		SELECT ' . implode(', ', $context['cffp']['list_columns']['stats']) . '
		FROM {db_prefix}cffp_stats
		WHERE id_msg IN ({array_int:id_msg_array})
			AND id_group = {int:id_group}',
		array(
			'id_msg_array' => $id_msg_array,
			'id_group' => $id_group,
		)
	);
	$stats = array();
	while ($row = $smcFunc['db_fetch_assoc']($request))
	{
		$stats[$row['id_msg']] = array();
		foreach ($context['cffp']['list_columns']['stats'] as $column)
		{
			switch ($column)
			{
				case 'data':
				{
					$stats[$row['id_msg']][$column] = unserialize($row[$column]);
					break;
				}
				case 'options_text':
				{
					$stats[$row['id_msg']][$column] = 0 < strlen($row[$column]) ? explode(',', $row[$column]) : array();
					break;
				}
				case 'average':
				case 'sum':
				{
					$stats[$row['id_msg']][$column] = is_numeric($row[$column]) ? (float) $row[$column] : '';
					break;
				}
				case 'max':
				case 'min':
				{
					$stats[$row['id_msg']][$column . '_num'] = is_numeric($row[$column]) ? (float) $row[$column] : '';
					$stats[$row['id_msg']][$column] = is_numeric($row[$column]) ? $stats[$row['id_msg']]['data'][$row[$column]]['text'] : '';
					break;
				}
				case 'highest':
				case 'lowest':
				{
					$stats[$row['id_msg']][$column] = is_numeric($row[$column]) ? $stats[$row['id_msg']]['data'][$row[$column]]['text'] : '';
					break;
				}
				case 'most_frequent':
				case 'least_frequent':
				{
					$stats[$row['id_msg']][$column] = 0 < strlen($row[$column]) ? explode(',', $row[$column]) : array();
					foreach ($stats[$row['id_msg']][$column] as $k => $v)
						$stats[$row['id_msg']][$column][$k] = $stats[$row['id_msg']]['data'][$v]['text'];
					break;
				}
				default: // is_numeric, total.
				{
					$stats[$row['id_msg']][$column] = $row[$column];
					break;
				}
			}
		}
	}
	$smcFunc['db_free_result']($request);

	if ($reload)
	{
		if (!empty($stats))
			$id_msg_array = array_diff($id_msg_array, array_keys($stats));

		if (!empty($id_msg_array))
		{
			cffpUpdateStats($id_group, $id_msg_array);
			$stats2 = cffpLoadStatsOfGroupByMsg($id_group, $id_msg_array);
			foreach ($stats2 as $id_msg => $s2)
				$stats[$id_msg] = $s2;
		}
	}

	return $stats;
}

// Gets statistics of a group.
function cffpGetStats($stats, $group, $pos)
{
	global $txt;

	if (!empty($group['show_stats_' . $pos . '_chart_average']) && $stats['is_numeric'])
	{
		$chart_average = explode(',', $group['show_stats_' . $pos . '_chart_average']);
		$stats['chart_average'] = array(
			'show' => true,
			'average' => in_array('average', $chart_average),
			'minmax' => in_array('minmax', $chart_average),
			'options_text' => in_array('options_text', $chart_average),
		);
	}

	if (!empty($group['show_stats_' . $pos . '_chart_hbars']))
	{
		$stats['chart_hbars'] = array(
			'show' => true,
		);
	}

	if (!empty($group['show_stats_' . $pos . '_chart_vbars']))
	{
		$chart_vbars = explode(',', $group['show_stats_' . $pos . '_chart_vbars']);
		$stats['chart_vbars'] = array(
			'show' => true,
			'options_text' => in_array('options_text', $chart_vbars),
			'width_category' => floor(100 / count($stats['data']) * 100) / 100 . '%',
		);
	}

	if (!empty($group['show_stats_' . $pos . '_summary']))
	{
		$summary = explode(',', $group['show_stats_' . $pos . '_summary']);
		$stats['summary'] = array();
		foreach ($summary as $s)
		{
			if (($stats['is_numeric'] && in_array($s, array('average', 'sum', 'max', 'min', 'highest', 'lowest'))) || in_array($s, array('most_frequent', 'least_frequent', 'total')))
			{
				$stats['summary'][$s] = array(
					$txt['cffp_show_stats_summary_' . $s],
					$stats[$s],
				);
			}
		}
	}

	if (!empty($group['show_stats_' . $pos . '_value']))
	{
		if (($stats['is_numeric'] && in_array($group['show_stats_' . $pos . '_value'], array('average', 'sum', 'max', 'min', 'highest', 'lowest'))) || in_array($group['show_stats_' . $pos . '_value'], array('most_frequent', 'least_frequent', 'total')))
			$stats['value'] = $stats[$group['show_stats_' . $pos . '_value']];
	}

	if (empty($stats['chart_average']) && empty($stats['chart_hbars']) && empty($stats['chart_vbars']) && empty($stats['summary']) && !isset($stats['value']))
		$stats = array();
	elseif (!isset($stats['value']))
		$stats['text'] = $group['show_stats_' . $pos . '_text'];

	return $stats;
}

// Returns HTML code with template.
function cffpParseTemplate($text, $type, $trans = array())
{
	global $context, $boardurl, $scripturl, $settings;

	if ('php' == $type)
	{
		$trans['{SHOW_GROUP_MSG}'] = 'cffpShowGroupMsg';
		$trans['{SHOW_GROUP_MSGINDEX}'] = 'cffpShowGroupMsgIndex';
		$trans['{SHOW_GROUP_MSG_STATS}'] = 'cffpShowGroupMsgStats';
	}
	$trans = array_merge($trans, $context['cffp']['settings_vars'], array(
		'{BOARDURL}' => $boardurl,
		'{SCRIPTURL}' => $scripturl,
		'{IMAGES_URL}' => $settings['images_url'],
		'{DEFAULT_IMAGES_URL}' => $settings['default_images_url'],
	));
	$text = strtr($text, $trans);

	switch ($type)
	{
		case 'bbc':
		{
			$text = parse_bbc(censorText($text));
			break;
		}
		case 'php':
		{
			$text = trim($text);

			if ('<?php' == substr($text, 0, 5))
				$text = substr($text, 5);
			if ('?>' == substr($text, -2))
				$text = substr($text, 0, -2);

			ob_start();
			cffpEval($text);
			$text = ob_get_contents();
			ob_end_clean();
			break;
		}
		case 'html':
		default:
		{
			// VOID
			break;
		}
	}

	return $text;
}

// Shows a group of a message.
function cffpShowGroup($cffp_action, $id_msg, $id_group)
{
	global $smcFunc, $context;

	$request = $smcFunc['db_query']('', '
		SELECT id_member, id_board
		FROM {db_prefix}messages
		WHERE id_msg = {int:id_msg}',
		array(
			'id_msg' => $id_msg,
		)
	);
	$row = $smcFunc['db_fetch_assoc']($request);
	$smcFunc['db_free_result']($request);

	if (!$row)
		return;

	cffpLoadValueOfFieldsByMsg($id_msg);
	cffpLoadDataMsgTopic($id_msg);

	if (isset($context['cffp'][$cffp_action]))
		$context_cffp_action = $context['cffp'][$cffp_action];

	cffpLoad($id_msg, $row['id_member'], $cffp_action, $row['id_board'], $id_group);
	template_cffp_show($cffp_action);

	if (isset($context_cffp_action))
		$context['cffp'][$cffp_action] = $context_cffp_action;
	else
		unset($context['cffp'][$cffp_action]);
}

// Shows a group of a message like in a message.
function cffpShowGroupMsg($id_msg, $id_group)
{
	cffpShowGroup('display_readonly', $id_msg, $id_group);
}

// Shows a group of a message like on the message index.
function cffpShowGroupMsgIndex($id_msg, $id_group)
{
	cffpShowGroup('message_index', $id_msg, $id_group);
}

// Shows statistical data of the group of a message.
function cffpShowGroupMsgStats($id_msg, $id_group, $type, $params = '')
{
	if (empty($id_msg) || empty($id_group) || !in_array($type, array('chart_average', 'chart_hbars', 'chart_vbars', 'summary', 'value')))
		return;

	if (empty($params))
	{
		if (in_array($type, array('chart_average', 'chart_hbars', 'chart_vbars')))
			$params = 'show';
		else // summary, value.
			return;
	}

	$stats = cffpLoadStatsOfGroupByMsg($id_group, $id_msg, true);
	if (empty($stats))
		return;

	$group = array(
		'show_stats_post_' . $type => $params,
		'show_stats_post_text' => '',
	);
	$stats = cffpGetStats($stats[$id_msg], $group, 'post');
	if (empty($stats))
		return;

	if ('value' == $type)
		echo is_array($stats['value']) ? implode('<br />', $stats['value']) : $stats['value'];
	else
		template_cffp_show_stats($stats, $type);
}

// Evaluates PHP code.
function cffpEval($cffp_eval)
{
	eval($cffp_eval);
}

// Saves custom post fields of groups type no member.
function cffpSaveGroupsPost($id_msg, $id_member_poster)
{
	global $txt, $context;

	if (empty($id_msg))
		return false;

	if (empty($_REQUEST['cffp']))
		return true;

	$user = cffpGetTypeUser();
	$owner = cffpIsOwner(false, $id_member_poster);
	$data_msg = cffpGetDataMsgTopic($id_msg);
	$group_array = cffpGetGroupsAndFieldsOfGroup($data_msg, 0, 'post', 0, $user, $owner);
	if (empty($group_array))
		return true;

	foreach ($group_array as $id_group => $group)
	{
		if (!isset($_REQUEST['cffp'][$id_group]))
			continue;

		if ('fields' == $group['type'])
		{
			// Are we saving?
			if (isset($_REQUEST['cffp'][$id_group]['set']))
			{
				// Do the insertion/updates.
				if (empty($_REQUEST['cffp'][$id_group]['id_cffp']))
					cffpInsertFields('post', $id_msg, $id_group, $group['settings_fields']);
				else
					cffpUpdateFields($_REQUEST['cffp'][$id_group]['id_cffp'], $id_group, $group['settings_fields']);
			}
			// Deleting?
			else
				cffpDeleteFields($_REQUEST['cffp'][$id_group]['id_cffp']);
		}
		else // record.
		{
			// Are we saving?
			if (isset($_REQUEST['cffp'][$id_group]['set']))
			{
				// Do the insertion/updates.
				if ($group['can_add'] && empty($_REQUEST['cffp'][$id_group]['id_cffp']))
					cffpInsertFields('post', $id_msg, $id_group, $group['settings_fields']);
				elseif ($group['can_modify'] && !empty($_REQUEST['cffp'][$id_group]['id_cffp']))
					cffpUpdateFields($_REQUEST['cffp'][$id_group]['id_cffp'], $id_group, $group['settings_fields']);
			}
			// Deleting?
			elseif ($group['can_remove'] && !empty($_REQUEST['cffp'][$id_group]['id_cffp']))
				cffpDeleteFields($_REQUEST['cffp'][$id_group]['id_cffp']);
		}
	}

	return true;
}

// Gets search pages.
function cffpGetSearchPages()
{
	global $context, $txt, $scripturl, $modSettings;

	$context['cffp']['search_pages'] = array();

	cffpLoad(0, 0, 'search_posts_menu');
	if (!empty($context['cffp']['search_posts_menu']) && (!$context['user']['is_guest'] || $modSettings['allow_guestAccess']))
	{
		$search_page_array = array();
		foreach ($context['cffp']['search_posts_menu']['group_array'] as $id_group => $group)
		{
			if (strpos($group['search_page_name'], ','))
			{
				$search_page_name_array = explode(',', $group['search_page_name']);
				$search_page_order_by_array = explode(',', $group['search_page_order_by']);
				foreach ($search_page_name_array as $k => $search_page_name)
					$search_page_array[] = $search_page_name . ',' . $search_page_order_by_array[$k];
			}
			else
				$search_page_array[] = $group['search_page_name'] . ',' . $group['search_page_order_by'];
		}
		$search_page_array = array_unique($search_page_array);
		sort($search_page_array);

		foreach ($search_page_array as $k => $search_page)
		{
			if (',' == substr($search_page, 0, 1))
			{
				$search_page_name = '';
				$search_page_order_by = substr($search_page, 1);
			}
			else
				list ($search_page_name, $search_page_order_by) = explode(',', $search_page);

			$context['cffp']['search_pages']['posts_' . $k] = array(
				'title' => cffpIsNotEmptyString($search_page_name) ? $search_page_name : $txt['cffp_search_posts'],
				'href' => $scripturl . '?action=cffpsearch;sa=search_posts;search_page=' . cffpUrlParamsCompress(array(array($search_page_name, $search_page_order_by))),
				'type' => 'posts',
				'search_page_name' => $search_page_name,
				'search_page_order_by' => $search_page_order_by,
			);
		}
	}

	cffpLoad(0, 0, 'search_records_menu');
	if (!empty($context['cffp']['search_records_menu']) && (!$context['user']['is_guest'] || $modSettings['allow_guestAccess']))
	{
		foreach ($context['cffp']['search_records_menu']['group_array'] as $id_group => $group)
		{
			$context['cffp']['search_pages']['records_' . $id_group] = array(
				'title' => cffpIsNotEmptyString($group['search_page_name']) ? $group['search_page_name'] : sprintf($txt['cffp_search_records_type'], $group['name']),
				'href' => $scripturl . '?action=cffpsearch;sa=search_records;id_group=' . $id_group,
				'type' => 'record',
				'id' => $id_group,
			);
		}
	}
}

// Search custom post fields.
function cffpSearch()
{
	global $context, $scripturl, $txt, $sourcedir, $modSettings;

	loadTemplate('CustomFieldFilterPost');
	require_once ($sourcedir . '/Subs-List.php');

	// This comes under the umbrella of search posts.
	isAllowedTo('search_posts');

	if (empty($context['cffp']['search_pages']))
		fatal_lang_error('error_cffp_search_not_allowed');

	if (empty($_GET['sa']) || !in_array($_GET['sa'], array('search_posts', 'search_records')) || ('search_records' == $_GET['sa'] && empty($_GET['id_group'])))
	{
		$search = reset($context['cffp']['search_pages']);
		if ('posts' == $search['type'])
		{
			$_GET['sa'] = 'search_posts';
			unset($_GET['id_group']);
			$_GET['search_page'] = cffpUrlParamsCompress(array(array($search['search_page_name'], $search['search_page_order_by'])));
		}
		else // record.
		{
			$_GET['sa'] = 'search_records';
			$_GET['id_group'] = $search['id'];
			unset($_GET['search_page']);
		}
	}

	// Sort out the context!
	$context['cffp_action'] = $_GET['sa'];
	$context['id_group'] = 'search_records' == $context['cffp_action'] ? (int) $_GET['id_group'] : 0;
	if ('search_posts' == $context['cffp_action'] && !empty($_GET['search_page']))
	{
		$search_page = cffpUrlParamsDecompress($_GET['search_page']);
		$context['search_page_name'] = $search_page[0][0];
		$context['search_page_order_by'] = $search_page[0][1];
	}
	else
	{
		$context['search_page_name'] = '';
		$context['search_page_order_by'] = '';
	}
	$context['search_page_compress'] = cffpUrlParamsCompress(array(array($context['search_page_name'], $context['search_page_order_by'])));

	if (isset($_REQUEST['cffp_search']))
		$_REQUEST['cffp'] = cffpUrlParamsDecompress($_REQUEST['cffp_search']);
	if (isset($_REQUEST['cffp_search_operator']))
		$_REQUEST['cffp_operator'] = cffpUrlParamsDecompress($_REQUEST['cffp_search_operator']);

	cffpLoad(0, 0, $context['cffp_action'], 0, $context['id_group'], $context['search_page_name']);
	if (empty($context['cffp'][$context['cffp_action']]))
		fatal_lang_error('error_cffp_search_not_allowed');

	$context['cffp_search']['expanded'] = empty($_REQUEST['no_expanded']);
	$context['cffp_search']['params'] = '';
	if (!$context['cffp_search']['expanded'])
		$context['cffp_search']['params'] .= ';no_expanded=1';
	if (isset($_REQUEST['cffp']))
		$context['cffp_search']['params'] .= ';cffp_search=' . cffpUrlParamsCompress($_REQUEST['cffp']);
	if (isset($_REQUEST['cffp_operator']))
		$context['cffp_search']['params'] .= ';cffp_search_operator=' . cffpUrlParamsCompress($_REQUEST['cffp_operator']);

	$context['cffp']['search'] = $context['cffp'][$context['cffp_action']];
	$context['page_desc'] = $txt['cffp_' . $context['cffp_action'] . '_desc'];
	$context['sub_template'] = 'cffp_' . $context['cffp_action'];

	if ('search_posts' == $context['cffp_action'])
	{
		$context['page_title'] = cffpIsNotEmptyString($context['search_page_name']) ? $context['search_page_name'] : $txt['cffp_search_posts'];
		$title = $txt['posts'];
		$base_href = $scripturl . '?action=cffpsearch;sa=search_posts;search_page=' . $context['search_page_compress'];
		$function_get_items = 'list_cffpGetSearchPosts';
		$function_get_count = 'list_cffpGetSearchPostsCount';
		$columns = array(
			'sticky_locked' => array(
				'header' => array(
					'style' => 'width: 4%;',
				),
				'data' => array(
					'function' => create_function('$rowData', '
						global $settings, $txt;

						$text = \'\';
						if ($rowData[\'is_sticky\'] && $rowData[\'locked\'])
						{
							$images_url = file_exists($settings[\'theme_dir\'] . \'/images/icons/quick_sticky_lock.gif\') ? \'images_url\' : \'default_images_url\';
							$text = \'<img src="\' . $settings[$images_url] . \'/icons/quick_sticky_lock.gif" alt="\' . $txt[\'cffp_show_filters_sticky_locked\'] . \'" />\';
						}
						elseif ($rowData[\'is_sticky\'])
						{
							$images_url = file_exists($settings[\'theme_dir\'] . \'/images/icons/quick_sticky.gif\') ? \'images_url\' : \'default_images_url\';
							$text = \'<img src="\' . $settings[$images_url] . \'/icons/quick_sticky.gif" alt="\' . $txt[\'cffp_show_filters_sticky\'] . \'" />\';
						}
						elseif ($rowData[\'locked\'])
						{
							$images_url = file_exists($settings[\'theme_dir\'] . \'/images/icons/quick_lock.gif\') ? \'images_url\' : \'default_images_url\';
							$text = \'<img src="\' . $settings[$images_url] . \'/icons/quick_lock.gif" alt="\' . $txt[\'cffp_show_filters_locked\'] . \'" />\';
						}

						return $text;
					'),
					'style' => 'text-align: center;',
				),
			),
			'icon' => array(
				'header' => array(
					'style' => 'width: 4%;',
				),
				'data' => array(
					'function' => create_function('$rowData', '
						global $modSettings, $settings, $context;

						// We need to check the topic icons exist...
						if (empty($modSettings[\'messageIconChecks_disable\']))
						{
							if (!isset($context[\'icon_sources\'][$rowData[\'icon\']]))
								$context[\'icon_sources\'][$rowData[\'icon\']] = file_exists($settings[\'theme_dir\'] . \'/images/post/\' . $rowData[\'icon\'] . \'.gif\') ? \'images_url\' : \'default_images_url\';
						}
						else
						{
							if (!isset($context[\'icon_sources\'][$rowData[\'icon\']]))
								$context[\'icon_sources\'][$rowData[\'icon\']] = \'images_url\';
						}

						return \'<img src="\' . $settings[$context[\'icon_sources\'][$rowData[\'icon\']]] . \'/post/\' . $rowData[\'icon\'] . \'.gif" alt="" />\';
					'),
					'style' => 'text-align: center;',
				),
			),
			'm.subject' => array(
				'header' => array(
					'value' => $txt['subject'],
				),
				'data' => array(
					'function' => create_function('$rowData', '
						global $scripturl, $context, $settings, $txt;

						$href = $scripturl . \'?topic=\' . $rowData[\'id_topic\'] . \'.msg\' . $rowData[\'id_msg\'];
						$text = \'\';

						if ($rowData[\'new\'] && $context[\'user\'][\'is_logged\'])
							$text .= \'
						<a href="\' . $href . \'#new" id="newicon\' . $rowData[\'id_msg\'] . \'"><img src="\' . $settings[\'lang_images_url\'] . \'/new.gif" alt="\' . $txt[\'new\'] . \'" /></a>\';

						loadTemplate(\'CustomFieldFilterPost\');
						cffpLoad($rowData[\'id_msg\'], $rowData[\'id_member\'], \'message_index\', $rowData[\'id_board\']);
						cffpLoad($rowData[\'id_msg\'], 0, \'message_index_member\', $rowData[\'id_board\']);

						ob_start();
						template_cffp_show(\'message_index\');
						template_cffp_show(\'message_index_member\');
						$text .= ob_get_contents();
						ob_end_clean();

						return \'<a href="\' . $href . \'#msg\' . $rowData[\'id_msg\'] . \'">\' . $rowData[\'subject\'] . \'</a>\' . $text;
					'),
				),
				'sort' => array(
					'default' => 'm.subject',
					'reverse' => 'm.subject DESC',
				),
			),
			'm.poster_time' => array(
				'header' => array(
					'value' => $txt['cffp_poster_time'],
					'style' => 'width: 18%;',
				),
				'data' => array(
					'db' => 'poster_time',
					'timeformat' => true,
					'style' => 'text-align: center;',
				),
				'sort' => array(
					'default' => 'm.poster_time',
					'reverse' => 'm.poster_time DESC',
				),
			),
			'display_name' => array(
				'header' => array(
					'value' => $txt['posted_by'],
					'style' => 'width: 10%;',
				),
				'data' => array(
					'function' => create_function('$rowData', '
						global $scripturl, $txt;

						return !empty($rowData[\'id_member\']) ? \'<a href="\' . $scripturl . \'?action=profile;u=\' . $rowData[\'id_member\'] . \'" title="\' . $txt[\'profile_of\'] . \' \' . $rowData[\'display_name\'] . \'">\' . $rowData[\'display_name\'] . \'</a>\' : $rowData[\'display_name\'];
					'),
				),
				'sort' => array(
					'default' => 'display_name',
					'reverse' => 'display_name DESC',
				),
			),
		);

		$search_page_order_by = !empty($context['search_page_order_by']) ? $context['search_page_order_by'] : 'm.poster_time desc';
		list ($default_sort_col, $default_sort_dir) = explode(' ', $search_page_order_by);
		if (!isset($columns[$default_sort_col]))
			$default_sort_col = 'm.poster_time';

		// Split groups in 2 columns.
		$group_num_fields_array = array();
		$group_order_array = array();
		foreach ($context['cffp']['search']['group_array'] as $id_group => $group)
		{
			$group_order_array[] = $id_group;
			$group_num_fields_array[$id_group] = count($group['settings_fields']);
		}
		arsort($group_num_fields_array);

		$column_num_fields_array = array(0, 0);
		$column_id_group_array = array(array(), array());
		foreach ($group_num_fields_array as $id_group => $num_fields)
		{
			$column = $column_num_fields_array[0] <= $column_num_fields_array[1] ? 0 : 1;
			$column_num_fields_array[$column] += $num_fields;
			$column_id_group_array[$column][] = $id_group;
		}

		$column_id_group_order_array = array(array(), array());
		foreach ($group_order_array as $id_group)
		{
			$column = in_array($id_group, $column_id_group_array[0]) ? 0 : 1;
			$column_id_group_order_array[$column][] = $id_group;
		}
		$context['cffp']['search']['columns_id_group'] = $column_id_group_order_array;
	}
	else // search_records.
	{
		global $cffp_record_settings_fields;

		$group = $context['cffp']['search']['group_array'][$context['id_group']];
		$cffp_record_settings_fields = $group['settings_fields'];

		$context['page_title'] = cffpIsNotEmptyString($group['search_page_name']) ? $group['search_page_name'] : sprintf($txt['cffp_search_records_type'], $group['name']);
		$title = $txt['cffp_records'];
		$base_href = $scripturl . '?action=cffpsearch;sa=search_records;id_group=' . $context['id_group'];
		$function_get_items = 'list_cffpGetSearchRecords';
		$function_get_count = 'list_cffpGetSearchRecordsCount';
		$text_align = array(
			'text' => 'left',
			'text_email' => 'center',
			'text_number' => 'right',
			'textarea' => 'left',
			'select' => 'center',
			'select_multiple' => 'left',
			'combobox' => 'center',
			'combobox_multiple' => 'left',
			'radio' => 'center',
			'check' => 'left',
			'boolean' => 'center',
		);

		$columns = array();
		$columns['icon'] = array(
			'header' => array(
				'style' => 'width: 4%;',
			),
			'data' => array(
				'function' => create_function('$rowData', '
					global $modSettings, $settings, $context;

					// We need to check the topic icons exist...
					if (empty($modSettings[\'messageIconChecks_disable\']))
					{
						if (!isset($context[\'icon_sources\'][$rowData[\'icon\']]))
							$context[\'icon_sources\'][$rowData[\'icon\']] = file_exists($settings[\'theme_dir\'] . \'/images/post/\' . $rowData[\'icon\'] . \'.gif\') ? \'images_url\' : \'default_images_url\';
					}
					else
					{
						if (!isset($context[\'icon_sources\'][$rowData[\'icon\']]))
							$context[\'icon_sources\'][$rowData[\'icon\']] = \'images_url\';
					}

					return \'<img src="\' . $settings[$context[\'icon_sources\'][$rowData[\'icon\']]] . \'/post/\' . $rowData[\'icon\'] . \'.gif" alt="" />\';
				'),
				'style' => 'text-align: center;',
			),
		);
		if ($group['can_view'])
		{
			$columns['cffpf0.id_msg'] = array(
				'header' => array(
					'style' => 'width: 4%;',
				),
				'data' => array(
					'function' => create_function('$rowData', '
						global $scripturl, $txt;

						return \'<a href="\' . $scripturl . \'?topic=\' . $rowData[\'id_topic\'] . \'.msg\' . $rowData[\'id_msg\'] . \'#msg\' . $rowData[\'id_msg\'] . \'">\' . $txt[\'view\'] . \'</a>\';
					'),
					'style' => 'text-align: center;',
				),
			);
		}
		foreach ($group['show_record_list'] as $column => $show_record_list)
		{
			if (is_numeric($column))
			{
				$id_field = $column;
				$column_data = $show_record_list;

				if (!isset($cffp_record_settings_fields[$id_field]))
					$cffp_record_settings_fields[$id_field] = $context['cffp']['settings_groups'][$context['id_group']]['settings_fields'][$id_field];
				$name_field = 'column_' . $cffp_record_settings_fields[$id_field]['id_column'];

				$type_align = $cffp_record_settings_fields[$id_field]['type'];
				switch ($cffp_record_settings_fields[$id_field]['type'])
				{
					case 'text':
					{
						if (in_array($cffp_record_settings_fields[$id_field]['mask'], array('email', 'number')))
							$type_align .= '_' . $cffp_record_settings_fields[$id_field]['mask'];
						break;
					}
					case 'select':
					case 'combobox':
					{
						$options = explode(',', $cffp_record_settings_fields[$id_field]['options']);
						$select_multiple = array_shift($options);
						if ($select_multiple)
							$type_align .= '_multiple';
						break;
					}
				}

				$columns['cffpf0.' . $name_field] = array(
					'header' => array(
						'value' => $cffp_record_settings_fields[$id_field]['name'],
						'style' => !empty($column_data['show_record_list_width']) ? ('width: ' . $column_data['show_record_list_width'] . ';') : '',
					),
					'data' => array(
						'function' => create_function('$rowData', '
							global $cffp_record_settings_fields;

							return cffpHtmlOfField($cffp_record_settings_fields[' . $id_field . '], $rowData[\'' . $name_field . '\'], true, $rowData[\'id_msg\']);
						'),
						'style' => 'text-align: ' . (!empty($text_align[$type_align]) ? $text_align[$type_align] : 'center') . ';',
					),
					'sort' => array(
						'default' => 'cffpf0.' . $name_field,
						'reverse' => 'cffpf0.' . $name_field . ' DESC',
					),
				);
			}
			else
			{
				$name_field = $column;
				$column_data = $show_record_list[0];
				$text = '';
				foreach ($show_record_list[1] as $id_field)
				{
					if (!isset($cffp_record_settings_fields[$id_field]))
						$cffp_record_settings_fields[$id_field] = $context['cffp']['settings_groups'][$context['id_group']]['settings_fields'][$id_field];
					$name_field2 = 'column_' . $cffp_record_settings_fields[$id_field]['id_column'];

					$text .= '
							$text2 = cffpHtmlOfField($cffp_record_settings_fields[' . $id_field . '], $rowData[\'' . $name_field2 . '\'], true, $rowData[\'id_msg\']);
							if (cffpIsNotEmptyString($text2))
							{
								if (cffpIsNotEmptyString($text))
									$text .= \'<br />\';
								$text .= $text2;
							}';
				}

				$columns['cffpf0.' . $name_field] = array(
					'header' => array(
						'value' => $column_data['show_record_list_name'],
						'style' => !empty($column_data['show_record_list_width']) ? ('width: ' . $column_data['show_record_list_width'] . ';') : '',
					),
					'data' => array(
						'function' => create_function('$rowData', '
							global $cffp_record_settings_fields;

							$text = \'\';' . $text . '
							return $text;
						'),
						'style' => 'text-align: left;',
					),
					'sort' => array(
						'default' => 'cffpf0.' . $name_field,
						'reverse' => 'cffpf0.' . $name_field . ' DESC',
					),
				);
			}
		}
		$columns['cffpf0.poster_time'] = array(
			'header' => array(
				'value' => $txt['cffp_poster_time'],
				'style' => !empty($group['show_record_list']) ? 'width: 10%;' : '',
			),
			'data' => array(
				'db' => 'poster_time',
				'timeformat' => true,
				'style' => 'text-align: center;',
			),
			'sort' => array(
				'default' => 'cffpf0.poster_time',
				'reverse' => 'cffpf0.poster_time DESC',
			),
		);
		$columns['display_name'] = array(
			'header' => array(
				'value' => $txt['posted_by'],
				'style' => !empty($group['show_record_list']) ? 'width: 10%;' : '',
			),
			'data' => array(
				'function' => create_function('$rowData', '
					global $scripturl, $txt;

					return !empty($rowData[\'id_member\']) ? \'<a href="\' . $scripturl . \'?action=profile;u=\' . $rowData[\'id_member\'] . \'" title="\' . $txt[\'profile_of\'] . \' \' . $rowData[\'display_name\'] . \'">\' . $rowData[\'display_name\'] . \'</a>\' : $rowData[\'display_name\'];
				'),
			),
			'sort' => array(
				'default' => 'display_name',
				'reverse' => 'display_name DESC',
			),
		);

		$search_page_order_by = !empty($group['search_page_order_by']) ? $group['search_page_order_by'] : 'cffpf0.poster_time desc';
		list ($default_sort_col, $default_sort_dir) = explode(' ', $search_page_order_by);
		if (!isset($columns[$default_sort_col]))
			$default_sort_col = 'cffpf0.poster_time';
	}

	$context['linktree'][] = array(
		'url' => $base_href,
		'name' => $context['page_title'],
	);

	// Result.
	$listOptions = array(
		'id' => 'cffp_search_result',
		'title' => $title,
		'base_href' => $base_href . $context['cffp_search']['params'],
		'default_sort_col' => $default_sort_col,
		'default_sort_dir' => $default_sort_dir,
		'no_items_label' => $txt['search_no_results'],
		'items_per_page' => !empty($modSettings['search_results_per_page']) ? $modSettings['search_results_per_page'] : 30,
		'get_items' => array(
			'function' => $function_get_items,
		),
		'get_count' => array(
			'function' => $function_get_count,
		),
		'columns' => $columns,
	);
	createList($listOptions);
}

// Compress parameters of URL.
function cffpUrlParamsCompress($params)
{
	// All search params have been checked, let's compile them to a single string... made less simple by PHP 4.3.9 and below.
	$temp_params = $params;
	$params = array();
	foreach ($temp_params as $k => $v)
	{
		$kv2 = array();
		foreach ($v as $k2 => $v2)
			$kv2[] = $k2 . '|+|' . $v2;
		$params[] = $k . '|\'|' . implode('|-|', $kv2);
	}

	if (!empty($params))
	{
		$params = implode('|"|', $params);
		// Due to old IE's 2083 character limit, we have to compress long search strings
		$temp_params = @gzcompress($params);
		// Gzcompress failed, use try non-gz
		if (!empty($temp_params))
			$params = $temp_params;
		// Base64 encode, then replace +/= with uri safe ones that can be reverted
		$params = str_replace(array('+', '/', '='), array('-', '_', '.'), base64_encode($params));
	}

	return $params;
}

// Decompress parameters of URL.
function cffpUrlParamsDecompress($params)
{
	if (!empty($params))
	{
		// Due to IE's 2083 character limit, we have to compress long search strings
		$params = base64_decode(str_replace(array('-', '_', '.'), array('+', '/', '='), $params));
		// Test for gzuncompress failing
		$temp_params = @gzuncompress($params);
		$temp_params = explode('|"|', (!empty($temp_params) ? $temp_params : $params));

		$params = array();
		foreach ($temp_params as $data)
		{
			@list ($k, $v) = explode('|\'|', $data);
			$kv2_array = explode('|-|', $v);
			foreach ($kv2_array as $kv2)
			{
				@list ($k2, $v2) = explode('|+|', $kv2);
				$params[$k][$k2] = $v2;
			}
		}
	}

	return $params;
}

// Validates value with mask.
function cffpValidateMask($mask, $value)
{
	global $smcFunc;

	$error = '';
	if (!empty($mask) && $mask != 'none')
	{
		if ('email' == $mask && (0 === preg_match('~^[0-9A-Za-z=_+\-/][0-9A-Za-z=_\'+\-/\.]*@[\w\-]+(\.[\w\-]+)*(\.[\w]{2,6})$~', $value)))
			$error = 'cffp_group_invalid_email';
		elseif ('number' == $mask && (!is_numeric($value) || $value != (int) $value))
			$error = 'cffp_group_not_number';
		elseif ('regex' == $smcFunc['substr']($mask, 0, 5) && 0 === preg_match($smcFunc['substr']($mask, 5), $value))
			$error = 'cffp_group_inproper_format';
	}

	return $error;
}

// Validates values of custom post fields on $_REQUEST['cffp'] array.
function cffpValidateRequest($type, $id_msg, $cffp_action)
{
	global $context, $smcFunc, $sourcedir, $modSettings;

	if ('return_post_errors' == $cffp_action)
		$post_errors = array();

	if (empty($_REQUEST['cffp']))
	{
		if ('return_post_errors' == $cffp_action)
			return $post_errors;
		else // fill_context.
			return;
	}

	$error = array();
	foreach ($_REQUEST['cffp'] as $id_group => $group)
	{
		if ('post' == $type && !empty($group['set']) && 0 < strlen($_REQUEST['subject']) && !empty($group['no_show_with_subject_prefix']))
		{
			$subject_prefix = explode(',', $group['no_show_with_subject_prefix']);
			foreach ($subject_prefix as $pre)
			{
				if (0 === $smcFunc['strpos']($_REQUEST['subject'], $pre))
				{
					unset($_REQUEST['cffp'][$id_group]['set']);
					continue 2; // Exit foreach $subject_prefix and continue foreach $_REQUEST['cffp'].
				}
			}
		}

		// Guest name and email.
		if ((('post' == $type && !empty($id_msg) && !empty($group['set'])) || 'member' == $type) && empty($group['id_cffp']) && $context['user']['is_guest'])
		{
			$_REQUEST['cffp'][$id_group]['guest_name'] = !isset($group['guest_name']) ? '' : trim($group['guest_name']);
			$_REQUEST['cffp'][$id_group]['guest_email'] = !isset($group['guest_email']) ? '' : trim($group['guest_email']);

			// Validate the name and email.
			if (!isset($_REQUEST['cffp'][$id_group]['guest_name']) || trim(strtr($_REQUEST['cffp'][$id_group]['guest_name'], '_', ' ')) == '')
				$error[$id_group]['cffp_group_no_name'] = true;
			elseif ($smcFunc['strlen']($_REQUEST['cffp'][$id_group]['guest_name']) > 25)
				$error[$id_group]['cffp_group_long_name'] = true;
			else
			{
				require_once($sourcedir . '/Subs-Members.php');
				if (isReservedName(htmlspecialchars($_REQUEST['cffp'][$id_group]['guest_name']), 0, true, false))
					$error[$id_group]['cffp_group_bad_name'] = true;
			}

			if (empty($modSettings['guest_post_no_email']))
			{
				if (!isset($_REQUEST['cffp'][$id_group]['guest_email']) || $_REQUEST['cffp'][$id_group]['guest_email'] == '')
					$error[$id_group]['cffp_group_no_email'] = true;
				else
				{
					$error_code = cffpValidateMask('email', $_REQUEST['cffp'][$id_group]['guest_email']);
					if (!empty($error_code))
						$error[$id_group]['cffp_group_bad_email'] = true;
				}
			}
		}

		if (!isset($group['set']) || !empty($group['without_fields']))
			continue;

		foreach ($context['cffp']['settings_groups'][$id_group]['settings_fields'] as $id_field => $settings_field)
		{
			$name_field = 'column_' . $settings_field['id_column'];
			if (!isset($group[$name_field]) && isset($group['name_fields'][$id_field]))
			{
				$_REQUEST['cffp'][$id_group][$name_field] = '';
				$group[$name_field] = '';
			}
			if (!isset($group[$name_field]))
				continue;

			// Not empty value.
			if ($settings_field['not_empty_value'] && !cffpIsNotEmptyString($group[$name_field]))
				$error[$id_group]['cffp_group_empty'][] = $id_field;

			// Unique value.
			if ($settings_field['unique_value'] && cffpIsNotEmptyString($group[$name_field]))
			{
				$request = $smcFunc['db_query']('', '
					SELECT {raw:name_field}
					FROM {db_prefix}cffp_fields
					WHERE id_group = {int:id_group}' . ('post' != $type || empty($id_msg) ? '' : '
						AND id_msg <> {int:id_msg}') . ('member' != $type || empty($group['id_cffp']) ? '' : '
						AND id_cffp <> {int:id_cffp}') . '
						AND {raw:name_field} = {string:value}',
					array(
						'id_group' => $id_group,
						'id_msg' => $id_msg,
						'id_cffp' => $group['id_cffp'],
						'name_field' => $name_field,
						'value' => $group[$name_field],
					)
				);
				$num_rows = $smcFunc['db_num_rows']($request);
				$smcFunc['db_free_result']($request);
				if (0 < $num_rows)
					$error[$id_group]['cffp_group_value_not_unique'][] = $id_field;
			}

			// Mask.
			if (!empty($settings_field['mask']) && 'none' != $settings_field['mask'] && cffpIsNotEmptyString($group[$name_field]))
			{
				$error_code = cffpValidateMask($settings_field['mask'], $group[$name_field]);
				if (!empty($error_code))
					$error[$id_group][$error_code][] = $id_field;
			}
		}
	}

	foreach ($error as $id_group => $eg)
	{
		$group_name = $smcFunc['htmlspecialchars']($_REQUEST['cffp'][$id_group]['name'], ENT_QUOTES);
		foreach ($eg as $error_code => $ef)
		{
			if (is_array($ef))
			{
				foreach ($ef as $id_field)
				{
					$field_name = $smcFunc['htmlspecialchars']($_REQUEST['cffp'][$id_group]['name_fields'][$id_field], ENT_QUOTES);
					$error_code2 = $error_code . ',' . $id_field . ',' . $id_group;
					$context['cffp']['validate'][$error_code2] = array($field_name, $group_name);
					if ('return_post_errors' == $cffp_action)
						$post_errors[] = $error_code2;
					else // fill_context.
						$context['post_error'][$error_code2] = true;
				}
			}
			else
			{
				$error_code2 = $error_code . ',' . $id_group;
				$context['cffp']['validate'][$error_code2] = array($group_name);
				if ('return_post_errors' == $cffp_action)
					$post_errors[] = $error_code2;
				else // fill_context.
					$context['post_error'][$error_code2] = true;
			}
		}
	}

	if ('return_post_errors' == $cffp_action)
		return $post_errors;
}

// Inserts fields.
function cffpInsertFields($type, $id_msg, $id_group, $settings_fields)
{
	global $smcFunc, $user_info, $context;

	if (empty($id_msg) || empty($id_group))
		return false;

	if ($context['user']['is_guest'])
	{
		if ('post' == $type)
		{
			// Guest creates a group of fields on post existent.
			if (isset($_REQUEST['cffp'][$id_group]['guest_name']))
			{
				$username = $_REQUEST['cffp'][$id_group]['guest_name'];
				$email = isset($_REQUEST['cffp'][$id_group]['guest_email']) ? $_REQUEST['cffp'][$id_group]['guest_email'] : '';
			}
			// Guest creates a group of fields on new post.
			else // isset($_REQUEST['guestname'])).
			{
				$username = $_REQUEST['guestname'];
				$email = isset($_REQUEST['email']) ? $_REQUEST['email'] : '';
			}
		}
		else // member.
		{
			$username = $_REQUEST['cffp'][$id_group]['guest_name'];
			$email = isset($_REQUEST['cffp'][$id_group]['guest_email']) ? $_REQUEST['cffp'][$id_group]['guest_email'] : '';
		}
		$_SESSION['guest_name'] = $username;
		$_SESSION['guest_email'] = $email;
	}
	// User no guest creates a group of fields.
	else
	{
		$username = $user_info['username'];
		$email = $user_info['email'];
	}

	$columns = array(
		'id_msg' => 'int', 'id_group' => 'int', 'id_member' => 'int',
		'poster_name' => 'string-255', 'poster_email' => 'string-255', 'poster_ip' => 'string-255', 'poster_time' => 'int',
		'modified_name' => 'string', 'modified_time' => 'int',
	);
	$data = array(
		$id_msg, $id_group, $user_info['id'],
		$username, $email, $user_info['ip'], time(),
		'', 0,
	);
	foreach ($settings_fields as $row)
	{
		$name_field = 'column_' . $row['id_column'];
		$columns[$name_field] = 'string';
		$data[] = cffpGetRequestValueOfField($id_group, $name_field, $row['type'], $row['length'], $row['options']);
	}

	$smcFunc['db_insert']('',
		'{db_prefix}cffp_fields',
		$columns, $data,
		array('id_cffp')
	);
	$id_cffp = $smcFunc['db_insert_id']('{db_prefix}cffp_fields', 'id_cffp');

	// Update counter record (group is active).
	if (!empty($user_info['id']) && 'record' == $context['cffp']['settings_groups'][$id_group]['type'])
	{
		$request = $smcFunc['db_query']('', '
			SELECT id_msg
			FROM {db_prefix}messages
			WHERE id_msg = {int:id_msg}
				AND icon != {string:recycled}
				AND approved = {int:is_approved}',
			array(
				'id_msg' => $id_msg,
				'recycled' => 'recycled',
				'is_approved' => 1,
			)
		);
		$num_rows = $smcFunc['db_num_rows']($request);
		$smcFunc['db_free_result']($request);
		if (0 < $num_rows)
			cffpUpdateCounterRecord($user_info['id'], $id_group, 1);
	}

	return $id_cffp;
}

// Updates fields.
function cffpUpdateFields($id_cffp, $id_group, $settings_fields)
{
	global $smcFunc, $modSettings, $user_info, $context, $txt;

	if (empty($id_cffp) || empty($id_group))
		return false;

	$request = $smcFunc['db_query']('', '
		SELECT cffpf.id_msg, id_topic, id_board, cffpf.id_member, cffpf.poster_time
		FROM {db_prefix}cffp_fields AS cffpf
			INNER JOIN {db_prefix}messages AS m ON (m.id_msg = cffpf.id_msg)
		WHERE id_cffp = {int:id_cffp}',
		array(
			'id_cffp' => $id_cffp,
		)
	);
	$fields_value = $smcFunc['db_fetch_assoc']($request);
	$smcFunc['db_free_result']($request);

	$set = '';
	$query_parameters = array(
		'id_cffp' => $id_cffp,
	);

	// Have admins allowed people to hide their screwups?
	if (time() - $fields_value['poster_time'] > $modSettings['edit_wait_time'] || $user_info['id'] != $fields_value['id_member'])
	{
		$set = 'modified_name = {string:modified_name}, modified_time = {int:modified_time}';
		if ($context['user']['is_guest'])
			$query_parameters['modified_name'] = isset($_SESSION['guest_name']) ? $_SESSION['guest_name'] : $txt['guest_title'];
		else
			$query_parameters['modified_name'] = $user_info['name'];
		$query_parameters['modified_time'] = time();
	}

	foreach ($settings_fields as $row)
	{
		$name_field = 'column_' . $row['id_column'];
		if (!empty($set))
			$set .= ', ';
		$set .= $name_field . ' = {string:' . $name_field . '}';
		$query_parameters[$name_field] = cffpGetRequestValueOfField($id_group, $name_field, $row['type'], $row['length'], $row['options']);
	}

	$smcFunc['db_query']('', '
		UPDATE {db_prefix}cffp_fields
		SET ' . $set . '
		WHERE id_cffp = {int:id_cffp}',
		$query_parameters
	);
	if ($user_info['id'] != $fields_value['id_member'])
	{
		logAction(
			'cffp_edit_data',
			array(
				'group_name' => $context['cffp']['settings_groups'][$id_group]['name'],
				'member' => $user_info['id'],
				'message' => $fields_value['id_msg'],
				'topic' => $fields_value['id_topic'],
				'board' => $fields_value['id_board'],
			)
		);
	}
}

// Deletes fields.
function cffpDeleteFields($id_cffp)
{
	global $smcFunc, $user_info, $context;

	if (empty($id_cffp))
		return false;

	$request = $smcFunc['db_query']('', '
		SELECT cffpf.id_msg, id_topic, id_board, id_group, cffpf.id_member, icon, approved
		FROM {db_prefix}cffp_fields AS cffpf
			INNER JOIN {db_prefix}messages AS m ON (m.id_msg = cffpf.id_msg)
		WHERE id_cffp = {int:id_cffp}',
		array(
			'id_cffp' => $id_cffp,
		)
	);
	$fields_value = $smcFunc['db_fetch_assoc']($request);
	$smcFunc['db_free_result']($request);

	if ($fields_value) // group is active.
	{
		$smcFunc['db_query']('', '
			DELETE FROM {db_prefix}cffp_fields
			WHERE id_cffp = {int:id_cffp}',
			array(
				'id_cffp' => $id_cffp,
			)
		);
		if ($user_info['id'] != $fields_value['id_member'])
		{
			logAction(
				'cffp_delete_data',
				array(
					'group_name' => $context['cffp']['settings_groups'][$fields_value['id_group']]['name'],
					'member' => $user_info['id'],
					'message' => $fields_value['id_msg'],
					'topic' => $fields_value['id_topic'],
					'board' => $fields_value['id_board'],
				)
			);
		}

		if ('recycled' != $fields_value['icon'] && $fields_value['approved'] && !empty($fields_value['id_member']) && 'record' == $context['cffp']['settings_groups'][$fields_value['id_group']]['type'])
			cffpUpdateCounterRecord($fields_value['id_member'], $fields_value['id_group'], -1);
	}
}

// Moves fields.
function cffpMoveFields($id_group, $id_cffp, $msg_from, $msg_to)
{
	global $smcFunc, $user_info, $context, $scripturl;

	if (empty($id_group) || empty($msg_from) || empty($msg_to))
		return false;

	// Get fields data.
	$request = $smcFunc['db_query']('', '
		SELECT DISTINCT id_member
		FROM {db_prefix}cffp_fields
		WHERE id_msg = {int:msg_from}
			AND id_group = {int:id_group}' . (!empty($id_cffp) ? '
			AND id_cffp = {int:id_cffp}' : ''),
		array(
			'id_group' => $id_group,
			'msg_from' => $msg_from,
			'id_cffp' => $id_cffp,
		)
	);
	$num_rows = $smcFunc['db_num_rows']($request);
	$fields_value = 1 == $num_rows ? $smcFunc['db_fetch_assoc']($request) : array();
	$smcFunc['db_free_result']($request);

	// Get messages data.
	$request = $smcFunc['db_query']('', '
		SELECT id_msg, id_topic, id_member, poster_name, poster_email, poster_ip, icon, approved
		FROM {db_prefix}messages
		WHERE id_msg IN ({array_int:id_msg_array})',
		array(
			'id_msg_array' => array($msg_from, $msg_to),
		)
	);
	$msg_array = array();
	while ($row = $smcFunc['db_fetch_assoc']($request))
		$msg_array[$row['id_msg']] = $row;
	$smcFunc['db_free_result']($request);

	// Prepare counter record (group is active).
	$group_record = 'record' == $context['cffp']['settings_groups'][$id_group]['type'];
	$group_no_member = 'member' != $context['cffp']['settings_groups'][$id_group]['type'];
	$change_member = $msg_array[$msg_from]['id_member'] != $msg_array[$msg_to]['id_member'];
	$count_from = 'recycled' != $msg_array[$msg_from]['icon'] && $msg_array[$msg_from]['approved'];
	$count_to = 'recycled' != $msg_array[$msg_to]['icon'] && $msg_array[$msg_to]['approved'];

	$inc = 0;
	if ($group_record && (($change_member && ($count_from || $count_to)) || ($count_from xor $count_to)))
	{
		// Only have one record for each group in a message.
		$request = $smcFunc['db_query']('', '
			SELECT COUNT(*)
			FROM {db_prefix}cffp_fields
			WHERE id_msg = {int:id_msg}
				AND id_group = {int:id_group}' . (!empty($id_cffp) ? '
				AND id_cffp = {int:id_cffp}' : ''),
			array(
				'id_msg' => $msg_from,
				'id_group' => $id_group,
				'id_cffp' => $id_cffp,
			)
		);
		list ($inc) = $smcFunc['db_fetch_row']($request);
		$smcFunc['db_free_result']($request);
	}

	// Move.
	$smcFunc['db_query']('', '
		UPDATE {db_prefix}cffp_fields
		SET
			id_msg = {int:msg_to}' . ($group_no_member ? ',
			id_member = {int:id_member},
			poster_name = {string:poster_name},
			poster_email = {string:poster_email},
			poster_ip = {string:poster_ip}' : '') . '
		WHERE id_msg = {int:msg_from}
			AND id_group = {int:id_group}' . (!empty($id_cffp) ? '
			AND id_cffp = {int:id_cffp}' : ''),
		array(
			'id_group' => $id_group,
			'msg_from' => $msg_from,
			'msg_to' => $msg_to,
			'id_member' => $msg_array[$msg_to]['id_member'],
			'poster_name' => $msg_array[$msg_to]['poster_name'],
			'poster_email' => $msg_array[$msg_to]['poster_email'],
			'poster_ip' => $msg_array[$msg_to]['poster_ip'],
			'id_cffp' => $id_cffp,
		)
	);
	if (empty($fields_value) || $user_info['id'] != $fields_value['id_member'])
	{
		$request = $smcFunc['db_query']('', '
			SELECT subject
			FROM {db_prefix}topics AS t
				INNER JOIN {db_prefix}messages AS m ON (m.id_msg = t.id_first_msg)
			WHERE t.id_topic = {int:id_topic}',
			array(
				'id_topic' => $msg_array[$msg_to]['id_topic'],
			)
		);
		$row = $smcFunc['db_fetch_assoc']($request);
		$topic2 = '<a href="' . $scripturl . '?topic=' . $msg_array[$msg_to]['id_topic'] . '.msg' . $msg_to . '#msg' . $msg_to . '">' . $row['subject'] . '</a>';
		$smcFunc['db_free_result']($request);

		logAction(
			'cffp_move_data',
			array(
				'group_name' => $context['cffp']['settings_groups'][$id_group]['name'],
				'member' => $user_info['id'],
				'message' => $msg_from,
				'topic' => $msg_array[$msg_from]['id_topic'],
				'message2' => $msg_to,
				'topic2' => $topic2,
			)
		);
	}

	// Count.
	if (!empty($inc)) // $group_record.
	{
		if ($change_member) // $count_from || $count_to.
		{
			if ($count_from)
				cffpUpdateCounterRecord($msg_array[$msg_from]['id_member'], $id_group, -$inc);
			if ($count_to)
				cffpUpdateCounterRecord($msg_array[$msg_to]['id_member'], $id_group, $inc);
		}
		else // $count_from xor $count_to.
		{
			$sign = $count_from && !$count_to ? -1 : 1;
			cffpUpdateCounterRecord($msg_array[$msg_from]['id_member'], $id_group, $sign * $inc);
		}
	}
}

// Updates statistics.
function cffpUpdateStats($id_group, $id_msg_array, $reload_settings = false)
{
	global $context, $smcFunc;

	if (empty($id_group))
		return;

	if (empty($id_msg_array))
	{
		$request = $smcFunc['db_query']('', '
			SELECT DISTINCT id_msg
			FROM {db_prefix}cffp_fields
			WHERE id_group = {int:id_group}',
			array(
				'id_group' => $id_group,
			)
		);
		$id_msg_array = array();
		while ($row = $smcFunc['db_fetch_assoc']($request))
			$id_msg_array[] = $row['id_msg'];
		$smcFunc['db_free_result']($request);
	}
	elseif (is_numeric($id_msg_array))
		$id_msg_array = array($id_msg_array);

	if (empty($id_msg_array))
		return;

	if ($reload_settings)
	{
		$request = $smcFunc['db_query']('', '
			SELECT ' . implode(', ', $context['cffp']['list_columns']['settings_groups']) . '
			FROM {db_prefix}cffp_settings_groups
			WHERE id_group = {int:id_group}
				AND active = {int:active}',
			array(
				'id_group' => $id_group,
				'active' => 1,
			)
		);
		$row = $smcFunc['db_fetch_assoc']($request);
		$smcFunc['db_free_result']($request);

		if (!$row)
			return;

		$id_group_array = array($id_group);
		$context['cffp']['settings_groups'][$id_group] = $row;
		$context['cffp']['settings_groups'][$id_group]['settings_fields'] = array();

		$request = $smcFunc['db_query']('', '
			SELECT ' . implode(', ', $context['cffp']['list_columns']['settings_fields']) . '
			FROM {db_prefix}cffp_settings_fields
			WHERE id_group IN ({array_int:id_group})
				AND active = {int:active}
			ORDER BY field_order ASC',
			array(
				'id_group' => $id_group_array,
				'active' => 1,
			)
		);
		while ($row = $smcFunc['db_fetch_assoc']($request))
		{
			$context['cffp']['settings_vars']['{FIELD_' . $row['id_field'] . '_DATABASE}'] = 'column_' . $row['id_column'];
			$context['cffp']['settings_groups'][$row['id_group']]['settings_fields'][$row['id_field']] = $row;
		}
		$smcFunc['db_free_result']($request);
	}
	elseif (!isset($context['cffp']['settings_groups'][$id_group]))
		return;

	$group = $context['cffp']['settings_groups'][$id_group];
	$show_stats = $group['show_stats'] && 1 == count($group['settings_fields']);
	if ($show_stats)
	{
		$settings_field = current($group['settings_fields']);
		switch ($settings_field['type'])
		{
			case 'select':
			{
				$options = explode(',', $settings_field['options']);
				$select_multiple = array_shift($options);
				array_shift($options);
				break;
			}
			case 'radio':
			case 'check':
			{
				$options = explode(',', $settings_field['options']);
				$select_multiple = 'check' == $settings_field['type'];
				break;
			}
			default:
			{
				$show_stats = 0;
				break;
			}
		}
	}

	if ($show_stats)
	{
		$fields = array(
			'data' => array(),
			'options_text' => array(),
			'is_numeric' => 1,
			'average' => '',
			'sum' => '',
			'max' => '',
			'min' => '',
			'highest' => '',
			'lowest' => '',
			'most_frequent' => '',
			'least_frequent' => '',
			'total' => '',
		);

		$trans = array();
		if (cffpIsNotEmptyString($settings_field['options_text']))
			$trans = array_combine($options, explode(',', $settings_field['options_text']));

		if (!empty($trans))
		{
			foreach ($trans as $value => $text)
			{
				$fields['data'][$value] = array(
					'value' => $value,
					'text' => $text,
					'num' => 0,
				);
				if ((string) $value != $text)
					$fields['options_text'][] = false === strpos($text, (string) $value) ? ($value . ' - ' . $text) : $text;
			}
		}
		else
		{
			foreach ($options as $value)
			{
				$fields['data'][$value] = array(
					'value' => $value,
					'text' => $value,
					'num' => 0,
				);
			}
		}
		$fields['data'][$value]['is_last'] = 1;
		$fields['options_text'] = !empty($fields['options_text']) ? implode(',', $fields['options_text']) : '';

		foreach ($fields['data'] as $value => $value_data)
		{
			if (!is_numeric($value))
			{
				$fields['is_numeric'] = 0;
				break; // Exit foreach.
			}
		}

		$data = $fields['data'];
		cffpLoadValueOfFieldsByMsg($id_msg_array, true);
		foreach ($id_msg_array as $id_msg)
		{
			// Set data.
			$fields['data'] = $data;
			$fields_value = cffpGetValueOfFieldsByMsg($id_msg, true);
			$fvm = isset($fields_value[$id_group]) ? $fields_value[$id_group] : array();
			foreach ($fvm as $fv)
			{
				$name_field = 'column_' . $settings_field['id_column'];
				$value = $fv[$name_field];

				if ($select_multiple)
				{
					$value_array = explode(',', $value);
					foreach ($value_array as $v)
					{
						if (!isset($fields['data'][$v]))
							continue;

						$fields['data'][$v]['num']++;
					}
				}
				else
				{
					if (!isset($fields['data'][$value]['num']))
						continue;

					$fields['data'][$value]['num']++;
				}
			}

			// Set summary data.
			if ($fields['is_numeric'])
			{
				$fields['sum'] = 0;
				$fields['max'] = max(array_keys($fields['data']));
				$fields['min'] = min(array_keys($fields['data']));
			}

			$fields['total'] = 0;
			$values_used = array();
			$values_frequent = array();
			foreach ($fields['data'] as $value => $value_data)
			{
				if (empty($value_data['num']))
					continue;

				$fields['total'] += $value_data['num'];
				if ($fields['is_numeric'])
					$fields['sum'] += $value * $value_data['num'];

				$values_used[$value] = $value_data['num'];

				if (!isset($values_frequent[$value_data['num']]))
					$values_frequent[$value_data['num']] = array();
				$values_frequent[$value_data['num']][] = $value;
			}
			if ($fields['is_numeric'])
				$fields['average'] = 0 != $fields['total'] ? $fields['sum'] / $fields['total'] : 0;

			foreach ($fields['data'] as $value => $value_data)
				$fields['data'][$value]['percent'] = round(0 != $fields['total'] ? ($value_data['num'] * 100) / $fields['total'] : 0, 1);

			if ($fields['is_numeric'] && !empty($values_used))
			{
				krsort($values_used);
				$fields['highest'] = key($values_used);
				end($values_used);
				$fields['lowest'] = key($values_used);
			}

			if (1 < count($values_frequent))
			{
				krsort($values_frequent);
				$fields['most_frequent'] = implode(',', current($values_frequent));
				end($values_frequent);
				$fields['least_frequent'] = implode(',', current($values_frequent));
			}

			$fields['data'] = serialize($fields['data']);

			if ($fields['is_numeric'])
			{
				$db_type_average = 'float';
				$db_type_sum = 'float';
				$db_type_max = 'float';
				$db_type_min = 'float';
			}
			else
			{
				$db_type_average = 'raw';
				$fields['average'] = 'NULL';
				$db_type_sum = 'raw';
				$fields['sum'] = 'NULL';
				$db_type_max = 'raw';
				$fields['max'] = 'NULL';
				$db_type_min = 'raw';
				$fields['min'] = 'NULL';
			}

			if ($fields['is_numeric'] && !empty($values_used))
			{
				$db_type_highest = 'float';
				$db_type_lowest = 'float';
			}
			else
			{
				$db_type_highest = 'raw';
				$fields['highest'] = 'NULL';
				$db_type_lowest = 'raw';
				$fields['lowest'] = 'NULL';
			}

			// Update or create stats.
			$request = $smcFunc['db_query']('', '
				SELECT id_stats
				FROM {db_prefix}cffp_stats
				WHERE id_msg = {int:id_msg}
					AND id_group = {int:id_group}',
				array(
					'id_msg' => $id_msg,
					'id_group' => $id_group,
				)
			);
			$stats = $smcFunc['db_fetch_assoc']($request);
			$smcFunc['db_free_result']($request);
			if ($stats) // Update.
			{
				$smcFunc['db_query']('', '
					UPDATE {db_prefix}cffp_stats
					SET
						data = {string:data}, options_text = {string:options_text},
						is_numeric = {int:is_numeric},
						average = {' . $db_type_average . ':average}, sum = {' . $db_type_sum . ':sum},
						max = {' . $db_type_max . ':max}, min = {' . $db_type_min . ':min},
						highest = {' . $db_type_highest . ':highest}, lowest = {' . $db_type_lowest . ':lowest},
						most_frequent = {string:most_frequent}, least_frequent = {string:least_frequent},
						total = {int:total}
					WHERE id_stats = {int:id_stats}',
					array(
						'id_stats' => $stats['id_stats'],
						'data' => $fields['data'],
						'options_text' => $fields['options_text'],
						'is_numeric' => $fields['is_numeric'],
						'average' => $fields['average'],
						'sum' => $fields['sum'],
						'max' => $fields['max'],
						'min' => $fields['min'],
						'highest' => $fields['highest'],
						'lowest' => $fields['lowest'],
						'most_frequent' => $fields['most_frequent'],
						'least_frequent' => $fields['least_frequent'],
						'total' => $fields['total'],
					)
				);
			}
			else // Create.
			{
				$smcFunc['db_insert']('',
					'{db_prefix}cffp_stats',
					array(
						'id_msg' => 'int', 'id_group' => 'int',
						'data' => 'string', 'options_text' => 'string',
						'is_numeric' => 'int',
						'average' => $db_type_average, 'sum' => $db_type_sum,
						'max' => $db_type_max, 'min' => $db_type_min,
						'highest' => $db_type_highest, 'lowest' => $db_type_lowest,
						'most_frequent' => 'string', 'least_frequent' => 'string',
						'total' => 'int',
					),
					array(
						$id_msg, $id_group,
						$fields['data'], $fields['options_text'],
						$fields['is_numeric'],
						$fields['average'], $fields['sum'],
						$fields['max'], $fields['min'],
						$fields['highest'], $fields['lowest'],
						$fields['most_frequent'], $fields['least_frequent'],
						$fields['total'],
					),
					array('id_stats')
				);
			}
		}
	}
	elseif ($group['show_stats'])
	{
		$smcFunc['db_query']('', '
			DELETE FROM {db_prefix}cffp_stats
			WHERE id_group = {int:id_group}',
			array(
				'id_group' => $id_group,
			)
		);
	}
}

// Updates all counters record of all member.
function cffpUpdateCounterRecordAllMembers($id_group = 0, $use_context = true)
{
	global $smcFunc, $context;

	if (empty($id_group))
		$use_context = true;
	if ($use_context && !isset($context['cffp']['settings_groups']))
		return false;

	$id_group_array = array();
	if (empty($id_group)) // $use_context = true.
	{
		foreach ($context['cffp']['settings_groups'] as $id_group => $settings_group)
		{
			if ('record' == $settings_group['type'])
				$id_group_array[] = $id_group;
		}
	}
	elseif (!$use_context || (isset($context['cffp']['settings_groups'][$id_group]) && 'record' == $context['cffp']['settings_groups'][$id_group]['type']))
		$id_group_array[] = $id_group;

	if (empty($id_group_array))
		return false;

	// Initialize counters.
	foreach ($id_group_array as $id_group)
	{
		$columns = array(
			'groups' => 'cffp' . $id_group . '_groups',
			'id_membergroup' => 'cffp' . $id_group . '_id_membergroup',
		);
		if (!in_array($columns['groups'], $context['cffp']['list_columns']['members']) || !in_array($columns['id_membergroup'], $context['cffp']['list_columns']['members']))
			cffpAddCounterRecordColumns($id_group);

		$smcFunc['db_query']('', '
			UPDATE {db_prefix}members
			SET
				{raw:groups_name} = {int:groups_value},
				{raw:id_membergroup_name} = {int:id_membergroup_value}',
			array(
				'groups_name' => $columns['groups'],
				'groups_value' => 0,
				'id_membergroup_name' => $columns['id_membergroup'],
				'id_membergroup_value' => 0,
			)
		);

		cffpUpdateMembergroupRecord($id_group, array(), $use_context);
	}

	// Recount.
	$request = $smcFunc['db_query']('', '
		SELECT DISTINCT cffpf.id_member, cffpf.id_group
		FROM {db_prefix}cffp_fields AS cffpf
			INNER JOIN {db_prefix}cffp_settings_groups AS cffpsg ON (cffpsg.id_group = cffpf.id_group)
			INNER JOIN {db_prefix}messages AS m ON (m.id_msg = cffpf.id_msg)
		WHERE cffpf.id_group IN ({array_int:id_group_array})
			AND cffpf.id_member != {int:guest}
			AND cffpsg.active = {int:active}
			AND icon != {string:recycled}
			AND approved = {int:is_approved}',
		array(
			'id_group_array' => $id_group_array,
			'guest' => 0,
			'active' => 1,
			'recycled' => 'recycled',
			'is_approved' => 1,
		)
	);
	while ($row = $smcFunc['db_fetch_assoc']($request))
	{
		if (function_exists('apache_reset_timeout'))
			@apache_reset_timeout();

		cffpUpdateCounterRecord($row['id_member'], $row['id_group'], 0, $use_context);
	}
	$smcFunc['db_free_result']($request);
}

// Updates all counters record of a member.
function cffpUpdateCounterRecordOfMember($id_member)
{
	global $context;

	if (empty($id_member) || !isset($context['cffp']['settings_groups']))
		return false;

	foreach ($context['cffp']['settings_groups'] as $id_group => $settings_group)
		cffpUpdateCounterRecord($id_member, $id_group);
}

// Updates counter record of member and group.
function cffpUpdateCounterRecord($id_member, $id_group, $inc = 0, $use_context = true)
{
	global $smcFunc, $context;

	if (empty($id_member) || empty($id_group))
		return false;

	if ($use_context)
	{
		if (!isset($context['cffp']['settings_groups'][$id_group]) || 'record' != $context['cffp']['settings_groups'][$id_group]['type'])
			return false;
	}
	else
	{
		$request = $smcFunc['db_query']('', '
			SELECT id_group
			FROM {db_prefix}cffp_settings_groups
			WHERE id_group = {int:id_group}
				AND type = {string:type}
				AND active = {int:active}',
			array(
				'id_group' => $id_group,
				'type' => 'record',
				'active' => 1,
			)
		);
		$num_rows = $smcFunc['db_num_rows']($request);
		$smcFunc['db_free_result']($request);

		if (0 == $num_rows)
			return false;
	}

	$columns = array(
		'groups' => 'cffp' . $id_group . '_groups',
		'id_membergroup' => 'cffp' . $id_group . '_id_membergroup',
	);
	if (!in_array($columns['groups'], $context['cffp']['list_columns']['members']) || !in_array($columns['id_membergroup'], $context['cffp']['list_columns']['members']))
		cffpAddCounterRecordColumns($id_group);

	if (empty($inc))
	{
		$request = $smcFunc['db_query']('', '
			SELECT COUNT(*)
			FROM {db_prefix}cffp_fields AS cffpf
				INNER JOIN {db_prefix}messages AS m ON (m.id_msg = cffpf.id_msg)
			WHERE cffpf.id_member = {int:id_member}
				AND id_group = {int:id_group}
				AND icon != {string:recycled}
				AND approved = {int:is_approved}',
			array(
				'id_member' => $id_member,
				'id_group' => $id_group,
				'recycled' => 'recycled',
				'is_approved' => 1,
			)
		);
		list ($groups) = $smcFunc['db_fetch_row']($request);
		$smcFunc['db_free_result']($request);

		$smcFunc['db_query']('', '
			UPDATE {db_prefix}members
			SET {raw:groups_name} = {int:groups_value}
			WHERE id_member = {int:id_member}',
			array(
				'id_member' => $id_member,
				'groups_name' => $columns['groups'],
				'groups_value' => $groups,
			)
		);
	}
	else
	{
		$smcFunc['db_query']('', '
			UPDATE {db_prefix}members
			SET {raw:groups_name} = {raw:groups_name} + {int:inc}
			WHERE id_member = {int:id_member}',
			array(
				'id_member' => $id_member,
				'groups_name' => $columns['groups'],
				'inc' => $inc,
			)
		);
	}

	cffpUpdateMembergroupRecord($id_group, $id_member, $use_context);
}

// Updates membergroup of record in members.
function cffpUpdateMembergroupRecord($id_group, $members = array(), $use_context = true, $reload_groups = false)
{
	static $min_groups = array();
	global $smcFunc, $context;

	if (empty($id_group))
		return false;

	if ($use_context)
	{
		if (!isset($context['cffp']['settings_groups'][$id_group]) || 'record' != $context['cffp']['settings_groups'][$id_group]['type'])
			return false;
	}
	else
	{
		$request = $smcFunc['db_query']('', '
			SELECT id_group
			FROM {db_prefix}cffp_settings_groups
			WHERE id_group = {int:id_group}
				AND type = {string:type}',
			array(
				'id_group' => $id_group,
				'type' => 'record',
			)
		);
		$num_rows = $smcFunc['db_num_rows']($request);
		$smcFunc['db_free_result']($request);

		if (0 == $num_rows)
			return false;
	}

	$columns = array(
		'groups' => 'cffp' . $id_group . '_groups',
		'id_membergroup' => 'cffp' . $id_group . '_id_membergroup',
	);

	if (empty($min_groups[$id_group]) || $reload_groups)
	{
		if (!in_array($columns['groups'], $context['cffp']['list_columns']['members']) || !in_array($columns['id_membergroup'], $context['cffp']['list_columns']['members']))
			cffpAddCounterRecordColumns($id_group);

		$request = $smcFunc['db_query']('', '
			SELECT id_membergroup, min_groups
			FROM {db_prefix}cffp_membergroups
			WHERE id_group = {int:id_group}
			ORDER BY min_groups DESC',
			array(
				'id_group' => $id_group,
			)
		);
		$min_groups[$id_group] = array();
		while ($row = $smcFunc['db_fetch_assoc']($request))
			$min_groups[$id_group][$row['id_membergroup']] = $row['min_groups'];
		$smcFunc['db_free_result']($request);
	}

	// Set all membergroups to 0.
	if (empty($min_groups[$id_group]))
	{
		$smcFunc['db_query']('', '
			UPDATE {db_prefix}members
			SET {raw:id_membergroup_name} = {int:id_membergroup_value}',
			array(
				'id_membergroup_name' => $columns['id_membergroup'],
				'id_membergroup_value' => 0,
			)
		);
	}
	// Set all membergroups from most groups to least groups.
	else
	{
		$cond_case = '';
		$query_parameters = array(
			'members' => $members,
			'groups_name' => $columns['groups'],
			'id_membergroup_name' => $columns['id_membergroup'],
		);
		$last_min = 0;
		foreach ($min_groups[$id_group] as $id_membergroup => $min)
		{
			$groups_value_min = $id_membergroup . 'groups_value_min';
			$groups_value_max = $id_membergroup . 'groups_value_max';
			$id_membergroup_value = $id_membergroup . 'id_membergroup_value';
			$cond_case .= '
					WHEN {raw:groups_name} >= {int:' . $groups_value_min . '}' . (!empty($last_min) ? ' AND {raw:groups_name} <= {int:' . $groups_value_max . '}' : '') . ' THEN {int:' . $id_membergroup_value . '}';
			$query_parameters[$groups_value_min] = $min;
			$query_parameters[$groups_value_max] = $last_min;
			$query_parameters[$id_membergroup_value] = $id_membergroup;
			$last_min = $min;
		}

		// A big fat CASE WHEN... END is faster than a zillion UPDATE's ;).
		$smcFunc['db_query']('', '
			UPDATE {db_prefix}members
			SET {raw:id_membergroup_name} = CASE' . $cond_case . '
					ELSE 0
				END' . (!empty($members) ? '
			WHERE id_member ' . (is_array($members) ? 'IN ({array_int:members})' : '= {int:members}') : ''),
			$query_parameters
		);
	}
}

// List on Moderation Center.
function cffpModerationList()
{
	global $txt, $scripturl, $modSettings, $sourcedir, $context, $user_info;

	loadTemplate('CustomFieldFilterPost');
	require_once ($sourcedir . '/Subs-List.php');

	// This comes under the umbrella of moderating posts.
	if ($user_info['mod_cache']['bq'] == '0=1')
		isAllowedTo('moderate_forum');

	// Set up the comforting bits...
	$context['page_title'] = $txt['cffp_moderation_area'];
	$context['sub_template'] = 'cffp_moderation_list';
	$context['id_group'] = isset($_GET['sa']) ? (int) $_GET['sa'] : 0;
	$context['id_cffp'] = isset($_GET['id_cffp']) ? (int) $_GET['id_cffp'] : 0;

	// Put the open and closed options into tabs, because we can...
	$context[$context['moderation_menu_name']]['tab_data'] = array(
		'title' => $context['page_title'],
		'help' => '',
		'description' => $txt['cffp_moderation_area_desc'],
	);

	if (!empty($context['id_group']) && isset($_GET['delete']) && !empty($context['id_cffp']))
		cffpDeleteFields($context['id_cffp']);

	if (empty($context['id_group']) && !empty($context['cffp']['moderation_list']))
		$context['id_group'] = key($context['cffp']['moderation_list']);

	global $cffp_moderation_settings_fields;

	$group = isset($context['cffp']['settings_groups'][$context['id_group']]) && $context['cffp']['settings_groups'][$context['id_group']]['moderation_list_show'] ? $context['cffp']['settings_groups'][$context['id_group']] : array();
	$cffp_moderation_settings_fields = !empty($group) ? $group['settings_fields'] : '';

	if (!empty($group))
	{
		$title = cffpIsNotEmptyString($group['moderation_list_name']) ? $group['moderation_list_name'] : $group['name'];

		$show_moderation_list = array();
		$column_list = array();
		foreach ($group['settings_fields'] as $id_field => $settings_field)
		{
			if ('no' == $settings_field['show_moderation_list_type'])
				continue;

			$settings_field2 = array(
				'id_field' => $settings_field['id_field'],
				'id_column' => $settings_field['id_column'],
				'show_moderation_list_type' => $settings_field['show_moderation_list_type'],
				'show_moderation_list_width' => $settings_field['show_moderation_list_width'],
				'show_moderation_list_name' => $settings_field['show_moderation_list_name'],
			);
			if ('multi' == $settings_field['show_moderation_list_type'])
			{
				$column = (!empty($settings_field['show_moderation_list_width']) ? $settings_field['show_moderation_list_width'] : 'auto') . ',' . $settings_field['show_moderation_list_name'];
				if (!isset($column_list[$column]))
				{
					$column_list[$column] = 'column_' . $settings_field['id_column'];
					$show_moderation_list[$column_list[$column]] = array($settings_field2, array());
				}
				$show_moderation_list[$column_list[$column]][1][] = $settings_field['id_field'];
			}
			else // one.
				$show_moderation_list[$settings_field['id_field']] = $settings_field2;
		}
	}
	else
		$title = $txt['cffp_moderation_area'];
	$text_align = array(
		'text' => 'left',
		'text_email' => 'center',
		'text_number' => 'right',
		'textarea' => 'left',
		'select' => 'center',
		'select_multiple' => 'left',
		'combobox' => 'center',
		'combobox_multiple' => 'left',
		'radio' => 'center',
		'check' => 'left',
		'boolean' => 'center',
	);

	$columns = array();
	$columns['sticky_locked'] = array(
		'header' => array(
			'style' => 'width: 4%;',
		),
		'data' => array(
			'function' => create_function('$rowData', '
				global $settings, $txt;

				$text = \'\';
				if ($rowData[\'is_sticky\'] && $rowData[\'locked\'])
				{
					$images_url = file_exists($settings[\'theme_dir\'] . \'/images/icons/quick_sticky_lock.gif\') ? \'images_url\' : \'default_images_url\';
					$text = \'<img src="\' . $settings[$images_url] . \'/icons/quick_sticky_lock.gif" alt="\' . $txt[\'cffp_show_filters_sticky_locked\'] . \'" />\';
				}
				elseif ($rowData[\'is_sticky\'])
				{
					$images_url = file_exists($settings[\'theme_dir\'] . \'/images/icons/quick_sticky.gif\') ? \'images_url\' : \'default_images_url\';
					$text = \'<img src="\' . $settings[$images_url] . \'/icons/quick_sticky.gif" alt="\' . $txt[\'cffp_show_filters_sticky\'] . \'" />\';
				}
				elseif ($rowData[\'locked\'])
				{
					$images_url = file_exists($settings[\'theme_dir\'] . \'/images/icons/quick_lock.gif\') ? \'images_url\' : \'default_images_url\';
					$text = \'<img src="\' . $settings[$images_url] . \'/icons/quick_lock.gif" alt="\' . $txt[\'cffp_show_filters_locked\'] . \'" />\';
				}

				return $text;
			'),
			'style' => 'text-align: center;',
		),
	);
	$columns['icon'] = array(
		'header' => array(
			'style' => 'width: 4%;',
		),
		'data' => array(
			'function' => create_function('$rowData', '
				global $modSettings, $settings, $context;

				// We need to check the topic icons exist...
				if (empty($modSettings[\'messageIconChecks_disable\']))
				{
					if (!isset($context[\'icon_sources\'][$rowData[\'icon\']]))
						$context[\'icon_sources\'][$rowData[\'icon\']] = file_exists($settings[\'theme_dir\'] . \'/images/post/\' . $rowData[\'icon\'] . \'.gif\') ? \'images_url\' : \'default_images_url\';
				}
				else
				{
					if (!isset($context[\'icon_sources\'][$rowData[\'icon\']]))
						$context[\'icon_sources\'][$rowData[\'icon\']] = \'images_url\';
				}

				return \'<img src="\' . $settings[$context[\'icon_sources\'][$rowData[\'icon\']]] . \'/post/\' . $rowData[\'icon\'] . \'.gif" alt="" />\';
			'),
			'style' => 'text-align: center;',
		),
	);
	$columns['m.subject'] = array(
		'header' => array(
			'value' => $txt['message'],
		),
		'data' => array(
			'function' => create_function('$rowData', '
				global $scripturl, $context, $settings, $txt;

				$href = $scripturl . \'?topic=\' . $rowData[\'id_topic\'] . \'.msg\' . $rowData[\'id_msg\'];
				$text = \'\';

				if ($rowData[\'new\'] && $context[\'user\'][\'is_logged\'])
					$text .= \'
				<a href="\' . $href . \'#new" id="newicon\' . $rowData[\'id_msg\'] . \'"><img src="\' . $settings[\'lang_images_url\'] . \'/new.gif" alt="\' . $txt[\'new\'] . \'" /></a>\';

				return \'<a href="\' . $href . \'#msg\' . $rowData[\'id_msg\'] . \'">\' . $rowData[\'subject\'] . \'</a>\' . $text;
			'),
		),
		'sort' => array(
			'default' => 'm.subject',
			'reverse' => 'm.subject DESC',
		),
	);
	if (!empty($group))
	{
		foreach ($show_moderation_list as $column => $show_moderation_list2)
		{
			if (is_numeric($column))
			{
				$id_field = $column;
				$column_data = $show_moderation_list2;

				if (!isset($cffp_moderation_settings_fields[$id_field]))
					$cffp_moderation_settings_fields[$id_field] = $context['cffp']['settings_groups'][$context['id_group']]['settings_fields'][$id_field];
				$name_field = 'column_' . $cffp_moderation_settings_fields[$id_field]['id_column'];

				$type_align = $cffp_moderation_settings_fields[$id_field]['type'];
				switch ($cffp_moderation_settings_fields[$id_field]['type'])
				{
					case 'text':
					{
						if (in_array($cffp_moderation_settings_fields[$id_field]['mask'], array('email', 'number')))
							$type_align .= '_' . $cffp_moderation_settings_fields[$id_field]['mask'];
						break;
					}
					case 'select':
					case 'combobox':
					{
						$options = explode(',', $cffp_moderation_settings_fields[$id_field]['options']);
						$select_multiple = array_shift($options);
						if ($select_multiple)
							$type_align .= '_multiple';
						break;
					}
				}

				$columns['cffpf.' . $name_field] = array(
					'header' => array(
						'value' => $cffp_moderation_settings_fields[$id_field]['name'],
						'style' => !empty($column_data['show_moderation_list_width']) ? ('width: ' . $column_data['show_moderation_list_width'] . ';') : '',
					),
					'data' => array(
						'function' => create_function('$rowData', '
							global $cffp_moderation_settings_fields;

							return cffpHtmlOfField($cffp_moderation_settings_fields[' . $id_field . '], $rowData[\'' . $name_field . '\'], true, $rowData[\'id_msg\']);
						'),
						'style' => 'text-align: ' . (!empty($text_align[$type_align]) ? $text_align[$type_align] : 'center') . ';',
					),
					'sort' => array(
						'default' => 'cffpf.' . $name_field,
						'reverse' => 'cffpf.' . $name_field . ' DESC',
					),
				);
			}
			else
			{
				$name_field = $column;
				$column_data = $show_moderation_list2[0];
				$text = '';
				foreach ($show_moderation_list2[1] as $id_field)
				{
					if (!isset($cffp_moderation_settings_fields[$id_field]))
						$cffp_moderation_settings_fields[$id_field] = $context['cffp']['settings_groups'][$context['id_group']]['settings_fields'][$id_field];
					$name_field2 = 'column_' . $cffp_moderation_settings_fields[$id_field]['id_column'];

					$text .= '
							$text2 = cffpHtmlOfField($cffp_moderation_settings_fields[' . $id_field . '], $rowData[\'' . $name_field2 . '\'], true, $rowData[\'id_msg\']);
							if (cffpIsNotEmptyString($text2))
							{
								if (cffpIsNotEmptyString($text))
									$text .= \'<br />\';
								$text .= $text2;
							}';
				}

				$columns['cffpf.' . $name_field] = array(
					'header' => array(
						'value' => $column_data['show_moderation_list_name'],
						'style' => !empty($column_data['show_moderation_list_width']) ? ('width: ' . $column_data['show_moderation_list_width'] . ';') : '',
					),
					'data' => array(
						'function' => create_function('$rowData', '
							global $cffp_moderation_settings_fields;

							$text = \'\';' . $text . '
							return $text;
						'),
						'style' => 'text-align: left;',
					),
					'sort' => array(
						'default' => 'cffpf.' . $name_field,
						'reverse' => 'cffpf.' . $name_field . ' DESC',
					),
				);
			}
		}
	}
	$columns['cffpf.poster_time'] = array(
		'header' => array(
			'value' => $txt['cffp_poster_time'],
			'style' => 'width: 10%;',
		),
		'data' => array(
			'db' => 'poster_time',
			'timeformat' => true,
			'style' => 'text-align: center;',
		),
		'sort' => array(
			'default' => 'cffpf.poster_time',
			'reverse' => 'cffpf.poster_time DESC',
		),
	);
	$columns['display_name'] = array(
		'header' => array(
			'value' => $txt['posted_by'],
			'style' => 'width: 10%;',
		),
		'data' => array(
			'function' => create_function('$rowData', '
				global $scripturl, $txt;

				return !empty($rowData[\'id_member\']) ? \'<a href="\' . $scripturl . \'?action=profile;u=\' . $rowData[\'id_member\'] . \'" title="\' . $txt[\'profile_of\'] . \' \' . $rowData[\'display_name\'] . \'">\' . $rowData[\'display_name\'] . \'</a>\' : $rowData[\'display_name\'];
			'),
		),
		'sort' => array(
			'default' => 'display_name',
			'reverse' => 'display_name DESC',
		),
	);

	$moderation_list_order_by = !empty($group['moderation_list_order_by']) ? $group['moderation_list_order_by'] : 'cffpf.poster_time desc';
	list ($default_sort_col, $default_sort_dir) = explode(' ', $moderation_list_order_by);
	if (!isset($columns[$default_sort_col]))
		$default_sort_col = 'cffpf.poster_time';

	$start = isset($_REQUEST['start']) ? (int) $_REQUEST['start'] : 0;
	if (isset($_REQUEST['sort']))
	{
		$sort = $_REQUEST['sort'];
		$desc = isset($_REQUEST['desc']);
	}
	else
	{
		$sort = $default_sort_col;
		$desc = 'desc' == $default_sort_dir;
	}
	$params_url = ';start=' . $start . ';sort=' . $sort . ($desc ? ';desc' : '');

	$columns['option_delete'] = array(
		'header' => array(
			'style' => 'width: 10%;',
		),
		'data' => array(
			'sprintf' => array(
				'format' => '<a href="' . $scripturl . '?action=moderate;area=cffp;sa=' . $context['id_group'] . ';delete=1;id_cffp=%1$s' . $params_url . ';' . $context['session_var'] . '=' . $context['session_id'] . '" onclick="return confirm(\'' . addslashes($txt['cffp_moderation_area_delete_sure']) . '\');">' . $txt['delete'] . '</a>',
				'params' => array(
					'id_cffp' => false,
				),
			),
			'style' => 'text-align: center;',
		),
	);

	$listOptions = array(
		'id' => 'cffp_moderation_list',
		'title' => $title,
		'base_href' => $scripturl . '?action=moderate;area=cffp;sa=' . $context['id_group'],
		'default_sort_col' => $default_sort_col,
		'default_sort_dir' => $default_sort_dir,
		'no_items_label' => $txt['cffp_moderation_area_none'],
		'items_per_page' => $modSettings['defaultMaxMessages'],
		'get_items' => array(
			'function' => 'list_cffpGetModeration',
		),
		'get_count' => array(
			'function' => 'list_cffpGetModerationCount',
		),
		'columns' => $columns,
	);
	createList($listOptions);
}

// Gets list of search posts.
function list_cffpGetSearchPosts($start, $items_per_page, $sort)
{
	global $smcFunc, $context;

	if (!is_numeric($start) || empty($items_per_page) || empty($sort))
		return false;

	$params = cffpGetSqlSearch('search_posts');

	$query_parameters = array(
		'sort' => $sort,
		'start' => $start,
		'items_per_page' => $items_per_page,
	);
	if (!empty($params['query_parameters']))
		$query_parameters = array_merge($query_parameters, $params['query_parameters']);

	$request = $smcFunc['db_query']('', '
		SELECT DISTINCT
			t.is_sticky, t.locked, t.id_first_msg,
			m.id_msg, m.id_topic, m.id_board, m.poster_time, m.subject, m.icon,
			m.poster_name, m.id_member,
			IFNULL(mem.real_name, m.poster_name) AS display_name,
			(IFNULL(lt.id_msg, IFNULL(lmr.id_msg, 0)) < m.id_msg) AS new
		FROM ' . $params['from'] . (!empty($params['cond']) ? '
		WHERE ' . $params['cond'] : '') . '
		ORDER BY {raw:sort}
		LIMIT {int:start}, {int:items_per_page}',
		$query_parameters
	);
	$list = array();
	$id_msg_array = array();
	while ($row = $smcFunc['db_fetch_assoc']($request))
	{
		$list[] = $row;

		$id_msg_array[] = $row['id_msg'];
		$context['cffp']['data_msg_topic'][$row['id_msg']] = array(
			'sticky' => $row['is_sticky'] ? 1 : 0,
			'locked' => $row['locked'] ? 1 : 0,
			'id_msg' => $row['id_msg'],
			'first_msg' => $row['id_msg'] == $row['id_first_msg'] ? 1 : 0,
			'subject' => $row['subject'],
		);
	}
	$smcFunc['db_free_result']($request);

	cffpLoadValueOfFieldsByMsg($id_msg_array);

	return $list;
}

// Gets size of list of search posts.
function list_cffpGetSearchPostsCount()
{
	global $smcFunc;

	$params = cffpGetSqlSearch('search_posts');

	$request = $smcFunc['db_query']('', '
		SELECT COUNT(DISTINCT m.id_msg)
		FROM ' . $params['from'] . (!empty($params['cond']) ? '
		WHERE ' . $params['cond'] : ''),
		$params['query_parameters']
	);

	list ($size) = $smcFunc['db_fetch_row']($request);
	$smcFunc['db_free_result']($request);

	return $size;
}

// Gets list of search records.
function list_cffpGetSearchRecords($start, $items_per_page, $sort)
{
	global $smcFunc, $context;

	if (!is_numeric($start) || empty($items_per_page) || empty($sort))
		return false;

	$params = cffpGetSqlSearch('search_records');

	$query_parameters = array(
		'sort' => $sort,
		'start' => $start,
		'items_per_page' => $items_per_page,
	);
	if (!empty($params['query_parameters']))
		$query_parameters = array_merge($query_parameters, $params['query_parameters']);

	$attr = '';
	foreach ($context['cffp']['settings_groups'][$context['id_group']]['settings_fields'] as $settings_fields)
	{
		if ('no' != $settings_fields['show_record_list_type'])
			$attr .= ', cffpf0.column_' . $settings_fields['id_column'];
	}

	$request = $smcFunc['db_query']('', '
		SELECT DISTINCT
			cffpf0.id_msg, cffpf0.id_member, cffpf0.poster_time,
			m.icon, m.id_topic,
			IFNULL(mem.real_name, cffpf0.poster_name) AS display_name' . $attr . '
		FROM ' . $params['from'] . '
		WHERE ' . $params['cond'] . '
		ORDER BY {raw:sort}
		LIMIT {int:start}, {int:items_per_page}',
		$query_parameters
	);
	$list = array();
	while ($row = $smcFunc['db_fetch_assoc']($request))
		$list[] = $row;
	$smcFunc['db_free_result']($request);

	return $list;
}

// Gets size of list of search records.
function list_cffpGetSearchRecordsCount()
{
	global $smcFunc;

	$params = cffpGetSqlSearch('search_records');

	$request = $smcFunc['db_query']('', '
		SELECT COUNT(DISTINCT cffpf0.id_cffp)
		FROM ' . $params['from'] . '
		WHERE ' . $params['cond'],
		$params['query_parameters']
	);

	list ($size) = $smcFunc['db_fetch_row']($request);
	$smcFunc['db_free_result']($request);

	return $size;
}

// Gets list of Moderation Center.
function list_cffpGetModeration($start, $items_per_page, $sort)
{
	global $smcFunc, $context, $user_info;

	if (!is_numeric($start) || empty($items_per_page) || empty($sort))
		return false;

	$request = $smcFunc['db_query']('', '
		SELECT DISTINCT
			t.is_sticky, t.locked, t.id_first_msg,
			cffpf.' . implode(', cffpf.', $context['cffp']['list_columns']['fields']) . ',
			m.id_topic, m.subject, m.icon,
			IFNULL(mem.real_name, cffpf.poster_name) AS display_name,
			(IFNULL(lt.id_msg, IFNULL(lmr.id_msg, 0)) < m.id_msg) AS new
		FROM {db_prefix}messages AS m
			INNER JOIN {db_prefix}topics AS t ON (t.id_topic = m.id_topic)
			INNER JOIN {db_prefix}cffp_fields AS cffpf ON (cffpf.id_msg = m.id_msg)
			LEFT JOIN {db_prefix}members AS mem ON (mem.id_member = cffpf.id_member)
			LEFT JOIN {db_prefix}log_topics AS lt ON (lt.id_topic = t.id_topic AND lt.id_member = {int:id_member})
			LEFT JOIN {db_prefix}log_mark_read AS lmr ON (lmr.id_board = t.id_board AND lmr.id_member = {int:id_member})
		WHERE cffpf.id_group = {int:id_group}
		ORDER BY {raw:sort}
		LIMIT {int:start}, {int:items_per_page}',
		array(
			'id_member' => $user_info['id'],
			'id_group' => $context['id_group'],
			'sort' => $sort,
			'start' => $start,
			'items_per_page' => $items_per_page,
		)
	);
	$list = array();
	while ($row = $smcFunc['db_fetch_assoc']($request))
		$list[] = $row;
	$smcFunc['db_free_result']($request);

	return $list;
}

// Gets size of list of Moderation Center.
function list_cffpGetModerationCount()
{
	global $smcFunc, $context;

	$request = $smcFunc['db_query']('', '
		SELECT COUNT(DISTINCT cffpf.id_cffp)
		FROM {db_prefix}messages AS m
			INNER JOIN {db_prefix}cffp_fields AS cffpf ON (cffpf.id_msg = m.id_msg)
		WHERE cffpf.id_group = {int:id_group}',
		array(
			'id_group' => $context['id_group'],
		)
	);

	list ($size) = $smcFunc['db_fetch_row']($request);
	$smcFunc['db_free_result']($request);

	return $size;
}

?>